<?php
/**
*
* @package Breizh Shoutbox Extension
* 
* @copyright (c) 2018 Sylver35  https://breizhcode.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

namespace sylver35\breizhshoutbox\controller;

class admin_controller
{
	/** @var \phpbb\cache\service */
	protected $cache;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/* @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var \phpbb\pagination */
	protected $pagination;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\log\log */
	protected $log;
	
	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;
	
	/** @var \phpbb\filesystem\filesystem_interface */
	protected $filesystem;
	
	/** @var \phpbb\path_helper */
	protected $path_helper;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;
	
	/** @var string Custom form action */
	protected $u_action;
	
	/* @var \sylver35\breizhshoutbox\core\breizhshoutbox */
	protected $shoutbox;
	
	/**
	* The database tables
	*
	* @var string */
	protected $shoutbox_table;
	protected $shoutbox_priv_table;
	protected $shoutbox_rules_table;

	/**
	* Constructor
	*
	* @param \sylver35\breizhshoutbox\core\shoutbox	$shoutbox
	* @param \phpbb\config\config					$config
	* @param \phpbb\controller\helper				$helper
	* @param \phpbb\db\driver\driver_interface		$db
	* @param \phpbb\template\template				$template
	* @param \phpbb\user							$user
	* @param \phpbb\extension\manager				$ext_manager
	* @param string                     			$phpbb_root_path
	* @param string                     			$php_ext
	*/
	public function __construct(
		\phpbb\cache\service $cache,
		\phpbb\config\config $config, 
		\phpbb\controller\helper $helper, 
		\phpbb\db\driver\driver_interface $db, 
		\phpbb\pagination $pagination,
		\phpbb\request\request $request, 
		\phpbb\template\template $template, 
		\phpbb\auth\auth $auth, 
		\phpbb\user $user,
		\phpbb\log\log $log,
		\phpbb\extension\manager $ext_manager,
		\phpbb\filesystem\filesystem_interface $filesystem,
		\phpbb\path_helper $path_helper,
		$phpbb_root_path, 
		$php_ext,
		$shoutbox_table,
		$shoutbox_priv_table,
		$shoutbox_rules_table,
		\sylver35\breizhshoutbox\core\shoutbox $shoutbox)
	{
		$this->cache 				= $cache;
		$this->config 				= $config;
		$this->helper 				= $helper;
		$this->db 					= $db;
		$this->pagination 			= $pagination;
		$this->request 				= $request;
		$this->template 			= $template;
		$this->auth 				= $auth;
		$this->user 				= $user;
		$this->log					= $log;
		$this->ext_manager	 		= $ext_manager;
		$this->filesystem 			= $filesystem;
		$this->path_helper	 		= $path_helper;
		$this->root_path 			= $phpbb_root_path;
		$this->php_ext 				= $php_ext;
		$this->shoutbox_table 		= $shoutbox_table;
		$this->shoutbox_priv_table 	= $shoutbox_priv_table;
		$this->shoutbox_rules_table = $shoutbox_rules_table;
		$this->shoutbox 			= $shoutbox;
		
		$this->ext_path = $this->ext_manager->get_extension_path('sylver35/breizhshoutbox', true);
		$this->ext_path_web = $this->path_helper->update_web_root_path($this->ext_path);
	}
	
	public function acp_shoutbox_configs()
	{
		$mode = $this->request->variable('mode', '');
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			
			$settings = array(
				'shout_enable'				=> $this->request->variable('shout_enable', 1),
				'shout_temp_users'			=> $this->request->variable('shout_temp_users', 5),
				'shout_temp_anonymous'		=> $this->request->variable('shout_temp_anonymous', 10),
				'shout_inactiv_anony'		=> $this->request->variable('shout_inactiv_anony', 15),
				'shout_inactiv_member'		=> $this->request->variable('shout_inactiv_member', 30),
				'shout_defil'				=> $this->request->variable('shout_defil', 1),
				'shout_dateformat'			=> $this->request->variable('shout_dateformat2', '', true),
				'shout_bbcode'				=> $this->request->variable('shout_bbcode', ''),
				'shout_bbcode_user'			=> $this->request->variable('shout_bbcode_user', ''),
				'shout_bbcode_size'			=> $this->request->variable('shout_bbcode_size', ''),
				'shout_see_buttons'			=> $this->request->variable('shout_see_buttons', 1),
				'shout_see_buttons_left'	=> $this->request->variable('shout_see_buttons_left', 1),
				'shout_avatar'				=> $this->request->variable('shout_avatar', 1),
				'shout_avatar_height'		=> $this->request->variable('shout_avatar_height', 20),
				'shout_avatar_robot'		=> $this->request->variable('shout_avatar_robot', 1),
				'shout_avatar_user'			=> $this->request->variable('shout_avatar_user', 1),
				'shout_avatar_img'			=> $this->request->variable('shout_avatar_img', 'no_avatar.gif'),
				'shout_avatar_img_robot'	=> $this->request->variable('shout_avatar_img_robot', 'avatar_robot.png'),
				'shout_sound_on'			=> $this->request->variable('shout_sound_on', 1),
				'shout_sound_new'			=> $this->request->variable('shout_sound_new', ''),
				'shout_sound_error'			=> $this->request->variable('shout_sound_error', ''),
				'shout_sound_del'			=> $this->request->variable('shout_sound_del', ''),
				'shout_sound_add'			=> $this->request->variable('shout_sound_add', ''),
				'shout_sound_edit'			=> $this->request->variable('shout_sound_edit', ''),
				'shout_rules'				=> $this->request->variable('shout_rules', 1),
				'shout_nr_acp'				=> $this->request->variable('shout_nr_acp', 20),
				'shout_max_post_chars'		=> $this->request->variable('shout_max_post_chars', 300),
				'shout_smilies_width'		=> $this->request->variable('shout_smilies_width', 350),
				'shout_smilies_height'		=> $this->request->variable('shout_smilies_height', 400),
				'shout_index'				=> $this->request->variable('shout_index', 1),
				'shout_position_index'		=> $this->request->variable('shout_position_index', 0),
				'shout_forum'				=> $this->request->variable('shout_forum', 1),
				'shout_position_forum'		=> $this->request->variable('shout_position_forum', 0),
				'shout_topic'				=> $this->request->variable('shout_topic', 1),
				'shout_position_topic'		=> $this->request->variable('shout_position_topic', 0),
			);
			foreach ($settings as $config_name => $config_value)
			{
				set_config($config_name, $config_value, false);
			}
			add_log('admin', 'LOG_SHOUT_' . strtoupper($mode));
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$this->template->assign_vars(array(
				'SHOUT_ENABLE'			=> $this->shoutbox->construct_radio_shout('shout_enable', 2),
				'SHOUT_TEMP_USERS'		=> isset($this->config['shout_temp_users']) ? (int)$this->config['shout_temp_users'] : '',
				'SHOUT_TEMP_ANONYMOUS'	=> isset($this->config['shout_temp_anonymous']) ? (int)$this->config['shout_temp_anonymous'] : '',
				'SHOUT_INACTIV_ANONY'	=> isset($this->config['shout_inactiv_anony']) ? (int)$this->config['shout_inactiv_anony'] : '',
				'SHOUT_INACTIV_MEMBER'	=> isset($this->config['shout_inactiv_member']) ? (int)$this->config['shout_inactiv_member'] : '',
				'SHOUT_DEFIL'			=> isset($this->config['shout_defil']) ? (bool)$this->config['shout_defil'] : '',
				'S_SHOUT_DATEFORMAT'	=> $this->shoutbox->shout_dateformat_select(isset($this->config['shout_dateformat']) ? (string)$this->config['shout_dateformat'] : ''),
				'SHOUT_BBCODE'			=> isset($this->config['shout_bbcode']) ? (string)$this->config['shout_bbcode'] : '',
				'SHOUT_BBCODE_USER'		=> isset($this->config['shout_bbcode_user']) ? (string)$this->config['shout_bbcode_user'] : '',
				'SHOUT_BBCODE_SIZE'		=> isset($this->config['shout_bbcode_size']) ? (int)$this->config['shout_bbcode_size'] : '',
				'SHOUT_SEE_BUTTONS'		=> $this->shoutbox->construct_radio_shout('shout_see_buttons', 1, false),
				'SHOUT_SEE_BUTTONS_LEFT'=> $this->shoutbox->construct_radio_shout('shout_see_buttons_left', 1, false),
				'SHOUT_AVATAR'			=> $this->shoutbox->construct_radio_shout('shout_avatar', 2),
				'SHOUT_AVATAR_HEIGHT'	=> isset($this->config['shout_avatar_height']) ? (int)$this->config['shout_avatar_height'] : '',
				'SHOUT_AVATAR_ROBOT'	=> $this->shoutbox->construct_radio_shout('shout_avatar_robot', 2),
				'SHOUT_AVATAR_USER'		=> $this->shoutbox->construct_radio_shout('shout_avatar_user', 2),
				'SHOUT_AVATAR_IMG'		=> isset($this->config['shout_avatar_img']) ? (string)$this->config['shout_avatar_img'] : '',
				'SHOUT_AVATAR_IMG_BOT'	=> isset($this->config['shout_avatar_img_robot']) ? (string)$this->config['shout_avatar_img_robot'] : '',
				'SHOUT_AVATAR_IMG_SRC'	=> isset($this->config['shout_avatar_img']) ? $this->ext_path. '/images/' .$this->config['shout_avatar_img'] : '',
				'SHOUT_AVATAR_IMG_BOT_SRC'=> isset($this->config['shout_avatar_img_robot']) ? $this->ext_path. '/images/' .$this->config['shout_avatar_img_robot'] : '',
				'SHOUT_SOUND_ON'		=> $this->shoutbox->construct_radio_shout('shout_sound_on', 2, false),
				'SHOUT_NR_ACP'			=> isset($this->config['shout_nr_acp']) ? (int)$this->config['shout_nr_acp'] : '',
				'SHOUT_MAX_POST_CHARS'	=> isset($this->config['shout_max_post_chars']) ? (int)$this->config['shout_max_post_chars'] : '',
				'SHOUT_SMILIES_W'		=> isset($this->config['shout_smilies_width']) ? (int)$this->config['shout_smilies_width'] : '',
				'SHOUT_SMILIES_H'		=> isset($this->config['shout_smilies_height']) ? (int)$this->config['shout_smilies_height'] : '',
				'SHOUT_RULES'			=> $this->shoutbox->construct_radio_shout('shout_rules', 2),
				'SHOUT_INDEX_ON'		=> $this->shoutbox->construct_radio_shout('shout_index', 2),
				'INDEX_SHOUT_TOP'		=> isset($this->config['shout_position_index']) ? (((int)$this->config['shout_position_index'] == 0) ? true : false) : '',
				'INDEX_SHOUT_AFTER'		=> isset($this->config['shout_position_index']) ? (((int)$this->config['shout_position_index'] == 1) ? true : false) : '',
				'INDEX_SHOUT_END'		=> isset($this->config['shout_position_index']) ? (((int)$this->config['shout_position_index'] == 2) ? true : false) : '',
				'SHOUT_FORUM_ON'		=> $this->shoutbox->construct_radio_shout('shout_forum', 2),
				'POS_SHOUT_FORUM_TOP'	=> isset($this->config['shout_position_forum']) ? (((int)$this->config['shout_position_forum'] == 0) ? true : false) : '',
				'POS_SHOUT_FORUM_END'	=> isset($this->config['shout_position_forum']) ? (((int)$this->config['shout_position_forum'] == 1) ? true : false) : '',
				'SHOUT_TOPIC_ON'		=> $this->shoutbox->construct_radio_shout('shout_topic', 2),
				'POS_SHOUT_TOPIC_TOP'	=> isset($this->config['shout_position_topic']) ? (((int)$this->config['shout_position_topic'] == 0) ? true : false) : '',
				'POS_SHOUT_TOPIC_END'	=> isset($this->config['shout_position_topic']) ? (((int)$this->config['shout_position_topic'] == 1) ? true : false) : '',
				'NEW_SOUND'				=> $this->shoutbox->build_adm_sound_select('new'),
				'ERROR_SOUND'			=> $this->shoutbox->build_adm_sound_select('error'),
				'DEL_SOUND'				=> $this->shoutbox->build_adm_sound_select('del'),
				'ADD_SOUND'				=> $this->shoutbox->build_adm_sound_select('add'),
				'EDIT_SOUND'			=> $this->shoutbox->build_adm_sound_select('edit'),
				'SHOUT_SOUNDS_PATH'		=> $this->ext_path . 'images/sounds/',
				'SHOUT_VERSION'			=> $this->config['shout_version'],
			));
		}
		$this->template->assign_var('S_CONFIGS', true);
	}
	
	public function acp_shoutbox_rules()
	{
		include($this->root_path . 'includes/functions_posting.' . $this->php_ext);
		include($this->root_path . 'includes/functions_display.' . $this->php_ext);
		$this->user->add_lang(array('posting'));
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'l.lang_iso',
				'FROM'      => array(LANG_TABLE => 'l'),
				'ORDER BY'	=> 'lang_id',
			));
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$iso = $row['lang_iso'];
				$rules_uid = $rules_bitfield = $rules_flags = $rules_uid_priv = $rules_bitfield_priv = $rules_flags_priv = ''; // will be modified by generate_text_for_storage
				$rules_text	= utf8_normalize_nfc($this->request->variable("rules_text_$iso", '', true));
				$rules_text_priv = utf8_normalize_nfc($this->request->variable("rules_text_priv_$iso", '', true));
				generate_text_for_storage($rules_text, $rules_uid, $rules_bitfield, $rules_flags, true, true, true);
				generate_text_for_storage($rules_text_priv, $rules_uid_priv, $rules_bitfield_priv, $rules_flags_priv, true, true, true);
				
				$data = array(
					'rules_lang'			=> $iso,
					'rules_text'  			=> $rules_text,
					'rules_bitfield'  		=> $rules_bitfield,
					'rules_uid'  			=> $rules_uid,
					'rules_flags'  			=> $rules_flags,
					'rules_text_priv'  		=> $rules_text_priv,
					'rules_bitfield_priv' 	=> $rules_bitfield_priv,
					'rules_uid_priv'  		=> $rules_uid_priv,
					'rules_flags_priv'  	=> $rules_flags_priv,
				);
				
				$sql_ary = $this->db->sql_build_query('SELECT', array(
					'SELECT'    => 'rules_lang',
					'FROM'      => array($this->shoutbox_rules_table => ''),
					'WHERE'		=> "rules_lang = '" .$this->db->sql_escape($iso). "'",
				));
				$result_ary = $this->db->sql_query($sql_ary);
				$on_rules = $this->db->sql_fetchfield('rules_lang', $result_ary);
				$this->db->sql_freeresult($result_ary);
				
				if ($on_rules)
				{
					$sql = 'UPDATE ' . $this->shoutbox_rules_table . ' 
						SET ' . $this->db->sql_build_array('UPDATE', $data) . " 
							WHERE rules_lang = '" .$this->db->sql_escape($iso). "'";
					$this->db->sql_query($sql);
				}
				else
				{
					$sql = 'INSERT INTO ' . $this->shoutbox_rules_table . ' ' . $this->db->sql_build_array('INSERT', $data);
					$this->db->sql_query($sql);
				}
				
				if ($data['rules_text'] != '')
				{
					set_config("shout_rules_$iso", 1, false);
				}
				else
				{
					set_config("shout_rules_$iso", 0, false);
				}
				if ($data['rules_text_priv'] != '')
				{
					set_config("shout_rules_priv_$iso", 1, false);
				}
				else
				{
					set_config("shout_rules_priv_$iso", 0, false);
				}
			}
			$this->db->sql_freeresult($result);
			$this->cache->destroy('_shout_rules');
			add_log('admin', 'LOG_SHOUT_RULES');
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$i = 1;
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'l.lang_iso, l.lang_local_name, r.*',
				'FROM'      => array(LANG_TABLE => 'l'),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array($this->shoutbox_rules_table => 'r'),
						'ON'	=> 'r.rules_lang = l.lang_iso'
					)
				),
				'ORDER BY'	=> 'l.lang_id',
			));
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$text 			= isset($row['rules_text']) ? $row['rules_text'] : '';
				$uid 			= isset($row['rules_uid']) ? $row['rules_uid'] : '';
				$bitfield 		= isset($row['rules_bitfield']) ? $row['rules_bitfield'] : '';
				$flags 			= isset($row['rules_flags']) ? $row['rules_flags'] : '';
				$text_priv 		= isset($row['rules_text_priv']) ? $row['rules_text_priv'] : '';
				$uid_priv		= isset($row['rules_uid_priv']) ? $row['rules_uid_priv'] : '';
				$bitfield_priv 	= isset($row['rules_bitfield_priv']) ? $row['rules_bitfield_priv'] : '';
				$flags_priv 	= isset($row['rules_flags_priv']) ? $row['rules_flags_priv'] : '';
				$display 		= generate_text_for_display($text, $uid, $bitfield, $flags);
				$display_priv 	= generate_text_for_display($text_priv, $uid_priv, $bitfield_priv, $flags_priv);
				
				decode_message($text, $uid);
				decode_message($text_priv, $uid_priv);
				
				$this->template->assign_block_vars('rules', array(
					'RULES_NR'				=> $i,
					'RULES_TEXT'			=> $text,
					'RULES_TEXT_PRIV'		=> $text_priv,
					'RULES_TEXT_DISPLAY'	=> $display,
					'RULES_TEXT_DISPLAY_PRIV'=> $display_priv,
					'RULES_ON'				=> sprintf($this->user->lang['SHOUT_RULES_ON'], $row['lang_local_name']),
					'RULES_ON_EXPLAIN'		=> sprintf($this->user->lang['SHOUT_RULES_ON_EXPLAIN'], $row['lang_local_name']),
					'RULES_ON_PRIV_EXPLAIN'	=> sprintf($this->user->lang['SHOUT_RULES_ON_PRIV_EXPLAIN'], $row['lang_local_name']),
					'RULES_LANG'			=> $row['lang_local_name'],
					'RULES_ISO'				=> $row['lang_iso'],
					'COPY_TO'				=> sprintf($this->user->lang['SHOUT_COPY_RULE'], $row['lang_iso'], $this->user->lang['SHOUT_NORMAL']),
					'COPY_TO_PRIV'			=> sprintf($this->user->lang['SHOUT_COPY_RULE'], $row['lang_iso'], $this->user->lang['ACP_SHOUT_PRIVATE_CAT']),
				));
				$i++;
			}
			$this->db->sql_freeresult($result);
			
			generate_smilies('inline', '', 1);
			display_custom_bbcodes();

			$this->template->assign_vars(array(
				'U_SHOUT_SMILIES'	=> append_sid("{$this->root_path}adm/index.$this->php_ext", 'i=-sylver35-breizhshoutbox-acp-main_module&amp;mode=smilies_pop'),
			));
		}
		$this->template->assign_var('S_RULES', true);
	}
	
	public function acp_shoutbox_overview()
	{
		$id 			= $this->request->variable('i', '');
		$mode 			= $this->request->variable('mode', '');
		$action 		= $this->request->variable('action', '');
		$start			= $this->request->variable('start', 0);
		$marked			= $this->request->variable('mark', array(0));
		$creation 		= $this->request->variable('creation_time', 0);
		$token 			= $this->request->variable('form_token', '');
		$deletemark 	= $this->request->is_set_post('delmarked') ? true : false;
		$deletemarklog 	= $this->request->is_set_post('delmarkedlog') ? true : false;
		$form_key 		= 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		
		if ($deletemark)
		{
			if (confirm_box(true))
			{
				if (!check_form_key($form_key))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$where_sql = '';
				if ($deletemark && sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
					}
					$where_sql = ' WHERE ' . $this->db->sql_in_set('shout_id', $sql_in);
					unset($sql_in);
				}
				if ($where_sql)
				{
					$sql = 'DELETE FROM ' . $this->shoutbox_table .  $where_sql;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					
					$log_entrie = 'LOG_SELECT' .$this->shoutbox->plural($deleted). '_SHOUTBOX';
					add_log('admin', $log_entrie, $deleted);
					set_config_count('shout_del_acp', $deleted, true);
				}
			}
			else
			{
				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'start'			=> $start,
					'delmarked'		=> $deletemark,
					'mark'			=> $marked,
					'i'				=> $id,
					'mode'			=> $mode,
					'action'		=> $action,
					'creation_time' => $creation,
					'form_token'	=> $token,
				)));
			}
		}
		
		if ($deletemarklog)
		{
			if (confirm_box(true))
			{
				if (!check_form_key($form_key))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$where_sql = '';
				if ($deletemarklog && sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
					}
					$where_sql = ' WHERE ' . $this->db->sql_in_set('log_id', $sql_in);
					unset($sql_in);
				}
				if ($where_sql)
				{
					$sql = 'DELETE FROM ' . LOG_TABLE . $where_sql;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					
					$log_entrie = 'LOG_LOG' .$this->shoutbox->plural($deleted). '_SHOUTBOX';
					add_log('admin', $log_entrie, $deleted);
				}
			}
			else
			{
				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'start'			=> $start,
					'delmarkedlog'	=> $deletemarklog,
					'mark'			=> $marked,
					'i'				=> $id,
					'mode'			=> $mode,
					'action'		=> $action,
					'creation_time' => $creation,
					'form_token'	=> $token,
				)));
			}
		}
		if ($action)
		{
			if (!confirm_box(true))
			{
				switch ($action)
				{
					default:
						$confirm = true;
						$confirm_lang = 'CONFIRM_OPERATION';
				}

				if ($confirm)
				{
					confirm_box(false, $this->user->lang[$confirm_lang], build_hidden_fields(array(
						'i'				=> $id,
						'mode'			=> $mode,
						'action'		=> $action,
						'creation_time' => $creation,
						'form_token'	=> $token,
					)));
				}
			}
			else
			{
				if (!check_form_key($form_key))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				switch ($action)
				{
					case 'purge':
						$sql = 'DELETE FROM ' . $this->shoutbox_table;
						$this->db->sql_query($sql);
						$deleted = $this->db->sql_affectedrows();
						
						set_config_count('shout_del_purge', $deleted, true);
						add_log('admin', 'LOG_PURGE_SHOUTBOX');
						$this->shoutbox->post_robot_shout(0, $this->user->ip, false, true, false);
					break;
					case 'purge_1':
					case 'purge_2':
					case 'purge_3':
					case 'purge_4':
					case 'purge_5':
					case 'purge_6':
					case 'purge_7':
					case 'purge_8':
						$sort = str_replace('purge_', '', $action);
						$retour = $this->purge_shout_admin($sort);
						if ($retour)
						{
							adm_back_link($this->u_action);
						}
					break;
				}
			}
		}
		$start = $this->request->variable('start', 0);
		$sql_nr = 'SELECT COUNT(DISTINCT shout_id) as total
			FROM ' . $this->shoutbox_table . '
			WHERE shout_inp = 0 
				OR shout_inp = ' .$this->user->data['user_id']. ' 
				OR shout_user_id = ' .$this->user->data['user_id'];
		$result_nr = $this->db->sql_query($sql_nr);
		$total_posts = $this->db->sql_fetchfield('total', $result_nr);
		$this->db->sql_freeresult($result_nr);
		
		$i = $start_log = $li = 0;
		$shout_number = $this->config['shout_nr_acp'];
		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 's.*, u.user_id, u.username, u.user_colour',
			'FROM'      => array($this->shoutbox_table => 's'),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 's.shout_user_id = u.user_id'
				)
			),
			'WHERE'		=> 'shout_inp = 0 OR shout_inp = ' .$this->user->data['user_id']. ' OR shout_user_id = ' .$this->user->data['user_id'],
			'ORDER_BY'	=> 's.shout_time DESC',
		));
		$result = $this->db->sql_query_limit($sql, $shout_number, $start);
		while ($row = $this->db->sql_fetchrow($result))
		{
			if ($row['shout_inp'])
			{
				if ($row['shout_inp'] != $this->user->data['user_id'] && $row['shout_user_id'] != $this->user->data['user_id'])
				{
					continue;
				}
				$row['shout_text'] = str_replace('SHOUT_USER_POST', $this->user->lang['SHOUT_USER_POST'], $row['shout_text']);
			}
			if ($row['shout_user_id'] == ANONYMOUS)
			{
				list($row['username'], $row['shout_text']) = explode('|||', $row['shout_text']);
				$row['shout_text'] = str_replace($row['username'].'|||', '', $row['shout_text']);
			}
			$row['user_colour'] = ($row['user_id'] == 0) ? $this->config['shout_color_robot'] : $row['user_colour'];
			$row['username'] 	= ($row['user_id'] == 0) ? $this->user->lang['SHOUT_ROBOT'] : $row['username'];// ROBOT
			$shout_text 		= ($row['shout_info'] > 0) ? $this->shoutbox->display_infos_robot($row['shout_info'], $row['shout_text'], true) : $row['shout_text'];
			$shout_text			= str_replace(array('./../', './'), generate_board_url(). '/', $shout_text);
			
			$this->template->assign_block_vars('messages', array(
				'TIME'				=> $this->user->format_date($row['shout_time']),
				'POSTER'			=> get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']),
				'ID'				=> $row['shout_id'],
				'MESSAGE'			=> generate_text_for_display($shout_text, $row['shout_bbcode_uid'], $row['shout_bbcode_bitfield'], $row['shout_bbcode_flags']),
				'ROW_NUMBER'		=> $i+ ($start +1),
			));
			$i++;
		}
		$this->db->sql_freeresult($result);
		
		$log_array = array('LOG_SHOUT_SCRIPT', 'LOG_SHOUT_ACTIVEX', 'LOG_SHOUT_APPLET', 'LOG_SHOUT_OBJECTS', 'LOG_SHOUT_IFRAME');
		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 'l.log_id, l.user_id, l.log_type, l.log_ip, l.log_time, l.log_operation, l.reportee_id, u.user_id, u.username, u.user_colour',
			'FROM'      => array(LOG_TABLE => 'l'),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'l.user_id = u.user_id'
				)
			),
			'WHERE'		=> $this->db->sql_in_set('log_operation', $log_array),
			'ORDER_BY'	=> 'log_time DESC',
		));
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$row['user_colour'] = ($row['user_id'] == 0) ? $this->config['shout_color_robot'] : $row['user_colour'];
			$row['username'] 	= ($row['user_id'] == ANONYMOUS) ? $this->user->lang['GUEST'] : (($row['user_id'] == 0) ? $this->user->lang['SHOUT_ROBOT'] : $row['username']);
			$username			= get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']);
			
			$this->template->assign_block_vars('logs', array(
				'TIME'				=> $this->user->format_date($row['log_time']),
				'REPORTEE'			=> $username,
				'OPERATION'			=> $this->user->lang[$row['log_operation']],
				'ID'				=> $row['log_id'],
				'IP'				=> $row['log_ip'],
				'ROW_NUMBER'		=> $li + ($start_log + 1),
			));
			$li++;
		}
		$this->db->sql_freeresult($result);
		
		$total_del = $this->config['shout_del_acp'] + $this->config['shout_del_auto'] + $this->config['shout_del_purge'] + $this->config['shout_del_user'];
		$this->template->assign_vars(array(
			'S_OVERVIEW'				=> true,
			'S_DISPLAY_MESSAGES'		=> ($i > 0) ? true : false,
			'S_DISPLAY_LOGS'			=> ($li > 0) ? true : false,
			'S_ON_PAGE'					=> ($total_posts > $shout_number) ? true : false,
			'TOTAL_POSTS'				=> $total_posts,
			'SHOUT_VERSION'     		=> $this->config['shout_version'],
			'TOTAL_MESSAGES'			=> sprintf($this->user->lang['NUMBER_MESSAGE' .$this->shoutbox->plural($total_posts)], $total_posts),
			'MESSAGES_TOTAL_NR'			=> sprintf($this->user->lang['SHOUT_MESSAGES_TOTAL_NR'], $this->config['shout_nr'], $this->user->format_date($this->config['shout_time'])),
			'PAGE_NUMBER' 				=> $this->pagination->validate_start($total_posts, $shout_number, $start),	
			'LAST_SHOUT_RUN'			=> ($this->config['shout_last_run'] == $this->config['shout_time']) ? $this->user->lang['SHOUT_NEVER'] : $this->user->format_date($this->config['shout_last_run']),
			'LOGS_TOTAL_NR'				=> sprintf($this->user->lang['NUMBER_LOG' .$this->shoutbox->plural($this->config['shout_nr_log']). '_TOTAL'], $this->config['shout_nr_log'], $this->user->format_date($this->config['shout_time'])),
			'MESSAGES_DEL_TOTAL'		=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($total_del)], $total_del) . $this->user->lang['SHOUT_DEL_TOTAL'],
			'MESSAGES_DEL_ACP'			=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_acp'])], $this->config['shout_del_acp']),
			'MESSAGES_DEL_AUTO'			=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_auto'])], $this->config['shout_del_auto']),
			'MESSAGES_DEL_PURGE'		=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_purge'])], $this->config['shout_del_purge']),
			'MESSAGES_DEL_USER'			=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_user'])], $this->config['shout_del_user']),
		));
		$this->pagination->generate_template_pagination($this->u_action, 'pagination', 'start', $total_posts, $shout_number, $start);
	}
	
	public function acp_shoutbox_config_gen()
	{
		$mode = $this->request->variable('mode', '');
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			$settings = array(
				'shout_title'				=> str_replace("'", $this->user->lang['SHOUT_PROTECT'], utf8_normalize_nfc($this->request->variable('shout_title', 'shoutbox', true))),
				'shout_width_post'			=> $this->request->variable('shout_width_post', 325),
				'shout_prune'				=> $this->request->variable('shout_prune', 0),
				'shout_max_posts_on'		=> $this->request->variable('shout_max_posts_on', 100),
				'shout_max_posts'			=> $this->request->variable('shout_max_posts', 300),
				'shout_on_cron'				=> $this->request->variable('shout_on_cron', 1),
				'shout_log_cron'			=> $this->request->variable('shout_log_cron', 0),
				'shout_non_ie_nr'			=> $this->request->variable('shout_non_ie_nr', 25),
				'shout_ie_nr'				=> $this->request->variable('shout_ie_nr', 5),
				'shout_height'				=> $this->request->variable('shout_height', 160),
				'shout_color_background' 	=> $this->request->variable('shout_color_background', 'blue'),
				'shout_button_background' 	=> $this->request->variable('shout_button_background', 1),
				'shout_bar_option'			=> $this->request->variable('shout_bar_option', 1),
				'shout_pagin_option'		=> $this->request->variable('shout_pagin_option', 0),
			);
			foreach ($settings as $config_name => $config_value)
			{
				set_config($config_name, $config_value, false);
			}
			$this->cache->destroy('config');
			add_log('admin', 'LOG_SHOUT_' . strtoupper($mode));
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$color_path = $this->ext_path. 'styles/prosilver/theme/images/shout/fond/'; 
			$option = $option_sub = '';
			$imglist = @filelist($color_path, '', 'gif');
			
			if (sizeof($imglist))
			{
				$option = $this->shoutbox->build_select_img($imglist, 'shout_color_background', false);
			}
			$this->template->assign_vars(array(
				'SHOUT_TITLE'			=> isset($this->config['shout_title']) ? (string)utf8_normalize_nfc($this->config['shout_title'], true) : '',
				'SHOUT_WIDTH_POST'		=> isset($this->config['shout_width_post']) ? (int)$this->config['shout_width_post'] : '',
				'SHOUT_PRUNE'			=> isset($this->config['shout_prune']) ? (int)$this->config['shout_prune'] : '',
				'SHOUT_MAX_POSTS'		=> isset($this->config['shout_max_posts']) ? (int)$this->config['shout_max_posts'] : '',
				'SHOUT_MAX_POSTS_ON'	=> isset($this->config['shout_max_posts_on']) ? (int)$this->config['shout_max_posts_on'] : '',
				'SHOUT_NON_IE_NR'		=> isset($this->config['shout_non_ie_nr']) ? (int)$this->config['shout_non_ie_nr'] : '',
				'SHOUT_IE_NR'			=> isset($this->config['shout_ie_nr']) ? (int)$this->config['shout_ie_nr'] : '',
				'SHOUT_HEIGHT'			=> isset($this->config['shout_height']) ? (int)$this->config['shout_height'] : '',
				'SHOUT_ON_CRON'			=> $this->shoutbox->construct_radio_shout('shout_on_cron', 2, false),
				'SHOUT_LOG_CRON'		=> $this->shoutbox->construct_radio_shout('shout_log_cron', 2, false),
				'SHOUT_BACKGROUND'		=> isset($this->config['shout_color_background']) ? (string)$this->config['shout_color_background'] : '',
				'COLOR_IMAGE'			=> isset($this->config['shout_color_background']) ? $color_path . $this->config['shout_color_background'] . '.gif' : '',
				'SHOUT_BUTTON'			=> $this->shoutbox->construct_radio_shout('shout_button_background', 1, false),
				'SHOUT_BAR_OPTION'		=> isset($this->config['shout_bar_option']) ? (bool)$this->config['shout_bar_option'] : '',
				'SHOUT_PAGIN_OPTION'	=> isset($this->config['shout_pagin_option']) ? (bool)$this->config['shout_pagin_option'] : '',
				'OPTION'				=> $option,
				'COLOR_PATH'			=> $color_path,
			));
		}
		$this->template->assign_var('S_CONFIG_GEN', true);
	}
	
	public function acp_shoutbox_private()
	{
		$id 			= $this->request->variable('i', '');
		$action 		= $this->request->variable('action', '');
		$start			= $this->request->variable('start', 0);
		$marked			= $this->request->variable('mark', array(0));
		$creation 		= $this->request->variable('creation_time', 0);
		$token 			= $this->request->variable('form_token', '');
		$mode			= $this->request->variable('mode', '');
		$deletemark 	= $this->request->is_set_post('delmarked') ? true : false;
		$deletemarklog 	= $this->request->is_set_post('delmarkedlog') ? true : false;
		$form_key 		= 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		
		if ($deletemark)
		{
			if (confirm_box(true))
			{
				if (!check_form_key($form_key))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$where_sql = '';
				if ($deletemark && sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
					}
					$where_sql = ' WHERE ' . $this->db->sql_in_set('shout_id', $sql_in);
					unset($sql_in);
				}
				if ($where_sql)
				{
					$sql = 'DELETE FROM ' . $this->shoutbox_priv_table . $where_sql;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					
					$log_entrie = 'LOG_SELECT' .$this->shoutbox->plural($deleted). '_SHOUTBOX_PRIV';
					add_log('admin', $log_entrie, $deleted);
					set_config_count('shout_del_acp_priv', $deleted, true);
				}
			}
			else
			{
				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'start'			=> $start,
					'delmarked'		=> $deletemark,
					'mark'			=> $marked,
					'i'				=> $id,
					'mode'			=> $mode,
					'action'		=> $action,
					'creation_time' => $creation,
					'form_token'	=> $token,
				)));
			}
		}
		if ($deletemarklog)
		{
			if (confirm_box(true))
			{
				if (!check_form_key($form_key))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				$where_sql = '';
				if ($deletemarklog && sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
					}
					$where_sql = ' WHERE ' . $this->db->sql_in_set('log_id', $sql_in);
					unset($sql_in);
				}
				if ($where_sql)
				{
					$sql = 'DELETE FROM ' . LOG_TABLE . $where_sql;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					
					$log_entrie = 'LOG_LOG' .$this->shoutbox->plural($deleted). '_SHOUTBOX_PRIV';
					add_log('admin', $log_entrie, $deleted);
				}
			}
			else
			{
				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'start'			=> $start,
					'delmarkedlog'	=> $deletemarklog,
					'mark'			=> $marked,
					'i'				=> $id,
					'creation_time' => $creation,
					'form_token'	=> $token,
				)));
			}
		}
		if ($action)
		{
			if (!confirm_box(true))
			{
				switch ($action)
				{
					default:
						$confirm = true;
						$confirm_lang = 'CONFIRM_OPERATION';
				}
				if ($confirm)
				{
					confirm_box(false, $this->user->lang[$confirm_lang], build_hidden_fields(array(
						'i'				=> $id,
						'action'		=> $action,
						'creation_time' => $creation,
						'form_token'	=> $token,
					)));
				}
			}
			else
			{
				if (!check_form_key($form_key))
				{
					trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
				}
				switch ($action)
				{
					case 'purge':
						$sql = 'DELETE FROM ' . $this->shoutbox_priv_table;
						$this->db->sql_query($sql);
						$deleted = $this->db->sql_affectedrows();
						
						set_config_count('shout_del_purge_priv', $deleted, true);
						add_log('admin', 'LOG_PURGE_SHOUTBOX_PRIV');
						$this->shoutbox->post_robot_shout(0, $this->user->ip, true, true, false);
					break;
					case 'purge_1':
					case 'purge_2':
					case 'purge_3':
					case 'purge_4':
					case 'purge_5':
					case 'purge_6':
					case 'purge_7':
					case 'purge_8':
						$sort = str_replace('purge_', '', $action);
						$retour = $this->purge_shout_admin($sort, true);
						if ($retour)
						{
							adm_back_link($this->u_action);
						}
					break;
				}
			}
		}
		
		$sql_nr = 'SELECT COUNT(DISTINCT shout_id) as total 
			FROM ' . $this->shoutbox_priv_table . '
			WHERE shout_inp = 0 
				OR shout_inp = ' .$this->user->data['user_id']. ' 
				OR shout_user_id = ' .$this->user->data['user_id'];
		$result_nr = $this->db->sql_query($sql_nr);
		$total_posts = $this->db->sql_fetchfield('total', $result_nr);
		$this->db->sql_freeresult($result_nr);
		
		$start = $this->request->variable('start', 0);
		$i = $start_log = $li = 0;
		$shout_number = $this->config['shout_nr_acp'];
		
		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 's.*, u.user_id, u.username, u.user_colour',
			'FROM'      => array($this->shoutbox_priv_table => 's'),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 's.shout_user_id = u.user_id'
				)
			),
			'WHERE'		=> 'shout_inp = 0 OR shout_inp = ' .$this->user->data['user_id']. ' OR shout_user_id = ' .$this->user->data['user_id'],
			'ORDER_BY'	=> 's.shout_time DESC',
		));
		$result = $this->db->sql_query_limit($sql, $shout_number, $start);
		while ($row = $this->db->sql_fetchrow($result))
		{
			if ($row['shout_inp'])
			{
				if ($row['shout_inp'] != $this->user->data['user_id'] && $row['shout_user_id'] != $this->user->data['user_id'])
				{
					continue;
				}
				$row['shout_text'] = str_replace('SHOUT_USER_POST', $this->user->lang['SHOUT_USER_POST'], $row['shout_text']);
			}
			$row['user_colour'] = ($row['user_id'] == 0) ? $this->config['shout_color_robot'] : $row['user_colour'];// ROBOT
			$row['username'] 	= ($row['user_id'] == ANONYMOUS) ? $this->user->lang['GUEST'] : (($row['user_id'] == 0) ? $this->user->lang['SHOUT_ROBOT'] : $row['username']);//ROBOT
			$shout_text 		= ($row['shout_info'] > 0) ? $this->shoutbox->display_infos_robot($row['shout_info'], $row['shout_text'], true) : $row['shout_text'];
			$shout_text			= str_replace(array('./', './../'), generate_board_url(). '/', $shout_text);
			
			$this->template->assign_block_vars('messages', array(
				'TIME'				=> $this->user->format_date($row['shout_time']),
				'POSTER'			=> get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']),
				'ID'				=> $row['shout_id'],
				'MESSAGE'			=> generate_text_for_display($shout_text, $row['shout_bbcode_uid'], $row['shout_bbcode_bitfield'], $row['shout_bbcode_flags']),
				'ROW_NUMBER'		=> $i+ ($start +1),
			));
			$i++;
		}
		$this->db->sql_freeresult($result);
		
		$log_array = array('LOG_SHOUT_SCRIPT_PRIV', 'LOG_SHOUT_ACTIVEX_PRIV', 'LOG_SHOUT_APPLET_PRIV', 'LOG_SHOUT_OBJECTS_PRIV', 'LOG_SHOUT_IFRAME_PRIV');
		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 'l.log_id, l.user_id, l.log_type, l.log_ip, l.log_time, l.log_operation, l.reportee_id, u.user_id, u.username, u.user_colour',
			'FROM'      => array(LOG_TABLE => 'l'),
			'LEFT_JOIN'	=> array(
				array(
					'FROM'	=> array(USERS_TABLE => 'u'),
					'ON'	=> 'l.user_id = u.user_id'
				)
			),
			'WHERE'		=> $this->db->sql_in_set('log_operation', $log_array),
			'ORDER_BY'	=> 'log_time DESC',
		));
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$row['user_colour'] = ($row['user_id'] == 0) ? $this->config['shout_color_robot'] : $row['user_colour'];
			$row['username'] 	= ($row['user_id'] == ANONYMOUS) ? $this->user->lang['GUEST'] : (($row['user_id'] == 0) ? $this->user->lang['SHOUT_ROBOT'] : $row['username']);
			$username			= get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']);
			
			$this->template->assign_block_vars('logs', array(
				'TIME'				=> $this->user->format_date($row['log_time']),
				'REPORTEE'			=> $username,
				'OPERATION'			=> $this->user->lang[$row['log_operation']],
				'ID'				=> $row['log_id'],
				'IP'				=> $row['log_ip'],
				'ROW_NUMBER'		=> $li + ($start_log + 1),
			));
			$li++;
		}
		$this->db->sql_freeresult($result);
		
		$total_del = $this->config['shout_del_acp_priv'] + $this->config['shout_del_auto_priv'] + $this->config['shout_del_purge_priv'] + $this->config['shout_del_user_priv'];
		$this->template->assign_vars(array(
			'S_PRIVATE'					=> true,
			'TOTAL_POSTS'				=> $total_posts,
			'S_DISPLAY_MESSAGES'		=> ($i > 0) ? true : false,
			'S_DISPLAY_LOGS'			=> ($li > 0) ? true : false,
			'S_ON_PAGE'					=> ($total_posts > $shout_number) ? true : false,
			'TOTAL_MESSAGES'			=> sprintf($this->user->lang['NUMBER_MESSAGE' . $this->shoutbox->plural($total_posts)], $total_posts),
			'MESSAGES_TOTAL_NR'			=> sprintf($this->user->lang['SHOUT_MESSAGES_TOTAL_NR'], $this->config['shout_nr_priv'], $this->user->format_date($this->config['shout_time_priv'])),
			'PAGE_NUMBER' 				=> $this->pagination->validate_start($total_posts, $shout_number, $start),	
			'LAST_SHOUT_RUN'			=> ($this->config['shout_last_run_priv'] == $this->config['shout_time_priv']) ? $this->user->lang['SHOUT_NEVER'] : $this->user->format_date($this->config['shout_last_run_priv']),
			'LOGS_TOTAL_NR'				=> sprintf($this->user->lang['NUMBER_LOG' .$this->shoutbox->plural($this->config['shout_nr_log_priv']). '_TOTAL'], $this->config['shout_nr_log_priv'], $this->user->format_date($this->config['shout_time_priv'])),
			'MESSAGES_DEL_TOTAL'		=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($total_del)], $total_del) . $this->user->lang['SHOUT_DEL_TOTAL'],
			'MESSAGES_DEL_ACP'			=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_acp_priv'])], $this->config['shout_del_acp_priv']),
			'MESSAGES_DEL_AUTO'			=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_auto_priv'])], $this->config['shout_del_auto_priv']),
			'MESSAGES_DEL_PURGE'		=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_purge_priv'])], $this->config['shout_del_purge_priv']),
			'MESSAGES_DEL_USER'			=> sprintf($this->user->lang['SHOUT_DEL_NR' .$this->shoutbox->plural($this->config['shout_del_user_priv'])], $this->config['shout_del_user_priv']),
		));
		$this->pagination->generate_template_pagination($this->u_action, 'pagination', 'start', $total_posts, $shout_number, $start);
	}
	
	public function acp_shoutbox_config_priv()
	{
		$mode = $this->request->variable('mode', '');
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			
			$settings = array(
				'shout_title_priv'					=> str_replace("'", $this->user->lang['SHOUT_PROTECT'], utf8_normalize_nfc($this->request->variable('shout_title_priv', '', true))),
				'shout_width_post_priv'				=> $this->request->variable('shout_width_post_priv', 325),
				'shout_prune_priv'					=> $this->request->variable('shout_prune_priv', 0),
				'shout_on_cron_priv'				=> $this->request->variable('shout_on_cron_priv', 1),
				'shout_log_cron_priv'				=> $this->request->variable('shout_log_cron_priv', 0),
				'shout_max_posts_priv'				=> $this->request->variable('shout_max_posts_priv', 400),
				'shout_ie_nr_priv'					=> $this->request->variable('shout_ie_nr_priv', 20),
				'shout_max_posts_on_priv'			=> $this->request->variable('shout_max_posts_on_priv', 300),
				'shout_non_ie_height_priv'			=> $this->request->variable('shout_non_ie_height_priv', 460),
				'shout_non_ie_nr_priv'				=> $this->request->variable('shout_non_ie_nr_priv', 25),
				'shout_color_background_priv'		=> $this->request->variable('shout_color_background_priv', ''),
				'shout_button_background_priv'		=> $this->request->variable('shout_button_background_priv', 1),
				'shout_bar_option_priv'				=> $this->request->variable('shout_bar_option_priv', 1),
				'shout_pagin_option_priv'			=> $this->request->variable('shout_pagin_option_priv', 0),
				'shout_sound_new_priv'				=> $this->request->variable('shout_sound_new_priv', ''),
			);
			
			foreach ($settings as $config_name => $config_value)
			{
				set_config($config_name, $config_value, false);
			}
			
			add_log('admin', 'LOG_SHOUT_' .strtoupper($mode));
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$option = '';
			$color_path = $this->ext_path. 'styles/prosilver/theme/images/shout/fond/';
			$imglist = @filelist($color_path);
			if (sizeof($imglist))
			{
				$option = $this->shoutbox->build_select_img(array_values($imglist), 'shout_color_background_priv', false);
			}
			
			$this->template->assign_vars(array(
				'SHOUT_TITLE_PRIV'			=> isset($this->config['shout_title_priv']) ? utf8_normalize_nfc($this->config['shout_title_priv'], true) : '',
				'SHOUT_WIDTH_POST'			=> isset($this->config['shout_width_post_priv']) ? (int)$this->config['shout_width_post_priv'] : '',
				'SHOUT_PRUNE_PRIV'			=> isset($this->config['shout_prune_priv']) ? (int)$this->config['shout_prune_priv'] : '',
				'SHOUT_ON_CRON_PRIV'		=> $this->shoutbox->construct_radio_shout('shout_on_cron_priv', 2, false),
				'SHOUT_LOG_CRON_PRIV'		=> $this->shoutbox->construct_radio_shout('shout_log_cron_priv', 2, false),
				'SHOUT_BUTTON'				=> $this->shoutbox->construct_radio_shout('shout_button_background_priv', 1, false),
				'NEW_SOUND_PRIV'			=> $this->shoutbox->build_adm_sound_select('new_priv'),
				'SHOUT_MAX_POSTS'			=> isset($this->config['shout_max_posts_priv']) ? (int)$this->config['shout_max_posts_priv'] : '',
				'SHOUT_IE_NR_PRIV'			=> isset($this->config['shout_ie_nr_priv']) ? (int)$this->config['shout_ie_nr_priv'] : '',
				'SHOUT_MAX_POSTS_ON'		=> isset($this->config['shout_max_posts_on_priv']) ? (int)$this->config['shout_max_posts_on_priv'] : '',
				'SHOUT_NON_IE_HEIGHT_PRIV' 	=> isset($this->config['shout_non_ie_height_priv']) ? (int)$this->config['shout_non_ie_height_priv'] : '',
				'SHOUT_NON_IE_NR_PRIV'		=> isset($this->config['shout_non_ie_nr_priv']) ? (int)$this->config['shout_non_ie_nr_priv'] : '',
				'SHOUT_BACKGROUND'			=> isset($this->config['shout_color_background_priv']) ? (string)$this->config['shout_color_background_priv'] : '',
				'COLOR_IMAGE'				=> isset($this->config['shout_color_background_priv']) ? $color_path . $this->config['shout_color_background_priv'] . '.gif' : '',
				'SHOUT_BAR_TOP'				=> isset($this->config['shout_bar_option_priv']) ? (bool)$this->config['shout_bar_option_priv'] : '',
				'SHOUT_PAGIN_OPTION'		=> isset($this->config['shout_pagin_option_priv']) ? (bool)$this->config['shout_pagin_option_priv'] : '',
				'SHOUT_SOUNDS_PATH'			=> $this->ext_path . 'images/sounds/',
				'OPTION_IMAGE'				=> $option,
				'COLOR_PATH'				=> $color_path,
				'S_PRIV_CONFIG'				=> true,
			));
		}
	}
	
	public function acp_shoutbox_popup()
	{
		$mode = $this->request->variable('mode', '');
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			
			$settings = array(
				'shout_width_post_pop'			=> $this->request->variable('shout_width_post_pop', 325),
				'shout_non_ie_height_pop'		=> $this->request->variable('shout_non_ie_height_pop', 460),
				'shout_ie_nr_pop'				=> $this->request->variable('shout_ie_nr_pop', 20),
				'shout_non_ie_nr_pop'			=> $this->request->variable('shout_non_ie_nr_pop', 25),
				'shout_popup_height'			=> $this->request->variable('shout_popup_height', 580),
				'shout_popup_width'				=> $this->request->variable('shout_popup_width', 1100),
				'shout_color_background_pop'	=> $this->request->variable('shout_color_background_pop', 'blue'),
				'shout_button_background_pop'	=> $this->request->variable('shout_button_background_pop', 1),
				'shout_bar_option_pop'			=> $this->request->variable('shout_bar_option_pop', 1),
				'shout_pagin_option_pop'		=> $this->request->variable('shout_pagin_option_pop', 0),
			);
			
			foreach ($settings as $config_name => $config_value)
			{
				set_config($config_name, $config_value, false);
			}
			
			add_log('admin', 'LOG_SHOUT_' . strtoupper($mode));
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$color_path = $this->ext_path. 'styles/prosilver/theme/images/shout/fond/';
			$option = $option_sub = $panel_open = $panel_close = '';
			$imglist = @filelist($color_path);
			
			if (sizeof($imglist))
			{
				$imglist = array_values($imglist);
				$option = $this->shoutbox->build_select_img($imglist, 'shout_color_background_pop', false);
			}
			
			$this->template->assign_vars(array(
				'SHOUT_WIDTH_POST'			=> isset($this->config['shout_width_post_pop']) ? (int)$this->config['shout_width_post_pop'] : '',
				'SHOUT_NON_IE_HEIGHT_POP' 	=> isset($this->config['shout_non_ie_height_pop']) ? (int)$this->config['shout_non_ie_height_pop'] : '',
				'SHOUT_IE_NR_POP'			=> isset($this->config['shout_ie_nr_pop']) ? (int)$this->config['shout_ie_nr_pop'] : '',
				'SHOUT_NON_IE_NR_POP'		=> isset($this->config['shout_non_ie_nr_pop']) ? (int)$this->config['shout_non_ie_nr_pop'] : '',
				'SHOUT_HEIGHT_POP'			=> isset($this->config['shout_popup_height']) ? (int)$this->config['shout_popup_height'] : '',
				'SHOUT_WIDTH_POP'			=> isset($this->config['shout_popup_width']) ? (int)$this->config['shout_popup_width'] : '',
				'SHOUT_BACKGROUND'			=> isset($this->config['shout_color_background_pop']) ? (string)$this->config['shout_color_background_pop'] : '',
				'COLOR_IMAGE'				=> isset($this->config['shout_color_background_pop']) ? (string)$color_path . $this->config['shout_color_background_pop'] . '.gif' : '',
				'SHOUT_BUTTON'				=> $this->shoutbox->construct_radio_shout('shout_button_background_pop', 1, false),
				'SHOUT_BAR_OPTION'			=> isset($this->config['shout_bar_option_pop']) ? (bool)$this->config['shout_bar_option_pop'] : '',
				'SHOUT_PAGIN_OPTION'		=> isset($this->config['shout_pagin_option_pop']) ? (bool)$this->config['shout_pagin_option_pop'] : '',
				'OPTION'					=> $option,
				'OPTION_SUB'				=> $option_sub,
				'COLOR_PATH'				=> $color_path,
				'S_POPUP'					=> true,
			));
		}
	}
	
	public function acp_shoutbox_panel()
	{
		$mode = $this->request->variable('mode', '');
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			
			$settings = array(
				'shout_panel'					=> $this->request->variable('shout_panel', 1),
				'shout_panel_all'				=> $this->request->variable('shout_panel_all', 0),
				'shout_panel_auto'				=> $this->request->variable('shout_panel_auto', 0),
				'shout_page_exclude'			=> str_replace("\n", '||', $this->request->variable('shout_page_exclude', '')),
				'shout_panel_img'				=> $this->request->variable('shout_panel_img', ''),
				'shout_panel_exit_img'			=> $this->request->variable('shout_panel_exit_img', ''),
				'shout_panel_width'				=> $this->request->variable('shout_panel_width', 800),
				'shout_panel_height'			=> $this->request->variable('shout_panel_height', 510),
			);
			foreach ($settings as $config_name => $config_value)
			{
				set_config($config_name, $config_value, false);
			}
			add_log('admin', 'LOG_SHOUT_' . strtoupper($mode));
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$panel_open = $panel_close = '';
			$panel_path = $this->ext_path. 'images/panel/';
			$panelist = @filelist($panel_path);
			
			if (sizeof($panelist))
			{
				$panelist = array_values($panelist);
				$panel_open = $this->shoutbox->build_select_img($panelist, 'shout_panel_img', false, true);
				$panel_close = $this->shoutbox->build_select_img($panelist, 'shout_panel_exit_img', true, true);
			}
			
			$this->template->assign_vars(array(
				'SHOUT_PANEL'				=> $this->shoutbox->construct_radio_shout('shout_panel', 2, false),
				'SHOUT_PANEL_ALL'			=> $this->shoutbox->construct_radio_shout('shout_panel_all', 2, false),
				'SHOUT_PANEL_AUTO'			=> $this->shoutbox->construct_radio_shout('shout_panel_auto', 2, false),
				'SHOUT_PAGE_EXCLUDE'		=> isset($this->config['shout_page_exclude']) ? str_replace('||', "\n", $this->config['shout_page_exclude']) : '',
				'PANEL_IMAGE'				=> isset($this->config['shout_panel_img']) ? $panel_path.(string)$this->config['shout_panel_img'] : $phpbb_root_path. 'images/spacer.gif',
				'PANEL_EXIT_IMAGE'			=> isset($this->config['shout_panel_exit_img']) ? $panel_path.(string)$this->config['shout_panel_exit_img'] : $phpbb_root_path. 'images/spacer.gif',
				'SHOUT_PANEL_WIDTH'			=> isset($this->config['shout_panel_width']) ? (string)$this->config['shout_panel_width'] : '',
				'SHOUT_PANEL_HEIGHT'		=> isset($this->config['shout_panel_height']) ? (string)$this->config['shout_panel_height'] : '',
				'PANEL_PATH'				=> $panel_path,
				'PANEL_OPTION'				=> $panel_open,
				'PANEL_EXIT_OPTION'			=> $panel_close,
			));
		}
		$this->template->assign_var('S_PANEL', true);
	}
	
	public function acp_shoutbox_smilies()
	{
		$id 		= $this->request->variable('i', '');
		$mode 		= $this->request->variable('mode', '');
		$action 	= $this->request->variable('action', '');
		$creation 	= $this->request->variable('creation', '');
		$token 		= $this->request->variable('token', '');
		$update 	= $this->request->is_set_post('update');
		$submit 	= $this->request->is_set_post('submit');
		$activer 	= $this->request->is_set_post('activer');
		$desactiver = $this->request->is_set_post('desactiver');
		$start		= $this->request->variable('start', 0);
		$marked		= $this->request->variable('mark', array(0));
		$form_key 	= 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		
		if ($activer)
		{
			if (confirm_box(true))
			{
				$where_sql = '';
				$del_acp = $ni = 0;
				if (sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
						$nb_del_acp = $ni + ($del_acp + 1);
						$ni++;
					}
					$where_sql = ' WHERE ' . $this->db->sql_in_set('smiley_id', $sql_in);
					unset($sql_in);
				}

				if ($where_sql)
				{
					$sql = 'UPDATE ' . SMILIES_TABLE . " 
						SET display_on_shout = 1 
						$where_sql";
					$this->db->sql_query($sql);
					set_config_count('shout_nb_smilies_pop', -$ni, true);
					$this->cache->destroy('_shout_smilies');
					$this->cache->destroy('_shout_smilies_pop');
				}
			}
			else
			{
				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'start'			=> $start,
					'activer'		=> $activer,
					'mark'			=> $marked,
					'i'				=> $id,
					'mode'			=> $mode,
					'action'		=> $action,
					'creation_time' => $creation,
					'form_token'	=> $token,
				)));
			}
		}
		if ($desactiver)
		{
			if (confirm_box(true))
			{
				$where_sql = '';
				$del_acp = $ni = 0;
				if (sizeof($marked))
				{
					$sql_in = array();
					foreach ($marked as $mark)
					{
						$sql_in[] = $mark;
						$nb_del_acp = $ni + ($del_acp + 1);
						$ni++;
					}
					$where_sql = ' WHERE ' . $this->db->sql_in_set('smiley_id', $sql_in);
					unset($sql_in);
				}

				if ($where_sql)
				{
					$sql = 'UPDATE ' . SMILIES_TABLE . ' 
						SET display_on_shout = 0 ' .$where_sql;
					$this->db->sql_query($sql);
					set_config_count('shout_nb_smilies_pop', $ni, true);
					$this->cache->destroy('_shout_smilies');
					$this->cache->destroy('_shout_smilies_pop');
				}
			}
			else
			{
				confirm_box(false, $this->user->lang['CONFIRM_OPERATION'], build_hidden_fields(array(
					'start'			=> $start,
					'desactiver'	=> $desactiver,
					'mark'			=> $marked,
					'i'				=> $id,
					'mode'			=> $mode,
					'action'		=> $action,
					'creation_time' => $creation,
					'form_token'	=> $token,
				)));
			}
		}
		
		$smilies_path = $this->config['smilies_path'];
		$i = 0;
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			$smiley_id	= $this->request->variable('smiley', array(0));
			$settings 	= array('smiley_id'	=> $smiley_id,);

			foreach ($settings as $key => $value)
			{
				$sql_ary = array('shout_order' => $value);
				$sql = 'UPDATE ' . SMILIES_TABLE . ' 
					SET ' . $this->db->sql_build_array('UPDATE', $sql_ary) . ' 
					WHERE smiley_id = ' .(int)$smiley_id;
				$this->db->sql_query($sql);
			}
			
			$this->cache->destroy(array('_shout_smilies', '_shout_smilies_pop'));
			add_log('admin', 'LOG_SHOUT_SMILIES');
			trigger_error($this->user->lang['CONFIG_SMILIES_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$i = $ii = $iii = 0;
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'COUNT(smiley_id) AS total',
				'FROM'      => array(SMILIES_TABLE => ''),
			));
			$result = $this->db->sql_query($sql);
			$total_i = (int)$this->db->sql_fetchfield('total', $result);
			$this->db->sql_freeresult($result);
			
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'smiley_id, smiley_url, code, emotion, smiley_width, smiley_height, smiley_order, display_on_shout',
				'FROM'      => array(SMILIES_TABLE => ''),
				'GROUP_BY'	=> 'emotion',
				'ORDER_BY'	=> 'smiley_order ASC',
			));
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$this->template->assign_block_vars('smilies', array(
					'S_SMILEY_URL'		=> $this->shoutbox->shout_img($row['smiley_url'], $row['emotion'], $row['smiley_width'], $row['smiley_height'], $this->root_path . $smilies_path. '/'),
					'ID'				=> $row['smiley_id'],
					'CODE'				=> $row['code'],
					'EMOTION'			=> $row['emotion'],
					'SMILEY_WIDTH'		=> $row['smiley_width'],
					'SMILEY_HEIGHT'		=> $row['smiley_height'],
					'DISPLAY_ON'		=> $row['display_on_shout'],
					'DISPLAY_COLOR'		=> 'color:' .($row['display_on_shout'] ? 'green' : 'red'). ';',
					'DISPLAY_TITLE'		=> $this->user->lang['SMILIES_CLIC_' .($row['display_on_shout'] ? 'NO' : 'YES')],
					'SMILEY_DISPLAYED'	=> $this->user->lang['SMILIES_' .($row['display_on_shout'] ? '' : 'NO_'). 'DISPLAYED'],
					'U_DISPLAYED'		=> $this->u_action . '&amp;smiley=' .$row['smiley_id']. '&amp;display=' .$row['display_on_shout'],
					'ORDER'				=> $row['smiley_order'],
					'FIRST_SMILIE'		=> ($i == 1) ? true : false,
					'LAST_SMILIE'		=> ($i == $total_i) ? true : false,
					'ROW_NUMBER'		=> $i + 1,
				));
				$i++;
			}
			$this->db->sql_freeresult($result);

			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'smiley_id, smiley_url, emotion, smiley_width, smiley_height',
				'FROM'      => array(SMILIES_TABLE => ''),
				'WHERE'		=> 'display_on_shout = 1',
				'GROUP_BY'	=> 'emotion',
				'ORDER_BY'	=> 'smiley_order ASC',
			));
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$this->template->assign_block_vars('smilies_shout', array(
					'S_SMILEY_URL'	=> $this->shoutbox->shout_img($row['smiley_url'], $row['emotion'], $row['smiley_width'], $row['smiley_height'], $this->root_path . $smilies_path. '/'),
				));
				$ii++;
			}
			$this->db->sql_freeresult($result);

			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'smiley_id, smiley_url, emotion, smiley_width, smiley_height',
				'FROM'      => array(SMILIES_TABLE => ''),
				'WHERE'		=> 'display_on_shout = 0',
				'GROUP_BY'	=> 'emotion',
				'ORDER_BY'	=> 'smiley_order ASC',
			));
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$this->template->assign_block_vars('smilies_popup', array(
					'S_SMILEY_URL'	=> $this->shoutbox->shout_img($row['smiley_url'], $row['emotion'], $row['smiley_width'], $row['smiley_height'], $this->root_path . $smilies_path. '/'),
				));
				$iii++;
			}
			$this->db->sql_freeresult($result);
		}
		
		$this->template->assign_vars(array(
			'S_SMILIES'					=> true,
			'U_DISPLAY_AJAX'			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'display_smilies', 's' => 2)),
			'ICON_UPDATE'				=> $this->shoutbox->shout_img('loading.gif', 'UPDATE'),
			'ICON_MOVE_UP'				=> $this->shoutbox->shout_img('icon_up.gif', 'MOVE_UP', false, false, $this->root_path. 'adm/images/'),
			'ICON_MOVE_UP_DISABLED'		=> $this->shoutbox->shout_img('icon_up_disabled.gif', 'MOVE_UP', false, false, $this->root_path. 'adm/images/'),
			'ICON_MOVE_DOWN'			=> $this->shoutbox->shout_img('icon_down.gif', 'MOVE_DOWN', false, false, $this->root_path. 'adm/images/'),
			'ICON_MOVE_DOWN_DISABLED'	=> $this->shoutbox->shout_img('icon_down_disabled.gif', 'MOVE_DOWN', false, false, $this->root_path. 'adm/images/'),
		));
	}
	
	public function acp_shoutbox_smilies_pop()
	{
		$this->user->setup('posting');

		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 'smiley_id, smiley_url, code, emotion, smiley_width, smiley_height, smiley_order',
			'FROM'      => array(SMILIES_TABLE => ''),
			'ORDER_BY'	=> 'smiley_order ASC',
		));
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$this->template->assign_block_vars('smiley', array(
				'A_SMILEY_CODE'	=> addslashes($row['code']),
				'SMILEY_IMG'	=> $this->shoutbox->shout_img($row['smiley_url'], $row['emotion'], $row['smiley_width'], $row['smiley_height'], $this->root_path.$this->config['smilies_path']. '/', $row['code']),
			));
		}
		$this->db->sql_freeresult($result);
		
		$this->template->assign_vars(array(
			'S_IN_SHOUT_ACP'	=> true,
			'SHOUTBOX_VERSION'	=> sprintf($this->user->lang['SHOUTBOX_VERSION_ACP_COPY'], $this->config['shout_board_url'], $this->config['shout_version']),
		));
		
		page_header($this->user->lang['SMILIES']);
		$this->template->set_filenames(array(
			'body' => '@sylver35_breizhshoutbox/shout_template.html')
		);
		page_footer();
	}
	
	public function acp_shoutbox_robot()
	{
		$mode = $this->request->variable('mode', '');
		$form_key = 'sylver35/breizhshoutbox';
		add_form_key ($form_key);
		
		if ($this->request->is_set_post('update'))
		{
			if (!check_form_key($form_key))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action), E_USER_WARNING);
			}
			
			$shout_cron_hour = $this->request->variable('shout_cron_hour', '');
			// Force 24 hours format
			$in = array('1', '2', '3', '4', '5', '6', '7', '8', '9');
			$out = array('01', '02', '03', '04', '05', '06', '07', '08', '09');
			if ($shout_cron_hour < 10 && $shout_cron_hour != '00')
			{
				$shout_cron_hour = str_replace($in, $out, $shout_cron_hour);
			}
			else if ($shout_cron_hour == '0')
			{
				$shout_cron_hour = '00';
			}

			$settings = array(
				'shout_enable_robot'		=> $this->request->variable('shout_enable_robot', 1),
				'shout_post_robot'			=> $this->request->variable('shout_post_robot', 1),
				'shout_rep_robot'			=> $this->request->variable('shout_rep_robot', 1),
				'shout_edit_robot'			=> $this->request->variable('shout_edit_robot', 1),
				'shout_post_robot_priv'		=> $this->request->variable('shout_post_robot_priv', 1),
				'shout_rep_robot_priv'		=> $this->request->variable('shout_rep_robot_priv', 1),
				'shout_edit_robot_priv'		=> $this->request->variable('shout_edit_robot_priv', 1),
				'shout_prez_form'			=> $this->request->variable('shout_prez_form', ''),
				'shout_color_robot'			=> $this->request->variable('shout_color_robot', ''),
				'shout_color_message'		=> $this->request->variable('shout_color_message', ''),
				'shout_delete_robot'		=> $this->request->variable('shout_delete_robot', 1),
				'shout_sessions'			=> $this->request->variable('shout_sessions', 1),
				'shout_sessions_priv'		=> $this->request->variable('shout_sessions_priv', 0),
				'shout_sessions_time'		=> $this->request->variable('shout_sessions_time', 15),
				'shout_sessions_bots'		=> $this->request->variable('shout_sessions_bots', 0),
				'shout_sessions_bots_priv'	=> $this->request->variable('shout_sessions_bots_priv', 0),
				'shout_hello'				=> $this->request->variable('shout_hello', 1),
				'shout_hello_priv'			=> $this->request->variable('shout_hello_priv', 1),
				'shout_newest'				=> $this->request->variable('shout_newest', 1),
				'shout_newest_priv'			=> $this->request->variable('shout_newest_priv', 1),
				'shout_birthday'			=> $this->request->variable('shout_birthday', 1),
				'shout_birthday_priv'		=> $this->request->variable('shout_birthday_priv', 1),
				'shout_cron_hour'			=> $shout_cron_hour,
				'shout_exclude_forums'		=> implode(', ', $this->request->variable('shout_exclude_forums', array(0))),
				'shout_birthday_exclude'	=> implode(', ', $this->request->variable('shout_birthday_exclude', array(0))),
				'shout_robot_choice'		=> implode(', ', $this->request->variable('shout_robot_choice', array(0))),
				'shout_robot_choice_priv'	=> implode(', ', $this->request->variable('shout_robot_choice_priv', array(0))),
			);

			foreach ($settings as $config_name => $config_value)
			{
				set_config($config_name, $config_value, false);
			}
			
			add_log('admin', 'LOG_SHOUT_' . strtoupper($mode));
			trigger_error($this->user->lang['CONFIG_UPDATED'] . adm_back_link($this->u_action));
		}
		else
		{
			$group_options = '';
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'   => 'DISTINCT group_type, group_name, group_id, group_colour',
				'FROM'      => array(GROUPS_TABLE => ''),
				'WHERE'		=> $this->db->sql_in_set('group_name', array('GUESTS', 'BOTS'), true),
				'ORDER_BY'	=> 'group_type DESC, group_name ASC',
			));
			$result = $this->db->sql_query($sql);
			while ($row = $this->db->sql_fetchrow($result))
			{
				$selected = (in_array($row['group_id'], explode(', ', $this->config['shout_birthday_exclude']))) ? ' selected="selected"' : '';
				$group_options .= '<option' .(($row['group_type'] == GROUP_SPECIAL) ? ' class="sep"' : ''). ' value="' .$row['group_id']. '" style="color:#' .$row['group_colour']. ';font-weight:bold;"' .$selected. '>' .(($row['group_type'] == GROUP_SPECIAL) ? $this->user->lang['G_' . $row['group_name']] : $row['group_name']). "</option>\n";
			}
			$this->db->sql_freeresult($result);
			
			$select_prez = '<select id="shout_prez_form" name="shout_prez_form"><option value="0">' .$this->user->lang['SELECT_FORUM']. '</option><option value="0" disabled="disabled"></option>';
			$select_prez .= make_forum_select((int)$this->config['shout_prez_form'], false, true, true). '</select>';
			
			$this->template->assign_vars(array(
				'SHOUT_ENABLE_ROBOT'		=> $this->shoutbox->construct_radio_shout('shout_enable_robot', 2),
				'SHOUT_POST_ROBOT'			=> $this->shoutbox->construct_radio_shout('shout_post_robot', 2, false),
				'SHOUT_REP_ROBOT'			=> $this->shoutbox->construct_radio_shout('shout_rep_robot', 2, false),
				'SHOUT_EDIT_ROBOT'			=> $this->shoutbox->construct_radio_shout('shout_edit_robot', 2, false),
				'SHOUT_POST_ROBOT_PRIV' 	=> $this->shoutbox->construct_radio_shout('shout_post_robot_priv', 2, false),
				'SHOUT_REP_ROBOT_PRIV'		=> $this->shoutbox->construct_radio_shout('shout_rep_robot_priv', 2, false),
				'SHOUT_EDIT_ROBOT_PRIV'		=> $this->shoutbox->construct_radio_shout('shout_edit_robot_priv', 2, false),
				'SHOUT_DELETE_ROBOT'		=> $this->shoutbox->construct_radio_shout('shout_delete_robot', 2, false),
				'SHOUT_SESSIONS'			=> $this->shoutbox->construct_radio_shout('shout_sessions', 2, false),
				'SHOUT_SESSIONS_PRIV'		=> $this->shoutbox->construct_radio_shout('shout_sessions_priv', 2, false),
				'SHOUT_SESSIONS_BOTS'		=> $this->shoutbox->construct_radio_shout('shout_sessions_bots', 2, false),
				'SHOUT_SESSIONS_BOTS_PRIV'	=> $this->shoutbox->construct_radio_shout('shout_sessions_bots_priv', 2, false),
				'SHOUT_HELLO'				=> $this->shoutbox->construct_radio_shout('shout_hello', 2, false),
				'SHOUT_HELLO_PRIV'			=> $this->shoutbox->construct_radio_shout('shout_hello_priv', 2, false),
				'SHOUT_NEWEST'				=> $this->shoutbox->construct_radio_shout('shout_newest', 2, false),
				'SHOUT_NEWEST_PRIV'			=> $this->shoutbox->construct_radio_shout('shout_newest_priv', 2, false),
				'SHOUT_BIRTHDAY'			=> $this->shoutbox->construct_radio_shout('shout_birthday', 2, false),
				'SHOUT_BIRTHDAY_PRIV'		=> $this->shoutbox->construct_radio_shout('shout_birthday_priv', 2, false),
				'SHOUT_COLOR_ROBOT'			=> isset($this->config['shout_color_robot']) ? (string)$this->config['shout_color_robot'] : '',
				'SHOUT_COLOR_MESSAGE'		=> isset($this->config['shout_color_message']) ? (string)$this->config['shout_color_message'] : '',
				'SHOUT_SESSIONS_TIME'		=> isset($this->config['shout_sessions_time']) ? (int)$this->config['shout_sessions_time'] : '',
				'SHOUT_CRON_HOUR'			=> $this->shoutbox->hour_select((isset($this->config['shout_cron_hour']) ? (int)$this->config['shout_cron_hour'] : ''), 'shout_cron_hour'),
				'SHOUT_PREZ_FORM'			=> $select_prez,
				'SHOUT_EXCLUDE_FORUMS'		=> make_forum_select(explode(', ', $this->config['shout_exclude_forums']), false, false, false, false),
				'GROUP_OPTIONS'				=> $group_options,
				'SERVER_HOUR'				=> sprintf($this->user->lang['SHOUT_SERVER_HOUR' .$this->shoutbox->plural(date('H'))], date('H'), date('i')),
			));
			
			for ($i = 1; $i < 8; $i++)
			{
				$this->template->assign_var('CHOICE_' .$i, preg_match('/'.$i.'/i', $this->config['shout_robot_choice']) ? ' checked="checked"' : '');
			}
			for ($i = 1; $i < 8; $i++)
			{
				$this->template->assign_var('CHOICE_PRIV_'.$i, preg_match('/'.$i.'/i', $this->config['shout_robot_choice_priv']) ? ' checked="checked"' : '');
			}
		}
		$this->template->assign_var('S_ROBOT', true);
	}

	/**
	 * Set page url
	 *
	 * @param string $u_action Custom form action
	 * @return null
	 * @access public
	 */
	public function set_page_url($u_action)
	{
		$this->u_action = $u_action;
	}
}
