<?php
/**
*
* @package Breizh Shoutbox Extension
* @version $Id: ajax.php 100
* @copyright (c) 2018 Sylver35  https://breizhcode.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

namespace sylver35\breizhshoutbox\controller;

use phpbb\json_response;

class ajax
{
	/* @var \sylver35\breizhshoutbox\core\breizhshoutbox */
	protected $shoutbox;
	
	/** @var \phpbb\cache\service */
	protected $cache;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/* @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var \phpbb\pagination */
	protected $pagination;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\log\log */
	protected $log;
	
	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;
	
	/** @var \phpbb\path_helper */
	protected $path_helper;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;
	
	/** @var string Custom form action */
	protected $u_action;
	
	/**
	* The database tables
	*
	* @var string */
	protected $shoutbox_table;
	protected $shoutbox_priv_table;

	/**
	 * Constructor
	 *
	 * @param \phpbb\config\config			$config				Config object
	 * @param \phpbb\controller\helper		$helper				Controller helper object
	 * @param \phpbb\cache\service			$cache				Cache object
	 * @param \phpbb\db\driver\driver		$db					Database object
	 * @param \phpbb\template\template		$template			Template object
	 * @param \phpbb\user					$user				User object
	 * @param string						$flags_table		Name of the table used to store flag data
	 * @param \phpbb\extension\manager		$ext_manager		Extension manager object
	 * @param \phpbb\path_helper			$path_helper		Path helper object
	 */
	public function __construct(
		\sylver35\breizhshoutbox\core\shoutbox $shoutbox, 
		\phpbb\cache\service $cache, 
		\phpbb\config\config $config, 
		\phpbb\controller\helper $helper, 
		\phpbb\db\driver\driver_interface $db, 
		\phpbb\pagination $pagination,
		\phpbb\request\request $request, 
		\phpbb\template\template $template, 
		\phpbb\auth\auth $auth, 
		\phpbb\user $user,
		\phpbb\log\log $log,
		\phpbb\extension\manager $ext_manager,
		\phpbb\path_helper $path_helper,
		$phpbb_root_path, 
		$php_ext,
		$shoutbox_table,
		$shoutbox_priv_table)
	{
		$this->shoutbox 			= $shoutbox;
		$this->cache 				= $cache;
		$this->config 				= $config;
		$this->helper 				= $helper;
		$this->db 					= $db;
		$this->pagination 			= $pagination;
		$this->request 				= $request;
		$this->template 			= $template;
		$this->auth 				= $auth;
		$this->user 				= $user;
		$this->log					= $log;
		$this->ext_manager	 		= $ext_manager;
		$this->path_helper	 		= $path_helper;
		$this->root_path 			= $phpbb_root_path;
		$this->php_ext 				= $php_ext;
		$this->shoutbox_table 		= $shoutbox_table;
		$this->shoutbox_priv_table 	= $shoutbox_priv_table;

		$this->ext_path = $this->ext_manager->get_extension_path('sylver35/breizhshoutbox', true);
		$this->ext_path_web = $this->path_helper->update_web_root_path($this->ext_path);
	}

	/**
	 * Function construct_ajax
	 *
	 * @param string	$m		$Mode
	 * @param string	$s		$Sort
	 */
	public function construct_ajax($m, $s)
	{
		$last = $this->request->variable('l', 0);
		$id = $this->request->variable('user', 0);
		$userid = $this->user->data['user_id'];
		$response = new json_response();
		$mode = $m;
		if (!$mode || !$s)
		{
			return;
		}

		// We have our own error handling!
		$this->db->sql_return_on_error(true);
		
		// First select the good table for the type of shoutbox
		switch ($s)
		{
			case 1: // Popup shoutbox
			case 2: // Normal shoutbox
				$_table = $this->shoutbox_table;
			break;
			case 3: // Private shoutbox
				$_table = $this->shoutbox_priv_table;
			break;
		}

		switch ($mode)
		{
			case 'smilies':
				$data = $this->shoutbox->get_shout_smilies(); // Get smilies from cache
				if (!$data || !$this->auth->acl_get('u_shout_smilies'))
				{
					$this->shoutbox->shout_error('NO_SMILIES');
					break;
				}
				$i = 0;
				$return = 'smilies||';
				$return .= $this->config['shout_nb_smilies_pop']. '||';
				foreach ($data as $row)
				{
					$return .= '<span style="cursor:pointer;" onclick="shout_insert_text(\'' .$row['code']. '\',true,false);return false;" title="' .$row['emotion']. '"><img src="' . generate_board_url() . '/' . $this->config['smilies_path']. '/' .$row['smiley_url']. '" title="' .$row['emotion']. '" alt="' .$row['code']. '" height="' .$row['smiley_height']. '" width="' .$row['smiley_width']. '" /></span> ';
					$i++;
				}
				$return .= '||' .$i;
				$response->send($return);
			break;
			
			case 'smilies_popup':
				$data = $this->shoutbox->get_shout_smilies_pop(); // Get smilies popup from cache
				if (!$data || !$this->auth->acl_get('u_shout_smilies'))
				{
					$this->shoutbox->shout_error('NO_SMILIES');
					break;
				}
				$i = 0;
				$return = 'smilies_popup||0||';
				foreach ($data as $row)
				{
					$return .= '<span style="cursor:pointer;" onclick="shout_insert_text(\'' .$row['code']. '\',true,false);return false;" title="' .$row['emotion']. '"><img src="' . generate_board_url() . '/' . $this->config['smilies_path']. '/' .$row['smiley_url']. '" title="' .$row['emotion']. '" alt="' .$row['code']. '" height="' .$row['smiley_height']. '" width="' .$row['smiley_width']. '" /></span> ';
					$i++;
				}
				$return .= '||' .$i;
				$response->send($return);
			break;

			case 'display_smilies':
				$smiley = $this->request->variable('smiley', 0);
				$display = $this->request->variable('display', -1);
				if ($smiley && $display != -1)
				{
					if ($display == 1)
					{
						$sql = 'UPDATE ' . SMILIES_TABLE . ' 
							SET display_on_shout = 0 
								WHERE smiley_id = ' .$smiley;
						$this->db->sql_query($sql);
						set_config_count('shout_nb_smilies_pop', 1, true);
						$content = '1';
					}
					else if ($display == 0)
					{
						$sql = 'UPDATE ' . SMILIES_TABLE . ' 
							SET display_on_shout = 1 
								WHERE smiley_id = ' .$smiley;
						$this->db->sql_query($sql);
						set_config_count('shout_nb_smilies_pop', -1, true);
						$content = '2';
					}

					$list_data = $list_data_pop = '';
					$sql_ary = array(
						'SELECT'    => 'smiley_id, code, smiley_url, smiley_width, smiley_height, emotion',
						'FROM'      => array(SMILIES_TABLE => ''),
						'WHERE'		=> 'display_on_shout = 1',
						'GROUP_BY'	=> 'emotion',
						'ORDER_BY'	=> 'smiley_order ASC',
					);
					$result = $this->db->sql_query($this->db->sql_build_query('SELECT', $sql_ary));
					if (!$result)
					{
						$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
						break;
					}
					while ($row = $this->db->sql_fetchrow($result))
					{
						$list_data .= '<img src="' . generate_board_url() . '/' . $this->config['smilies_path']. '/' .$row['smiley_url']. '" title="' .$row['emotion']. '" alt="' .$row['code']. '" height="' .$row['smiley_height']. '" width="' .$row['smiley_width']. '" /> ';
					}
					$this->db->sql_freeresult($result);
					
					$sql_ary = array(
						'SELECT'    => 'smiley_id, code, smiley_url, smiley_width, smiley_height, emotion',
						'FROM'      => array(SMILIES_TABLE => ''),
						'WHERE'		=> 'display_on_shout = 0',
						'GROUP_BY'	=> 'emotion',
						'ORDER_BY'	=> 'smiley_order',
					);
					$result = $this->db->sql_query($this->db->sql_build_query('SELECT', $sql_ary));
					if (!$result)
					{
						$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
						break;
					}
					while ($row_pop = $this->db->sql_fetchrow($result))
					{
						$list_data_pop .= '<img src="' . generate_board_url() . '/' . $this->config['smilies_path']. '/' .$row_pop['smiley_url']. '" title="' .$row_pop['emotion']. '" alt="' .$row_pop['code']. '" height="' .$row_pop['smiley_height']. '" width="' .$row_pop['smiley_width']. '" /> ';
					}
					$this->db->sql_freeresult($result);
					$this->cache->destroy('_shout_smilies');
					$this->cache->destroy('_shout_smilies_pop');
					$content .= '||'.$list_data.'||'.$list_data_pop;
				}
				else
				{
					$content = '3||';
				}
				$response->send($content);
			break;

			case 'user_bbcode':
				$open = $this->request->variable('open', '');
				$close = $this->request->variable('close', '');
				$other = $this->request->variable('ot', 0);
				$on_user = $other ? $other : $userid;
				$data = $this->shoutbox->parse_shout_bbcodes($open, $close, $other);
				switch ($data['sort']) // the result of parse
				{
					case 1: // Remove the bbcodes
						$sql = 'UPDATE ' . USERS_TABLE . " 
							SET shout_bbcode = '' 
							WHERE user_id = $on_user";
						$this->db->sql_query($sql);
						$content = '1||1||0||'.$on_user;
					break;
					case 2: // Retun error message
						$content = '2||2||' .$data['message'].'||'.$on_user;
					break;
					case 3: // Good ! Update the bbcodes
						$ok_bbcode = (string) ($open. '||' .$close);
						$uid = $bitfield = $options = '';
						// Change it in the db
						$sql = 'UPDATE ' . USERS_TABLE . " 
							SET shout_bbcode = '" .$this->db->sql_escape($ok_bbcode). "' 
								WHERE user_id = $on_user";
						$this->db->sql_query($sql);
						$text = $open.$this->user->lang['SHOUT_EXEMPLE'].$close;
						generate_text_for_storage($text, $uid, $bitfield, $options, true, false, true);
						$text = generate_text_for_display($text, $uid, $bitfield, $options);
						$content = '3||'.$open. '||' .$close. '||' .$text.'||'.$on_user;
					break;
					case 4: // Return no change message
						$uid = $bitfield = $options = '';
						if ($open != '1')
						{
							$text = $open.$this->user->lang['SHOUT_EXEMPLE'].$close;
							generate_text_for_storage($text, $uid, $bitfield, $options, true, false, true);
							$text = generate_text_for_display($text, $uid, $bitfield, $options);
						}
						else
						{
							$text = $this->user->lang['SHOUT_EXEMPLE'];
						}
						$content = '4||4||' .$data['message'].'||'.$on_user.'||'.$text;
					break;
					case 5: // Return error no permission
						$content = '5||5||' .$data['message'].'||'.$on_user;
					break;
				}
				$response->send($content);
			break;

			case 'charge_bbcode':
				$id = (int) $this->request->variable('user', 0);
				$sql = $this->db->sql_build_query('SELECT', array(
					'SELECT'    => 'user_id, user_type, username, user_colour, shout_bbcode',
					'FROM'      => array(USERS_TABLE => ''),
					'WHERE'		=> 'user_id = ' .$id,
				));
				$result = $this->db->sql_query_limit($sql, 1);
				$row = $this->db->sql_fetchrow($result);
				if ($row['shout_bbcode'])
				{
					$on_bbcode = explode('||', $row['shout_bbcode']);
					$uid = $bitfield = $options = '';
					$message = $on_bbcode[0].$this->user->lang['SHOUT_EXEMPLE'].$on_bbcode[1];
					generate_text_for_storage($message, $uid, $bitfield, $options, true, false, true);
					$message = generate_text_for_display($message, $uid, $bitfield, $options);
				}
				else
				{
					$on_bbcode[0] = $on_bbcode[1] = 0;
					$message = $this->user->lang['SHOUT_EXEMPLE'];
				}
				$name = get_username_string('full', $row['user_id'], $row['username'], $row['user_colour']);
				$this->db->sql_freeresult($result);
				$content = $id. '||' .$name. '||' .$on_bbcode[0]. '||' .$on_bbcode[1]. '||' .$message;
				$response->send($content);
			break;

			case 'online':
				$online = obtain_users_online();
				$online_strings = obtain_users_online_string($online);
				$l_online = $online_strings['l_online_users'];
				$userlist = $online_strings['online_userlist'];
				$l_time = $this->user->lang('VIEW_ONLINE_TIMES', (int)$this->config['load_online_time']);
				$content = $l_online.'<br />('.$l_time.')'. '||' .$userlist;
				$response->send($content);
			break;

			case 'rules':
				$rules = $this->shoutbox->shout_rules($s); // Get rules from cache
				$content = ($rules) ? '1||' .$rules : '0||';
				$response->send($content);
			break;

			case 'action_sound':
				$sound = explode('||', $this->user->data['user_shout']);
				switch ($sound[0])
				{
					case 0: // Turn off the sounds
						$sort = 1;
						$content = '1||button_shout_sound_off button_shout||' .$this->user->lang['SHOUT_CLICK_SOUND_ON']; // Apply css sound off and title to turn on
					break;
					case 1: // Turn on the sounds
						$sort = 0;
						$content = '0||button_shout_sound button_shout||' .$this->user->lang['SHOUT_CLICK_SOUND_OFF']; // Apply css sound on and title to turn off
					break;
				}
				$config_user_shout = implode('||', array($sort, $sound[1], $sound[2], $sound[3], $sound[4], $sound[5], $sound[6], $sound[7], $sound[8], $sound[9]));
				$sql = 'UPDATE ' . USERS_TABLE . " 
					SET user_shout = '" .$this->db->sql_escape($config_user_shout). "' 
					WHERE user_id = $userid";
				$this->db->sql_query($sql);
				$response->send($content);
			break;

			case 'cite':
				$sql = $this->db->sql_build_query('SELECT', array(
					'SELECT'    => 'user_id, user_type, username, user_colour',
					'FROM'      => array(USERS_TABLE => ''),
					'WHERE'		=> 'user_id = ' .$id,
				));
				$result = $this->db->sql_query_limit($sql, 1);
				if (!$result)
				{
					$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
					break;
				}
				$row = $this->db->sql_fetchrow($result);
				if (!$row || $row['user_type'] == USER_IGNORE)
				{
					$content = '0||' .$this->user->lang['NO_USER'];// From common.php
				}
				else
				{
					$content = '1||[b]' .($row['user_colour'] ? '[color=#' .$row['user_colour']. ']' .$row['username']. '[/color]' : $row['username']). '[/b] ';
				}
				$this->db->sql_freeresult($result);
				$response->send($content);
			break;

			case 'action_user':
				if ($this->user->data['user_id'] == ANONYMOUS || !$id || $this->user->data['is_bot'])
				{
					$content = '0||0';
				}
				else
				{
					$sql = 'SELECT z.*, u.user_id, u.user_type, u.username, u.user_colour, u.user_avatar, u.user_avatar_type, u.user_avatar_width, u.user_avatar_height
						FROM ' . USERS_TABLE . ' AS u
						LEFT JOIN ' . ZEBRA_TABLE . ' AS z ON u.user_id = z.zebra_id AND z.user_id = ' .$this->user->data['user_id']. ' 
						WHERE u.user_id = ' . $id;
					$result = $this->db->sql_query_limit($sql, 1);
					if (!$result)
					{
						$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
						break;
					}
					$row = $this->db->sql_fetchrow($result);
					if (!$row || $row['user_type'] == USER_IGNORE)
					{
						$content = '1||' .$this->user->lang['NO_USER'];// From common.php
					}
					else
					{
						// Construct urls to be displayed via javascript
						// !Important: HTML must be formated here, never in html file because of security issues, changes with permissions and ajax work
						$inp		= ($this->auth->acl_get('u_shout_post_inp') || $this->auth->acl_get('a_') || $this->auth->acl_get('m_')) ? true : false; // administrators & moderators can always use this part
						$go_founder	= ($row['user_type'] != USER_FOUNDER || $this->user->data['user_type'] == USER_FOUNDER) ? true : false; // Founder protections
						$adm_path	= $this->root_path. 'adm/';
						$robot		= '<span style="color:#' .$this->config['shout_color_robot']. ';font-weight:bold;">' .$this->user->lang['SHOUT_ROBOT']. '</span>';
						$tpl_open 	= '<strong>&#187;</strong><span class="profile-shout">';
						$tpl_close 	= '</a></span>';
						
						$avatar		= $this->shoutbox->shout_user_avatar($row['user_id'], $row, 60, true);
						
						$username 	= get_username_string('full', $row['user_id'], $row['username'], $row['user_colour'], false, $this->shoutbox->real_board_url(true));
						
						$url_message = $inp ? $tpl_open. '<a style="cursor:pointer;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" onclick="personalMsg();return false;" title="' .$this->user->lang['SHOUT_ACTION_MSG']. '">' .$this->user->lang['SHOUT_ACTION_MSG'] . $tpl_close. '<br /><br />' : '';
						
						$url_message = ($row['foe'] == 1) ? $tpl_open . $this->user->lang['SHOUT_USER_IGNORE'] . $tpl_close : $url_message;
						
						$url_profile = $tpl_open. '<a href="' .$this->construct_shout_url(append_sid("{$this->root_path}memberlist.{$this->php_ext}", "mode=viewprofile&amp;u=$id", false)). '" onclick="this.target=\'_blank\';" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_ACTION_PROFIL']. ' ' .$this->user->lang['FROM']. ' ' .$row['username']. '">' .$this->user->lang['SHOUT_ACTION_PROFIL'] . $tpl_close;
						
						$url_cite	= $tpl_open. '<a style="cursor:pointer;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" onclick="citeMsg();return false;" title="' .$this->user->lang['SHOUT_ACTION_CITE_EXPLAIN']. '">' .$this->user->lang['SHOUT_ACTION_CITE'] . $tpl_close;
						
						$url_admin 	= $this->auth->acl_get('a_user') ? $tpl_open. '<a href="' .$this->construct_shout_url(append_sid("{$adm_path}index.{$this->php_ext}", "i=users&amp;mode=overview&amp;u=$id", true, $this->user->session_id)). '" onclick="this.target=\'_blank\';" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_USER_ADMIN']. '">' .$this->user->lang['SHOUT_USER_ADMIN'] . $tpl_close : '';
						
						$url_modo 	= $this->auth->acl_get('m_') ? $tpl_open. '<a href="' .$this->construct_shout_url(append_sid("{$this->root_path}mcp.{$this->php_ext}", "i=notes&amp;mode=user_notes&amp;u=$id", true, $this->user->session_id)). '" onclick="this.target=\'_blank\';" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_ACTION_MCP']. '">' .$this->user->lang['SHOUT_ACTION_MCP'] . $tpl_close : '';
						
						$url_ban 	= ($this->auth->acl_get('m_ban') && $go_founder) ? $tpl_open. '<a href="' .$this->construct_shout_url(append_sid("{$this->root_path}mcp.{$this->php_ext}", "i=ban&amp;mode=user&amp;u=$id", true, $this->user->session_id)). '" onclick="this.target=\'_blank\';" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_ACTION_BAN']. '">' .$this->user->lang['SHOUT_ACTION_BAN'] . $tpl_close : '';
						
						$url_remove	= (($this->auth->acl_get('a_') || $this->auth->acl_get('m_shout_delete')) && $go_founder) ? '<br /><br />' .$tpl_open. '<a style="cursor:pointer;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" onclick="if(confirm(\'' .$this->user->lang['SHOUT_ACTION_REMOVE_EXPLAIN']. '\'))removeMsg(' .$id. ',document.getElementById(\'user_inp_sort\').value);return false;" title="' .$this->user->lang['SHOUT_ACTION_REMOVE']. '">' .$this->user->lang['SHOUT_ACTION_REMOVE'] . $tpl_close : '';
						
						$url_perso	= (($this->auth->acl_get('a_') || $this->auth->acl_get('m_shout_personal')) && $go_founder) ? '<br /><br />' .$tpl_open. '<a style="cursor:pointer;" onclick="changePerso('.$id.');return false;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_ACTION_PERSO']. '">' .$this->user->lang['SHOUT_ACTION_PERSO'] . $tpl_close : '';
						
						$url_del_to = $inp ? $tpl_open. '<a style="cursor:pointer;" onclick="if(confirm(\'' .$this->user->lang['SHOUT_ACTION_DEL_TO_EXPLAIN']. '\'))delReqTo(' .$userid.',document.getElementById(\'user_inp_sort\').value);return false;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_ACTION_DEL_TO']. '">' .$this->user->lang['SHOUT_ACTION_DEL_TO'] . $tpl_close. '<br /><br />' : '';
						
						$url_del 	= $inp ? $tpl_open. '<a style="cursor:pointer;" onclick="if(confirm(\'' .$this->user->lang['SHOUT_ACTION_DELETE_EXPLAIN']. '\'))delReq(' .$userid. ',document.getElementById(\'user_inp_sort\').value);return false;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .$this->user->lang['SHOUT_ACTION_DELETE']. '">' .$this->user->lang['SHOUT_ACTION_DELETE'] . $tpl_close : '';
						
						$url_robot 	= ($this->auth->acl_get('a_') || $this->auth->acl_get('m_shout_robot')) ? '<br /><br />' .$tpl_open. '<a style="cursor:pointer;" onclick="iH(\'shout_avatar\',\'\');robotMsg(' .$s. ');return false;" onmouseover="iH(\'onText\',this.title);" onmouseout="iH(\'onText\',\'\');" title="' .sprintf($this->user->lang['SHOUT_ACTION_MSG_ROBOT'], $this->user->lang['SHOUT_ROBOT']). '">' .sprintf($this->user->lang['SHOUT_ACTION_MSG_ROBOT'], $robot) . $tpl_close : '';
						
						$retour 	= ($url_admin || $url_modo || $url_ban) ? '<br /><br />' : '';
						
						$content 	= $id. '||' .$username. '||' .$url_message.$url_profile.$url_cite . $retour . $url_admin . $url_modo . $url_ban . $url_remove.$url_perso. '<br /><hr class="dotted" />' .$url_del_to . $url_del . $url_robot. '||' .$avatar;
					}
					$this->db->sql_freeresult($result);
				}
				$response->send($content);
			break;

			case 'action_post':
				$go = $personal = $username = false; // Important!
				if ($this->auth->acl_get('u_shout_post_inp') || $this->auth->acl_get('m_shout_robot') || $this->auth->acl_get('a_') || $this->auth->acl_get('m_')) // Administrators and moderators can always post personnal messages
				{
					$message = utf8_normalize_nfc($this->request->variable('message', '', true));
					$row = array();
					$pr = $this->request->variable('pr', 0);
					switch ($pr)
					{
						case 1:
						case 2:
							$_table = $this->shoutbox_table;
							$priv = '';
						break;
						case 3:
							$_table = $this->shoutbox_priv_table;
							$priv = '_priv';
						break;
					}
					if (!$id) // any user id
					{
						$content = '';
					}
					else if ($id == 1) // post a robot message
					{
						if ($this->auth->acl_get('a_') || $this->auth->acl_get('m_shout_robot')) // let's go
						{
							$username = get_username_string('no_profile', 0, $this->user->lang['SHOUT_ROBOT'], $this->config['shout_color_robot']);
							$personal = false;
							$go = true;
							$id = $userid = 0;
						}
						else // no perm, out...
						{
							$content = '';
						}
					}
					else if ($id > 1) // post a personal message
					{
						$sql = $this->db->sql_build_query('SELECT', array(
							'SELECT'    => 'user_id, user_type, username, user_colour',
							'FROM'      => array(USERS_TABLE => ''),
							'WHERE'		=> 'user_id = ' .(int)$id,
						));
						$result = $this->db->sql_query_limit($sql, 1);
						$row = $this->db->sql_fetchrow($result);
						if (!$row || $row['user_type'] == USER_IGNORE) // user id don't exist or ignore
						{
							$content = '';
						}
						else // let's go
						{
							$username = get_username_string('full', $row['user_id'], $row['username'], $row['user_colour'], false, $this->shoutbox->real_board_url(true));
							$personal = true;
							$go = true;
						}
						$this->db->sql_freeresult($result);
					}
					if ($go)
					{
						$message = $this->shoutbox->parse_shout_message($message);
						if (!$message)
						{
							$this->shoutbox->shout_error('MESSAGE_EMPTY');
							break;
						}
						// Personalize message
						if ($this->user->data['shout_bbcode'] && $this->auth->acl_get('u_shout_bbcode_change') && $id != 0)
						{
							$message = $this->shoutbox->personalize_shout_message($message, $this->user->data['shout_bbcode']);
						}

						// BBCodes must be formated here to be modified just after
						$message = $this->shoutbox->bbcode_shout_message($message, $personal, $row);
						$uid = $bitfield = $options = ''; // will be modified by generate_text_for_storage
						$allow_bbcode 	= ($this->auth->acl_get('u_shout_bbcode')) ? true : false;
						$allow_smilies 	= ($this->auth->acl_get('u_shout_smilies')) ? true : false;
						generate_text_for_storage($message, $uid, $bitfield, $options, $allow_bbcode, true, $allow_smilies);
						$sql_ary = array(
							'shout_text'				=> (string)$message,
							'shout_bbcode_uid'			=> $uid,
							'shout_bbcode_bitfield'		=> $bitfield,
							'shout_bbcode_flags'		=> $options,
							'shout_time'				=> (int)time(),
							'shout_user_id'				=> (int)$userid,
							'shout_ip'					=> (string)$this->user->ip,
							'shout_robot'				=> 0,
							'shout_forum'				=> 0,
							'shout_inp'					=> (int)$id,
						);
						$sql = 'INSERT INTO ' .$_table. ' ' .$this->db->sql_build_array('INSERT', $sql_ary);
						$this->db->sql_query($sql);
						set_config_count('shout_nr' .$priv, 1, true);
						$content = '1||';
					}
				}
				else
				{
					$content = '';
				}
				$response->send($content);
			break;

			case 'action_del':
				$pr = $this->request->variable('pr', 0);
				switch ($pr)
				{
					case 0:
						$_table = $this->shoutbox_table;
						$priv = '';
						$sort = false;
					break;
					case 1:
						$_table = $this->shoutbox_priv_table;
						$priv = '_priv';
						$sort = true;
					break;
				}
				if ($id != $userid)
				{
					$content = '0||' .$this->user->lang['SERVER_ERR'];
				}
				else
				{
					// Delete all personnal messages of this user
					$sql = 'DELETE FROM ' .$_table. ' 
						WHERE shout_user_id = ' .$userid. ' 
							AND shout_inp <> 0';
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					if (!$deleted)
					{
						$content = '1||' .$this->user->lang['SHOUT_ACTION_DEL_NO'];
					}
					else
					{
						$this->shoutbox->update_shout_messages($sort);
						set_config_count('shout_del_user' .$priv, $deleted, true);
						$content = '1||' .$this->user->lang['SHOUT_ACTION_DEL_REP']. ' ' .sprintf($this->user->lang['NUMBER_MESSAGE' .(($deleted > 1) ? 'S' : '')], $deleted);
					}
				}
				$response->send($content);
			break;
			
			case 'action_del_to':
				$pr = $this->request->variable('pr', 0);
				switch ($pr)
				{
					case 0:
						$_table = $this->shoutbox_table;
						$priv = '';
						$sort = false;
					break;
					case 1:
						$_table = $this->shoutbox_priv_table;
						$priv = '_priv';
						$sort = true;
					break;
				}
				if ($id != $userid)
				{
					$content = '0||' .$this->user->lang['SERVER_ERR'];
				}
				else
				{
					// Delete all personnal messages to this user
					$sql = 'DELETE FROM ' .$_table. ' 
						WHERE shout_inp = ' .$userid. ' 
							AND shout_user_id <> ' .$userid;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					if (!$deleted)
					{
						$content = '1||' .$this->user->lang['SHOUT_ACTION_DEL_NO'];
					}
					else
					{
						$this->shoutbox->update_shout_messages($sort);
						set_config_count('shout_del_user' .$priv, $deleted, true);
						$content = '1||' .$this->user->lang['SHOUT_ACTION_DEL_REP']. ' ' .sprintf($this->user->lang['NUMBER_MESSAGE' .(($deleted > 1) ? 'S' : '')], $deleted);
					}
				}
				$response->send($content);
			break;

			case 'action_remove':
				$pr = $this->request->variable('pr', 0);
				switch ($pr)
				{
					case 0:
						$_table = $this->shoutbox_table;
						$priv = '';
						$sort = false;
					break;
					case 1:
						$_table = $this->shoutbox_priv_table;
						$priv = '_priv';
						$sort = true;
					break;
				}
				if ($this->auth->acl_get('a_') || $this->auth->acl_get('m_shout_delete'))
				{
					// Delete all messages of this user
					$sql = 'DELETE FROM ' .$_table. ' 
						WHERE shout_user_id = ' .$id. ' 
						OR shout_robot_user = ' .$id. ' 
						OR shout_inp = ' .$id;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					if (!$deleted)
					{
						$content = '1||' .$this->user->lang['SHOUT_ACTION_REMOVE_NO'];
					}
					else
					{
						$this->shoutbox->update_shout_messages($sort);
						set_config_count('shout_del_user' .$priv, $deleted, true);
						$content = '1||' .$this->user->lang['SHOUT_ACTION_REMOVE_REP']. ' ' .sprintf($this->user->lang['NUMBER_MESSAGE' .(($deleted > 1) ? 'S' : '')], $deleted);
					}
				}
				else
				{
					$content = '0||' .$this->user->lang['SERVER_ERR'];
				}
				$response->send($content);
			break;

			case 'delete':
			case 'delete_priv':
				$post = $this->request->variable('post', 0);
				if (!$post)
				{
					$this->shoutbox->shout_error('NO_SHOUT_ID');
					return;
				}
				else if ($userid == ANONYMOUS)
				{
					$this->shoutbox->shout_error('NO_DELETE_PERM');
					return;
				}
				else
				{
					switch ($mode)
					{
						case 'delete':
							$sort = false;
							$_priv = $_Priv = '';
							$auth_shout = 'manage';
							$_table = $this->shoutbox_table;
						break;
						case 'delete_priv':
							$sort = true;
							$_priv = '_priv';
							$_Priv = '_PRIV';
							$auth_shout = 'priv';
							$_table = $this->shoutbox_priv_table;
						break;
					}
					// If someone can delete all messages, he can delete it's messages :)
					$can_delete_mod = ($this->auth->acl_get('m_shout_delete') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? true : false;
					$can_delete 	= $can_delete_mod ? true : $this->auth->acl_get('u_shout_delete_s');
					
					$sql = 'SELECT shout_user_id 
						FROM ' .$_table. ' 
						WHERE shout_id = ' .$post;
					$result = $this->db->sql_query_limit($sql, 1);
					if (!$result)
					{
						$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
						break;
					}
					$on_id = $this->db->sql_fetchfield('shout_user_id');
					$this->db->sql_freeresult($result);
					
					if (!$can_delete && ($userid == $on_id))
					{
						$this->shoutbox->shout_error('NO_DELETE_PERM_S');
						return;
					}
					else if (!$can_delete_mod && $can_delete && ($userid != $on_id))
					{
						$this->shoutbox->shout_error('NO_DELETE_PERM_T');
						return;
					}
					else if (!$can_delete)
					{
						$this->shoutbox->shout_error('NO_DELETE_PERM');
						return;
					}
					else if ($can_delete && ($userid == $on_id) || $can_delete_mod)
					{
						// Lets delete this post :D
						$sql = 'DELETE FROM ' .$_table. ' 
							WHERE shout_id = ' .$post;
						$this->db->sql_query($sql);
						
						$this->shoutbox->update_shout_messages($sort, $post);
						set_config_count('shout_del_user' .$_priv, 1, true);
						$content = 'delete||1';
					}
					else
					{
						$this->shoutbox->shout_error('NO_DELETE_PERM');
						return;
					}
				}
				$response->send($content);
			break;

			case 'purge':
			case 'purge_priv':
				if (!$this->auth->acl_get('m_shout_purge') && !$this->auth->acl_get('a_shout_manage'))
				{
					$this->shoutbox->shout_error('NO_PURGE_PERM');
					return;
				}
				else
				{
					switch ($mode)
					{
						case 'purge':
							$on_priv = false;
							$_priv = $_Priv = '';
							$_table = $this->shoutbox_table;
						break;
						case 'purge_priv':
							$on_priv = true;
							$_priv = '_priv';
							$_Priv = '_PRIV';
							$_table = $this->shoutbox_priv_table;
						break;
					}
					$sql = 'DELETE FROM ' .$_table;
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					
					set_config_count('shout_del_purge' .$_priv, $deleted, true);
					$this->shoutbox->post_robot_shout($userid, $this->user->ip, $on_priv, true, false, false, false);
					$content = $mode. '||' .$deleted;
				}
				$response->send($content);
			break;
			
			case 'purge_robot':
			case 'purge_robot_priv':
				if (!$this->auth->acl_get('m_shout_purge') && !$this->auth->acl_get('a_shout_manage'))
				{
					$this->shoutbox->shout_error('NO_PURGE_ROBOT_PERM');
					return;
				}
				else
				{
					$purge_sort = $this->request->variable('purge_sort', '');
					switch ($mode)
					{
						case 'purge_robot':
							$on_priv = false;
							$_priv = $_Priv = '';
							$_table = $this->shoutbox_table;
						break;
						case 'purge_robot_priv':
							$on_priv = true;
							$_priv = '_priv';
							$_Priv = '_PRIV';
							$_table = $this->shoutbox_priv_table;
						break;
					}
					$this->config['shout_robot_choice'.$_priv] = $this->config['shout_robot_choice'.$_priv]. ', 4';
					$sort_on = explode(', ', $this->config['shout_robot_choice'.$_priv]);
					
					$sql = 'DELETE FROM ' .$_table. ' 
						WHERE ' .$this->db->sql_in_set('shout_info', $sort_on, false, true);
					$this->db->sql_query($sql);
					$deleted = $this->db->sql_affectedrows();
					
					set_config_count('shout_del_purge' .$_priv, $deleted, true);
					$this->shoutbox->post_robot_shout($userid, $this->user->ip, $on_priv, true, true, false, false);
					
					$sql = 'SELECT COUNT(shout_id) as nr 
						FROM ' .$_table;
					$result = $this->db->sql_query($sql);
					$nr = $this->db->sql_fetchfield('nr');
					$this->db->sql_freeresult($result);
					$content = $purge_sort. '||' .$nr;
				}
				$response->send($content);
			break;
			
			case 'post':
			case 'edit':	
			case 'post_priv':
			case 'edit_priv':
				$shout_id = $this->request->variable('shout_id', 0);
				$mode_s = str_replace(array('_priv', '_pop'), '', $mode);

				switch ($mode)
				{
					case 'post':
					case 'edit':
						$sort = false;
						$perm = '_view';
						$_priv = $_Priv = '';
						$auth_shout = 'manage';
						$_table = $this->shoutbox_table;
					break;
					case 'post_priv':
					case 'edit_priv':
						$sort = true;
						$_priv = $perm = '_priv';
						$_Priv = '_PRIV';
						$auth_shout = 'priv';
						$_table = $this->shoutbox_priv_table;
					break;
				}
				
				$uid = $bitfield = $options = ''; // will be modified by generate_text_for_storage
				$allow_urls = true;
				
				// Protect by checking permissions
				$allow_bbcode 	= $this->auth->acl_get('u_shout_bbcode') ? true : false;
				$allow_smilies 	= $this->auth->acl_get('u_shout_smilies') ? true : false;
				// If someone can edit all messages, he can edit it's messages :) (if errors in permissions set)
				$can_edit_mod = ($this->auth->acl_get('m_shout_edit_mod') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? true : false;
				$can_edit = $can_edit_mod ? true : $this->auth->acl_get('u_shout_edit');
				
				if (!$this->auth->acl_get('u_shout' .$perm))
				{
					$this->shoutbox->shout_error('NO_VIEW' .$_Priv. '_PERM');
					break;
				}
				else if (!$this->auth->acl_get('u_shout_post'))
				{
					$this->shoutbox->shout_error('NO_POST_PERM');
					break;
				}
				else if ($mode_s == 'edit' && !$shout_id)
				{
					$this->shoutbox->shout_error('NO_SHOUT_ID');
					break;
				}
				else if ($mode_s == 'edit' && !$can_edit || $mode_s == 'edit' && $this->user->data['user_id'] == ANONYMOUS)
				{
					$this->shoutbox->shout_error('NO_EDIT_PERM');
					break;
				}
				else
				{
					// Flood control
					$current_time = time();
					if (!$this->auth->acl_get('u_shout_ignore_flood') && $mode_s == 'post') // Not in private
					{
						$sql = $this->db->sql_build_query('SELECT', array(
							'SELECT'    => 'MAX(shout_time) AS last_post_time',
							'FROM'      => array($_table => ''),
							'WHERE'		=> ($userid == ANONYMOUS) ? "shout_ip = '" .$this->db->sql_escape((string)$this->user->ip). "'" : 'shout_user_id = ' .$userid,
						));
						if ($result = $this->db->sql_query($sql))
						{
							if ($row = $this->db->sql_fetchrow($result))
							{
								if ($row['last_post_time'] > 0 && ($current_time - $row['last_post_time']) < $this->config['shout_flood_interval'])
								{
									$this->db->sql_freeresult($result);
									$this->shoutbox->shout_error('FLOOD_ERROR');
									break;
								}
							}
							$this->db->sql_freeresult($result);
						}
						else
						{
							$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
							break;
						}
					}
					
					if ($mode_s == 'edit')
					{
						$ok_edit = false;
						// We need to be sure its this users his shout.
						$sql = 'SELECT shout_user_id 
							FROM ' .$_table. ' 
							WHERE shout_id = ' .$shout_id;
						$result = $this->db->sql_query_limit($sql, 1);
						if (!$result)
						{
							$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
							break;
						}
						$on_id = $this->db->sql_fetchfield('shout_user_id');
						$this->db->sql_freeresult($result);
						
						if (!$can_edit_mod)  // If not able to edit all messages
						{
							// Not his shout, display error
							if (!$on_id || $on_id != $userid)
							{
								$this->shoutbox->shout_error('NO_EDIT_PERM');
								break;
							}
							else
							{
								$ok_edit = true;
							}
						}
						else
						{
							$ok_edit = true;
						}
					}
					
					$message = utf8_normalize_nfc($this->request->variable('chat_message', '', true));
					$cite = utf8_normalize_nfc($this->request->variable('cite', '', true));
					
					// First verification of empty message
					if ($message == '')
					{
						$this->shoutbox->shout_error('MESSAGE_EMPTY');
						break;
					}
					
					// Don't parse img if unautorised and return img url only
					if ((strpos($message, '[img]') !== false) && (strpos($message, '[/img]') !== false) && !$this->auth->acl_get('u_shout_image'))
					{
						$_message = str_replace(array('[img]', '[/img]'), '', $message);
					}
					// Multi protections at this time...
					$message = $this->shoutbox->parse_shout_message($message, (($_priv == '_priv') ? true : false), $mode_s);
					if (strpos($message, '{errors}') !== false)
					{
						$this->shoutbox->shout_error($message);
						break;
					}
					if (!$message)
					{
						$this->shoutbox->shout_error('MESSAGE_EMPTY');
						break;
					}
					// Personalize message
					if ($userid != ANONYMOUS && $this->user->data['shout_bbcode'] && $this->auth->acl_get('u_shout_bbcode_change'))
					{
						if ($mode_s == 'post' || ($mode_s == 'edit') && ($on_id == $userid))
						{
							$message = $this->shoutbox->personalize_shout_message($message, $this->user->data['shout_bbcode']);
						}
					}
					
					if (($cite != '') && ($userid != ANONYMOUS))
					{
						$message = $this->user->lang['SHOUT_ACTION_CITE_ON'] . $cite . ':' . $message;
						$allow_bbcode = true; // We must turn it to true here
					}
					
					generate_text_for_storage($message, $uid, $bitfield, $options, $allow_bbcode, $allow_urls, $allow_smilies);
					
					// Second verification of empty message (kill hidden caracters)
					$message == str_replace(array('  ', '   ', '    '), ' ', $message);
					if ($message == '' || $message == ' ')
					{
						$this->shoutbox->shout_error('MESSAGE_EMPTY');
						break;
					}
					if ($userid == ANONYMOUS) // Select a username for anonymous
					{
						$username = $this->db->sql_escape(str_replace(array("'", '"'), '&#8242;', $this->request->variable($this->config['cookie_name'] . '_shout-name', '', true, \phpbb\request\request_interface::COOKIE)));
						if (!$username) // If no, return an error
						{
							$this->shoutbox->shout_error('SHOUT_CHOICE_NAME_RETURN');
							break;
						}
						$username = $username. ':' .$this->shoutbox->random_ip($this->user->ip); // If yes, add a random number from ip
						$message = $username. '|||' .$message;
					}
					$sql_ary = array(
						'shout_text'				=> (string)$message,
						'shout_bbcode_uid'			=> $uid,
						'shout_bbcode_bitfield'		=> $bitfield,
						'shout_bbcode_flags'		=> $options,
					);
					if ($mode_s == 'edit')
					{
						if (!$ok_edit) // !Important second verification
						{
							$this->shoutbox->shout_error('NO_EDIT_PERM');
							break;
						}
						else // let's do it
						{
							$sql = 'UPDATE ' .$_table. '
								SET ' .$this->db->sql_build_array('UPDATE', $sql_ary). ' 
								WHERE shout_id = ' .$shout_id;
							$this->db->sql_query($sql);
							$this->shoutbox->update_shout_messages($sort); // For reload the message to all
						}
					}
					else
					{
						$sql_ary = array_merge($sql_ary, array(
							'shout_time'		=> (int)time(),
							'shout_user_id'		=> (int)$userid,
							'shout_ip'			=> (string)$this->user->ip,
							'shout_robot'		=> 0,
							'shout_forum'		=> 0,
						));
						
						$sql = 'INSERT INTO ' .$_table. ' ' .$this->db->sql_build_array('INSERT', $sql_ary);
						set_config_count('shout_nr' .$_priv, 1, true);
					}
					if (!$this->db->sql_query($sql)) 
					{
						$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
						break;
					}
					if ($this->config['shout_on_cron' .$_priv])
					{
						if ($this->config['shout_max_posts' .$_priv] > 0)
						{
							$this->shoutbox->delete_shout_posts(($_priv == '_priv') ? true : false);
						}
					}
					$content = $mode. '||' .$this->user->lang['POSTED'];
				}
				$response->send($content);
			break;

			case 'check':
			case 'check_pop':
			case 'check_priv':
				switch ($mode)
				{
					case 'check':
					case 'check_pop':
						$perm = '_view';
						$_priv = $_Priv = '';
						$_table = $this->shoutbox_table;
					break;
					case 'check_priv':
						$_priv = $perm = '_priv';
						$_Priv = '_PRIV';
						$_table = $this->shoutbox_priv_table;
					break;
				}
				// !Important, always read the good permission before
				if (!$this->auth->acl_get('u_shout' .$perm))
				{
					$this->shoutbox->shout_error('NO_VIEW' .$_Priv. '_PERM');
					break;
				}
				
				// Say hello Mr Robot :-)
				if ($this->config['shout_enable_robot'] && date('H') == $this->config['shout_cron_hour'])
				{
					if ($this->config['shout_cron_run'] != date('d-m-Y'))
					{
						if ($this->config['shout_hello'] || $this->config['shout_hello_priv'])
						{
							$this->shoutbox->hello_robot_shout();
							set_config('shout_cron_run', date('d-m-Y'), true);
						}
					}
				}

				// Read the forums permissions
				$sql_where = ($this->auth->acl_getf_global('f_read')) ? '(' .$this->db->sql_in_set('shout_forum', array_keys($this->auth->acl_getf('f_read', true)), false, true). ' OR shout_forum = 0)' : 'shout_forum = 0';
				// count personal messages if needed
				$sql_and = ($userid == ANONYMOUS || $this->user->data['is_bot']) ? " AND shout_inp = 0" : " AND shout_inp = 0 OR (shout_inp = $userid OR shout_user_id = $userid)";
				
				$sql = $this->db->sql_build_query('SELECT', array(
					'SELECT'    => 'shout_time',
					'FROM'      => array($_table => ''),
					'WHERE'		=> $sql_where . $sql_and,
					'ORDER_BY'	=> 'shout_time DESC',
				));
				$result = $this->db->sql_query_limit($sql, 1);
				$time = $this->db->sql_fetchfield('shout_time');
				$this->db->sql_freeresult($result);
				if (!$result)
				{
					$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
					break;
				}
				else
				{
					// Reload the shoutbox to display correct minutes ago
					if (!$this->user->data['is_bot'] && $userid != ANONYMOUS)
					{
						// Only if user have choose that, not the sound...
						$correct = explode('||', $this->user->data['user_shout']);
						if ($correct[0] == 1)
						{
							// And only for users with shout format date like "a minute ago"
							$shout_dateformat = explode(';', $this->user->data['user_shoutbox']);
							if (strpos($shout_dateformat[7], '|') !== false)
							{
								// Do this only for messages < 1 hour + 5 secondes to pass the hour...
								if ($time + 3605 > time())
								{
									$last = 1;
								}
							}
						}
					}
					$time = substr($time, 7, 10);
					$content = $time. '|' .(int)($time != $last);
				}
				$response->send($content);
			break;

			case 'number':
			case 'number_pop':
			case 'number_priv':
				switch ($mode)
				{
					case 'number':
					case 'number_pop':
						$perm = '_view';
						$_priv = $_Priv = '';
						$_table = $this->shoutbox_table;
					break;
					case 'number_priv':
						$_priv = $perm = '_priv';
						$_Priv = '_PRIV';
						$_table = $this->shoutbox_priv_table;
					break;
				}
				if (!$this->auth->acl_get('u_shout' .$perm))
				{
					$this->shoutbox->shout_error('NO_VIEW' .$_Priv. '_PERM');
					break;
				}
				else
				{
					// Read the forums permissions
					$sql_where = $this->auth->acl_getf_global('f_read') ? '(' .$this->db->sql_in_set('shout_forum', array_keys($this->auth->acl_getf('f_read', true)), false, true). ' OR shout_forum = 0)' : 'shout_forum = 0';
					// count personal messages if needed
					$sql_and = ($userid == ANONYMOUS || $this->user->data['is_bot']) ? ' AND shout_inp = 0' : ' AND shout_inp = 0 OR (shout_inp = ' .$userid. ' OR shout_user_id = ' .$userid. ')';
					
					$sql = $this->db->sql_build_query('SELECT', array(
						'SELECT'    => 'COUNT(shout_id) as nr',
						'FROM'      => array($_table => ''),
						'WHERE'		=> $sql_where . $sql_and,
					));
					$result = $this->db->sql_query($sql);
					if (!$result)
					{
						$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
						return;
					}
					$nb = (int)$this->db->sql_fetchfield('nr');
					$this->db->sql_freeresult($result);
					if (!$nb)
					{
						$this->shoutbox->shout_error('NO_MESSAGE');
						break;
					}
					else
					{
						// Limit the number of messages to display
						$nb = ($this->config['shout_max_posts_on' .$_priv] > 0) ? (($nb > $this->config['shout_max_posts_on' .$_priv]) ? $this->config['shout_max_posts_on' .$_priv] : $nb) : $nb;
						$content = $nb. '|';
					}
				}
				$response->send($content);
			break;
			
			case 'view':
			case 'view_pop':
			case 'view_priv':
				$i = 0;
				$data = '';
				$return = '';
				$start = $this->request->variable('start', 0);
				switch ($mode)
				{
					case 'view'; // Normal shoutbox
						$perm = '_view';
						$sort = $_priv = $_Priv = '';
						$auth_shout = 'manage';
						$_table = $this->shoutbox_table;
					break;
					case 'view_pop'; // Popup shoutbox
						$sort = '_pop';
						$perm = '_view';
						$_priv = $_Priv = '';
						$auth_shout = 'manage';
						$_table = $this->shoutbox_table;
					break;
					case 'view_priv'; // Private shoutbox
						$sort = $_priv = $perm = '_priv';
						$_Priv = '_PRIV';
						$auth_shout = 'priv';
						$_table = $this->shoutbox_priv_table;
					break;
				}
				
				// Permissions verification
				if (!$this->auth->acl_get('u_shout' .$perm))
				{
					$this->shoutbox->shout_error('NO_VIEW' .$_Priv. '_PERM');
				}
				
				// See compatibles browsers to change the display
				$nr_compatible = $this->shoutbox->compatibles_browsers();
				
				// Adjust number of messages to display
				$shout_number = ($nr_compatible > 2) ? $this->config['shout_non_ie_nr' .$sort] : $this->config['shout_ie_nr' .$sort];
				
				// Display avatars ?
				$see_avatar = ($this->user->optionget('viewavatars') && $this->config['shout_avatar'] && $this->config['allow_avatar'] && $nr_compatible != 2) ? true : false;
				
				// Prevents some errors for the allocation of permissions
				// If someone can edit all messages, he can edit its own messages :)
				$can_edit_mod 	= ($this->auth->acl_get('m_shout_edit_mod') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? true : false;
				$can_edit 		= $can_edit_mod ? true : $this->auth->acl_get('u_shout_edit');
				
				// If someone can view all ip, he can view its own ip :)
				$can_info_mod 	= ($this->auth->acl_get('m_shout_info') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? true : false;
				$can_info 		= $can_edit_mod ? true : $this->auth->acl_get('u_shout_info_s');
				
				// If someone can delete all messages, he can delete its own messages :)
				$can_delete_mod = ($this->auth->acl_get('m_shout_delete') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? true : false;
				$can_delete 	= $can_delete_mod ? true : $this->auth->acl_get('u_shout_delete_s');
				
				// Read the forums permissions
				$sql_where = $this->auth->acl_getf_global('f_read') ? $this->db->sql_in_set('s.shout_forum', array_keys($this->auth->acl_getf('f_read', true)), false, true). ' OR s.shout_forum = 0' : 's.shout_forum = 0';

				// Add personal messages if needed
				$sql_and = ($userid == ANONYMOUS || $this->user->data['is_bot']) ? ' AND s.shout_inp = 0' : ' AND s.shout_inp = 0 OR (s.shout_inp = ' .$userid. ' OR s.shout_user_id = ' .$userid. ')';
				
				$sql = $this->db->sql_build_query('SELECT', array(
					'SELECT'    => 's.*, u.user_id, u.username, u.user_colour, u.user_avatar, u.user_avatar_type, u.user_avatar_width, u.user_avatar_height, u.user_type',
					'FROM'      => array(
						$_table => 's'
					),
					'LEFT_JOIN'	=> array(
						array(
							'FROM'	=> array(USERS_TABLE => 'u'),
							'ON'	=> 'u.user_id = s.shout_user_id'
						)
					),
					'WHERE'		=> $sql_where . $sql_and,
					'ORDER_BY'	=> 's.shout_id DESC',
				));
				$result = $this->db->sql_query_limit($sql, $shout_number, $start);
				if (!$result)
				{
					$this->shoutbox->shout_sql_error($sql, __LINE__, __FILE__);
					return;
				}
				else
				{
					while ($row = $this->db->sql_fetchrow($result))
					{
						$data .= $i. '|+|';
						$row['edit'] = $row['delete'] = $row['show_ip'] = $row['avatar'] = '0';
						if (($userid != ANONYMOUS) && !$this->user->data['is_bot'])
						{
							$explode = explode(';', $this->user->data['user_shoutbox']);
							$shout_dateformat = $explode[7];
							$shout_dateformat = ($shout_dateformat != 'N') ? $explode[7] : $this->config['shout_dateformat'];
						}
						else
						{
							$shout_dateformat = $this->config['shout_dateformat'];
						}
						
						if ($row['shout_user_id'] == ANONYMOUS)
						{
							$shout_dateformat = $this->config['shout_dateformat'];
							list($row['username'], $row['shout_text']) = explode('|||', $row['shout_text']);
							$row['shout_text'] = str_replace($row['username'].'|||', '', $row['shout_text']);
						}
						$row['avatar'] = '';
						if ($see_avatar)
						{
							if ($row['shout_user_id'] == 0 || $row['shout_user_id'] == ANONYMOUS)
							{
								$row['avatar'] = $this->shoutbox->shout_user_avatar($row['user_id']);
							}
							else if (!$row['user_avatar'])
							{
								$row['avatar'] = $this->shoutbox->shout_user_avatar($row['user_id'], $row);
							}
							else
							{
								$_row = array();
								$_row['user_id'] = $row['user_id'];
								$_row['username'] = $row['username'];
								$_row['user_avatar'] = $row['user_avatar'];
								$_row['user_avatar_type'] = $row['user_avatar_type'];
								$_row['user_avatar_width'] = $row['user_avatar_width'];
								$_row['user_avatar_height'] = $row['user_avatar_height'];
								$row['avatar'] = $this->shoutbox->shout_user_avatar($row['user_id'], $_row);
								$row['avatar'] = str_replace(array('./../../../../', './../../../', './../../', './../', './'), generate_board_url(). '/', $row['avatar']);
							}
						}
						$row['username'] 	= ($row['shout_user_id'] == 0) ? $this->user->lang['SHOUT_ROBOT'] : htmlspecialchars_decode($row['username']);
						$row['user_colour'] = ($row['shout_user_id'] == 0) ? $this->config['shout_color_robot'] : $row['user_colour'];
						
						$row['shout_time']	= $this->user->format_date($row['shout_time'], $shout_dateformat);
						$row['msg_plain']   = '0'; // It will be replaced if user can edit ;).
						$row['username'] = $this->shoutbox->construct_action_shout($row['username'], $row['shout_user_id'], $row['user_colour']);
						if ($row['shout_user_id'] != ANONYMOUS && $userid != ANONYMOUS && !$this->user->data['is_bot'])
						{
							if ($row['shout_inp'])
							{
								if ($row['shout_inp'] != $userid && $row['shout_user_id'] != $userid) // This affects private message to both users concerned
								{
									continue;
								}
								$row['shout_text'] = str_replace('SHOUT_USER_POST', $this->user->lang['SHOUT_USER_POST'], $row['shout_text']);
							}
						}
						else if ($row['shout_inp']) // No permission to see it, continue...
						{
							continue;
						}
	
						// Checks permissions for delete, show_ip and edit
						if ($userid != ANONYMOUS && !$this->user->data['is_bot'])
						{
							if ($can_delete_mod || ($row['user_id'] == $userid) && $can_delete)
							{
								$row['delete'] = true;
							}
							if ($can_info_mod || ($row['user_id'] == $userid) && $can_info)
							{
								$row['show_ip'] = true;
							}
							if ($can_edit_mod || ($row['user_id'] == $userid) && $can_edit)
							{
								$row['edit'] = true;
								$row['msg_plain'] = $row['shout_text'];
								if ($row['shout_user_id'] == ANONYMOUS)
								{
									$row['msg_plain'] = $row['username'].'|||'.$row['msg_plain'];
								}
								decode_message($row['msg_plain'], $row['shout_bbcode_uid']);
							}
						}
						// Triple protect this information...
						$row['is_ip'] = '0';
						if (!$row['show_ip'])
						{
							$row['show_ip'] = '0';
							$row['shout_ip'] = $row['is_ip'] = $this->user->ip;
						}
						// Construct the info robots with translation
						if ($row['shout_info'] > 0)
						{
							$row['shout_text'] = $this->shoutbox->display_infos_robot($row['shout_info'], $row['shout_text']);
						}
						else
						{
							$row['shout_text'] = generate_text_for_display($row['shout_text'], $row['shout_bbcode_uid'], $row['shout_bbcode_bitfield'], $row['shout_bbcode_flags']);//$this->shoutbox->shout_url_free_sid()
						}
						
						// Active external links for all links in popup and private shoutbox
						// Only target="_blank"' work correctly in javascript...
						if ($mode != 'view')
						{
							// all urls
							if (preg_match('/class=\"postlink\"/i', $row['shout_text']))
							{
								$row['shout_text'] 	= str_replace('class="postlink', 'target="_blank" class="postlink', $row['shout_text']);
							}
							else
							{
								$row['shout_text'] 	= str_replace('a href="', 'a target="_blank" href="', $row['shout_text']);
							}
							// but not for user action
							$row['shout_text'] 	= str_replace('a target="_blank" href="javascript', 'a href="javascript', $row['shout_text']);
						}
						else
						{
							if (preg_match('/class=\"postlink\"/i', $row['shout_text']))
							{
								$row['shout_text'] 	= str_replace('class="postlink', 'target="_blank" class="postlink', $row['shout_text']);
							}
							else
							{
								$row['shout_text'] 	= str_replace('rel="nofollow"', 'rel="nofollow" target="_blank"', $row['shout_text']);
							}
						}
						// 5 is the length of <br /> - 1.
						if (substr($row['shout_text'], 0, 5) == '<br />')
						{
							$row['shout_text'] = substr($row['shout_text'], 5);
						}
						$i++;
						
						$data .= $row['shout_id']. '|+|';// 1
						$data .= $row['shout_time']. '|+|';// 2
						$data .= $row['shout_ip']. '|+|';// 3
						$data .= $row['shout_text']. '|+|';// 4
						$data .= $row['username']. '|+|';// 5
						$data .= $row['avatar']. '|+|';// 6
						$data .= $row['show_ip']. '|+|';// 7
						$data .= $row['delete']. '|+|';// 8
						$data .= $row['edit']. '|+|';// 9
						$data .= $row['msg_plain']. '|+|';// 10
						$data .= $row['is_ip']. '|+|';// 11
						$data .= '{posts}';
					}
					$this->db->sql_freeresult($result);
					
					$response->send($data);
				}
			break;
		}
	}
	
	public function construct_shout_url($url)
	{
		$url = generate_board_url(). '/' .str_replace(array('./../../../../', './../../../', './../../', './../', './'), '', $url);
		return $url;
	}
}
