<?php
/**
*
* @package Breizh Shoutbox Extension
* @version $Id: main.php 100
* @copyright (c) 2018 Sylver35  https://breizhcode.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace sylver35\breizhshoutbox\controller;

class main
{
	/** @var \phpbb\config\config */
	protected $config;
	
	/* @var \phpbb\controller\helper */
	protected $helper;
	
	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;
	
	/** @var \phpbb\path_helper */
	protected $path_helper;
	
	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\user */
	protected $user;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;
	
	/* @var \sylver35\breizhshoutbox\core\breizhshoutbox */
	protected $shoutbox;

	/**
	* Constructor
	*
	* @param \phpbb\config\config		$config
	* @param \phpbb\controller\helper	$helper
	* @param \phpbb\template\template	$template
	* @param \phpbb\user				$user
	*/
	public function __construct(
		\phpbb\config\config $config,
		\phpbb\controller\helper $helper,
		\phpbb\extension\manager $ext_manager,
		\phpbb\path_helper $path_helper,
		\phpbb\request\request $request,
		\phpbb\template\template $template, 
		\phpbb\auth\auth $auth, 
		\phpbb\user $user,  
		$phpbb_root_path, 
		$php_ext,
		\sylver35\breizhshoutbox\core\shoutbox $shoutbox)
	{
		$this->config 		= $config;
		$this->helper 		= $helper;
		$this->ext_manager	= $ext_manager;
		$this->path_helper	= $path_helper;
		$this->request 		= $request;
		$this->template 	= $template;
		$this->auth 		= $auth;
		$this->user 		= $user;
		$this->root_path 	= $phpbb_root_path;
		$this->php_ext 		= $php_ext;
		$this->shoutbox 	= $shoutbox;
		
		$this->ext_path = $this->ext_manager->get_extension_path('sylver35/breizhshoutbox', true);
		$this->ext_path_web = $this->path_helper->update_web_root_path($this->ext_path);
	}

	/**
	*
	* @return \Symfony\Component\HttpFoundation\Response A Symfony Response object
	*/
	public function handle_private()
	{
		if ($this->auth->acl_get('u_shout_priv'))
		{
			$this->shoutbox->shout_display(3);
			$this->template->assign_vars(array(
				'S_IN_PRIV'				=> true,
				'S_IN_SHOUT_POP'		=> false,
				'S_IN_SHOUT_TEMP'		=> true,
				'S_DISPLAY_ONLINE_LIST' => true,
			));
			// Add to navlinks
			$this->template->assign_block_vars_array('navlinks', array(
				array(
					'FORUM_NAME'	=> $this->user->lang('SHOUTBOX_SECRET'),
					'U_VIEW_FORUM'	=> $this->helper->route('sylver35_breizhshoutbox_controller_private'),
				),
			));
			page_header($this->user->lang['SHOUTBOX_SECRET'], true);
			return $this->helper->render('shout_private.html', $this->user->lang['SHOUTBOX_SECRET']);
		}
		else
		{
			meta_refresh(3, append_sid("{$this->root_path}index.$this->php_ext"));
			trigger_error($this->user->lang['NO_VIEW_PRIV_PERM']);
		}
	}
	
	public function handle_popup()
	{
		if ($this->auth->acl_get('u_shout_popup'))
		{
			$this->shoutbox->shout_display(1);
			$this->template->assign_vars(array(
				'S_IN_SHOUT_POP'	=> true,
				'S_IN_PRIV'			=> false,
				'S_IN_SHOUT_TEMP'	=> true,
				'ABBC3_EXTENSION'	=> (file_exists($this->root_path . 'ext/vse/abbc3/composer.json')) ? true : false,
			));
			return $this->helper->render('shout_popup.html', $this->user->lang['SHOUTBOX_POPUP']);
		}
		else
		{
			meta_refresh(3, append_sid("{$this->root_path}index.$this->php_ext"));
			trigger_error($this->user->lang['NO_SHOUT_POP']);
		}
	}
	
	public function handle_lateral()
	{
		if ($this->auth->acl_get('u_shout_lateral'))
		{
			$kill = $this->shoutbox->kill_lateral_on();
			if (!$kill)
			{
				$this->template->assign_var('KILL_LATERAL', true);
				return;
			}
			$affiche = $this->shoutbox->shout_panel();
			if ($affiche)
			{
				$this->shoutbox->shout_display(1);

				return $this->helper->render('shout_popup.html', $this->user->lang['SHOUT_LATERAL']);
			}
		}
		else
		{
			trigger_error($this->user->lang['NO_SHOUT_POP']);
		}
	}
	
	public function handle_config_shout()
	{
		if ($this->auth->acl_get('u_shout_post'))
		{
			$this->shoutbox->active_config_shoutbox();
			
			return $this->helper->render('shout_config.html', $this->user->lang['SHOUT_PANEL_USER']);
		}
		else
		{
			meta_refresh(3, append_sid("{$this->root_path}index.$this->php_ext"));
			trigger_error($this->user->lang['NO_VIEW_PERM']);
		}
	}
}
