<?php
/**
*
* @package Breizh Shoutbox Extension
* @version $Id: shoutbox.php 100
* @copyright (c) 2018 Sylver35  https://breizhcode.com
* @license http://opensource.org/licenses/gpl-license.php GNU Public License
*
*/

namespace sylver35\breizhshoutbox\core;

use phpbb\json_response;

class shoutbox
{
	/** @var \phpbb\cache\service */
	protected $cache;
	
	/** @var \phpbb\config\config */
	protected $config;
	
	/* @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;
	
	/** @var \phpbb\pagination */
	protected $pagination;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\log\log */
	protected $log;
	
	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;
	
	/** @var \phpbb\path_helper */
	protected $path_helper;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;
	
	/** @var string Custom form action */
	protected $u_action;
	
	/**
	* The database tables
	*
	* @var string */
	protected $shoutbox_table;
	protected $shoutbox_priv_table;
	protected $shoutbox_rules_table;

	/**
	 * Constructor
	 *
	 * @param \phpbb\config\config			$config				Config object
	 * @param \phpbb\controller\helper		$helper				Controller helper object
	 * @param \phpbb\cache\service			$cache				Cache object
	 * @param \phpbb\db\driver\driver		$db					Database object
	 * @param \phpbb\template\template		$template			Template object
	 * @param \phpbb\user					$user				User object
	 * @param string						$flags_table		Name of the table used to store flag data
	 * @param \phpbb\extension\manager		$ext_manager		Extension manager object
	 * @param \phpbb\path_helper			$path_helper		Path helper object
	 */
	public function __construct(
		\phpbb\cache\service $cache,
		\phpbb\config\config $config, 
		\phpbb\controller\helper $helper, 
		\phpbb\db\driver\driver_interface $db, 
		\phpbb\pagination $pagination,
		\phpbb\request\request $request, 
		\phpbb\template\template $template, 
		\phpbb\auth\auth $auth, 
		\phpbb\user $user,
		\phpbb\log\log $log,
		\phpbb\extension\manager $ext_manager,
		\phpbb\path_helper $path_helper,
		$phpbb_root_path, 
		$php_ext,
		$shoutbox_table,
		$shoutbox_priv_table,
		$shoutbox_rules_table)
	{
		$this->cache 				= $cache;
		$this->config 				= $config;
		$this->helper 				= $helper;
		$this->db 					= $db;
		$this->pagination 			= $pagination;
		$this->request 				= $request;
		$this->template 			= $template;
		$this->auth 				= $auth;
		$this->user 				= $user;
		$this->log					= $log;
		$this->ext_manager	 		= $ext_manager;
		$this->path_helper	 		= $path_helper;
		$this->root_path 			= $phpbb_root_path;
		$this->php_ext 				= $php_ext;
		$this->shoutbox_table 		= $shoutbox_table;
		$this->shoutbox_priv_table 	= $shoutbox_priv_table;
		$this->shoutbox_rules_table = $shoutbox_rules_table;

		$this->ext_path = $this->ext_manager->get_extension_path('sylver35/breizhshoutbox', true);
		$this->ext_path_web = $this->path_helper->update_web_root_path($this->ext_path);
		$this->response = new json_response();
	}

	/**
	* Returns cdata'd string
	* @param string $txt
	* @return string
	*/
	public function shout_xml($contents)
	{
		$contents = str_replace('&nbsp;', ' ', $contents);
		if (preg_match('/\<(.*?)\>/xsi', $contents))
		{
			$contents = preg_replace('/\<script[\s]+(.*)\>(.*)\<\/script\>/xsi', '', $contents);
		}

		if ((strpos($contents, '>') !== false) || (strpos($contents, '<') !== false) || (strpos($contents, '&') !== false))
		{
			// CDATA doesn't let you use ']]>' so fall back to WriteString
			if ((strpos($contents, ']]>') !== false))
			{
				return htmlspecialchars_decode($contents);
			}
			else
			{
				return '<![CDATA[' .$contents. ']]>';
			}
		}
		else
		{
			return htmlspecialchars_decode($contents);
		}
		
		return $contents;
	}

	/**
	* Prints a sql XML error.
	* @param string $sql Sql query
	* @param int $line Linenumber
	* @param string $file Filename
	*/
	public function shout_sql_error($sql, $line = __LINE__, $file = __FILE__)
	{
		$sql = $this->shout_xml($sql);
		$err = $this->db->sql_error();
		$err = $this->shout_xml($err['message']);
		$return = '0{errors}' .$err. '{errors}' .$sql. '{errors}';
		return $return;
	}

	/**
	* Prints an error.
	* @param string $message Error
	*/
	public function shout_error($message, $on1 = false, $on2 = false, $on3 = false)
	{
		if (!isset($this->user->lang[$message]))
		{
			$message = $message;
		}
		else
		{
			if ($on1 && !$on2 && !$on3)
			{
				$message = sprintf($this->user->lang[$message], $on1);
			}
			else if ($on1 && $on2 && !$on3)
			{
				$message = sprintf($this->user->lang[$message], $on1, $on2);
			}
			else if ($on1 && $on2 && $on3)
			{
				$message = sprintf($this->user->lang[$message], $on1, $on2, $on3);
			}
			else
			{
				$message = $this->user->lang[$message];
			}
		}
		$return = 'return{errors}' .$message. '{errors}0';
		$this->response->send($return);
	}

	/**
	* Run the cron function
	* Work with normal and private shoutbox
	*/
	public function shout_cron($priv)
	{
		$deleted = '';
		$_priv = $priv ? '_priv' : '';
		$_Priv = $priv ? '_PRIV' : '';
		$_table = $priv ? $this->shoutbox_priv_table : $this->shoutbox_table;

		if ($this->config['shout_prune' .$_priv] == '' || $this->config['shout_prune' .$_priv] == 0 || $this->config['shout_max_posts' .$_priv] > 0)
		{
			return;
		}
		else if (($this->config['shout_prune' .$_priv] > 0) && ($this->config['shout_max_posts' .$_priv] == 0))
		{
			$time = time() - ($this->config['shout_prune' .$_priv] * 3600);
			
			$sql = 'DELETE FROM ' .$_table. ' WHERE shout_time < ' .$time;
			$this->db->sql_query($sql);
			$deleted = $this->db->sql_affectedrows();
			if ($deleted > 0)
			{
				set_config_count('shout_del_auto' .$_priv, $deleted, true);
				if ($this->config['shout_log_cron' .$_priv])
				{
					if (!function_exists('add_log'))
					{
						include ($this->root_path. 'includes/functions_admin.' .$this->php_ext);
					}
					add_log('admin', 'LOG_SHOUT' .$_Priv. '_PURGED', $deleted);
				}
				if ($this->config['shout_delete_robot'])
				{
					$this->post_robot_shout(0, '0.0.0.0', $priv, true, false, true, false, $deleted);
				}
			}
			set_config('shout_last_run' .$_priv, time(), true);
		}
		return;
	}

	/**
	* Runs the cron functions if time is up
	* Work with normal and private shoutbox
	*/
	public function execute_shout_cron($sort)
	{
		$_priv = $sort ? '_priv' : '';
		if ((time() - 900) <= $this->config['shout_last_run' .$_priv])
		{
			return;
		}
		else
		{
			$this->shout_cron($sort ? true : false);
			return;
		}
	}

	/**
	* Delete posts when the maximum reaches
	* Work in normal and private shoutbox
	*/
	public function delete_shout_posts($sort)
	{
		$nb_to_del 	= 9; // delete 10 messages in 1 operation
		$deleted 	= '';
		$_priv 		= $sort ? '_priv' : '';
		$_Priv 		= $sort ? '_PRIV' : '';
		$_table 	= $sort ? $this->shoutbox_priv_table : $this->shoutbox_table;

		$sql = 'SELECT COUNT(shout_id) as total
			FROM ' .$_table;
		$result = $this->db->sql_query($sql);
		if (!$result)
		{
			$this->shout_sql_error($sql, __LINE__, __FILE__);
			return;
		}
		$row_nb = $this->db->sql_fetchfield('total', $result);
		$this->db->sql_freeresult($result);
		if ($row_nb > ((int)$this->config['shout_max_posts' .$_priv] + $nb_to_del))
		{
			$i = 0;
			$delete = array();
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_id',
				'FROM'      => array($_table => ''),
				'ORDER_BY'	=> 'shout_time DESC',
			));
			$result = $this->db->sql_query_limit($sql, $this->config['shout_max_posts' .$_priv]);
			if (!$result)
			{
				$this->shout_sql_error($sql, __LINE__, __FILE__);
				return;
			}
			while ($row = $this->db->sql_fetchrow($result))
			{
				$delete[] = $row['shout_id'];
				$i++;
			}
			$sql = 'DELETE FROM ' .$_table. ' 
				WHERE ' . $this->db->sql_in_set('shout_id', $delete, true);
			$this->db->sql_query($sql);
			$deleted = $this->db->sql_affectedrows();
			
			if ($this->config['shout_log_cron' .$_priv])
			{
				if (!function_exists('add_log'))
				{
					include ($this->root_path . 'includes/functions_admin.' . $this->php_ext);
				}
				add_log('admin', 'LOG_SHOUT' .$_Priv. '_REMOVED', $deleted);
			}
			set_config_count('shout_del_auto' .$_priv, $deleted, true);
			if ($this->config['shout_delete_robot'])
			{
				$this->post_robot_shout(0, '0.0.0.0', $sort, true, false, true, true, $deleted);
			}
		}
		return;
	}

	/*
	* Get smilies shoutbox from cache
	*/
	public function get_shout_smilies()
	{
		if (($data = $this->cache->get('_shout_smilies')) === false)
		{
			$data = array();
			$sql_ary = array(
				'SELECT'    => 'smiley_id, code, smiley_url, smiley_width, smiley_height, emotion',
				'FROM'      => array(SMILIES_TABLE => ''),
				'WHERE'		=> 'display_on_shout = 1',
				'GROUP_BY'	=> 'emotion',
				'ORDER_BY'	=> 'smiley_order ASC',
			);
			$result = $this->db->sql_query($this->db->sql_build_query('SELECT', $sql_ary));
			if (!$result)
			{
				$this->db->sql_freeresult($result);
				$this->shout_sql_error($sql, __LINE__, __FILE__);
				return;
			}
			while ($row = $this->db->sql_fetchrow($result))
			{
				$data[] = array(
					'smiley_id'		=> $row['smiley_id'],
					'code'			=> $row['code'],
					'smiley_url'	=> $row['smiley_url'],
					'smiley_width'	=> $row['smiley_width'],
					'smiley_height'	=> $row['smiley_height'],
					'emotion'		=> $row['emotion'],
				);
			}
			$this->db->sql_freeresult($result);
			
			$this->cache->put('_shout_smilies', $data, 604800); // cache for 7 days
		}
		return $data;
	}

	/*
	* Get smilies popup shoutbox from cache
	*/
	public function get_shout_smilies_pop()
	{
		$data = array();
		if (($data = $this->cache->get('_shout_smilies_pop')) === false)
		{
			$sql_ary = array(
				'SELECT'    => 'smiley_id, code, smiley_url, smiley_width, smiley_height, emotion',
				'FROM'      => array(SMILIES_TABLE => ''),
				'WHERE'		=> 'display_on_shout = 0',
				'GROUP_BY'	=> 'emotion',
				'ORDER_BY'	=> 'smiley_order ASC',
			);
			$result = $this->db->sql_query($this->db->sql_build_query('SELECT', $sql_ary));
			if (!$result)
			{
				$this->db->sql_freeresult($result);
				$this->shout_sql_error($sql, __LINE__, __FILE__);
				return;
			}
			while ($row = $this->db->sql_fetchrow($result))
			{
				$data[] = array(
					'smiley_id'		=> $row['smiley_id'],
					'code'			=> $row['code'],
					'smiley_url'	=> $row['smiley_url'],
					'smiley_width'	=> $row['smiley_width'],
					'smiley_height'	=> $row['smiley_height'],
					'emotion'		=> $row['emotion'],
				);
			}
			$this->db->sql_freeresult($result);
			
			$this->cache->put('_shout_smilies_pop', $data, 604800); // cache for 7 days
		}
		return $data;
	}

	/*
	* Change time of the last message from one second
	* to update the shoutbox of all users
	*/
	public function update_shout_messages($sort, $post = false)
	{
		$_table = $sort ? $this->shoutbox_priv_table : $this->shoutbox_table;
		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 'MAX(shout_id) AS shout_end',
			'FROM'      => array($_table => ''),
		));
		$result = $this->db->sql_query($sql);
		if (!$result)
		{
			$this->shout_sql_error($sql, __LINE__, __FILE__);
			return;
		}
		$max_shout = $this->db->sql_fetchfield('shout_end', $result);
		$this->db->sql_freeresult($result);
		
		if ($max_shout != $post)
		{
			$sql = 'UPDATE ' .$_table. ' 
				SET shout_time = shout_time + 1 
				WHERE shout_id = ' .$max_shout;
			$this->db->sql_query($sql);
		}
		else
		{
			return;
		}
	}

	/**
	* Check if the rules with apropriate language exist
	* @param bool sort of shoutbox 
	*/
	public function check_shout_rules($sort)
	{
		if (!$this->config['shout_rules'])
		{
			return;
		}
		$iso = $this->user->lang_name;
		if (isset($this->config['shout_rules' .$sort.'_'.$iso]))
		{
			if ($this->config['shout_rules' .$sort.'_'.$iso])
			{
				return $iso;
			}
		}
		else
		{
			if (isset($this->config['shout_rules_en' .$sort]))
			{
				$iso = 'en';
			}
			else
			{
				return;
			}
		}
		return;
	}

	/**
	* Get the rules from the cache
	*/
	public function get_shout_rules()
	{
		if (($rules = $this->cache->get('_shout_rules')) === false)
		{
			$sql_ary = array(
				'SELECT'    => 'l.lang_iso, r.*',
				'FROM'      => array(LANG_TABLE => 'l'),
				'LEFT_JOIN'	=> array(
					array(
						'FROM'	=> array($this->shoutbox_rules_table => 'r'),
						'ON'	=> 'r.rules_lang = l.lang_iso',
					)
				),
			);
			$result = $this->db->sql_query($this->db->sql_build_query('SELECT', $sql_ary));
			if (!$result)
			{
				$this->db->sql_freeresult($result);
				$this->shout_sql_error($sql, __LINE__, __FILE__);
				return;
			}
			while ($row = $this->db->sql_fetchrow($result))
			{
				$rules[$row['lang_iso']] = array(
					'rules_id'				=> $row['id'],
					'rules_text'			=> $row['rules_text'],
					'rules_uid'				=> $row['rules_uid'],
					'rules_bitfield'		=> $row['rules_bitfield'],
					'rules_flags'			=> $row['rules_flags'],
					'rules_text_priv'		=> $row['rules_text_priv'],
					'rules_uid_priv'		=> $row['rules_uid_priv'],
					'rules_bitfield_priv'	=> $row['rules_bitfield_priv'],
					'rules_flags_priv'		=> $row['rules_flags_priv'],
				);
			}
			$this->db->sql_freeresult($result);
			
			$this->cache->put('_shout_rules', $rules, 604800); // cache for 7 days
		}
		
		return $rules;
	}

	/**
	* Displays the rules with apropriate language
	* @param bool sort of shoutbox 
	*/
	public function shout_rules($sort)
	{
		$_rules = '';
		switch ($sort)
		{
			case 1:
			case 2:
				$priv = '';
			break;
			case 3:
				$priv = '_priv';
			break;
		}
		$iso = $this->check_shout_rules($priv);
		if (!$iso)
		{
			return;
		}
		$rules = $this->get_shout_rules();
		if ($rules)
		{
			if (!function_exists('gen_sort_selects'))
			{
				include($this->root_path . 'includes/functions_content.' . $this->php_ext);
			}
			$_rules = generate_text_for_display($rules[$iso]['rules_text'.$priv], $rules[$iso]['rules_uid'.$priv], $rules[$iso]['rules_bitfield'.$priv], $rules[$iso]['rules_flags'.$priv]);
		}
		return $_rules;
	}

	/**
	* Displays the shoutbox
	*/
	public function shout_display($sort)
	{
		// If it isnt installed or Innactive we cant display it.
		if (!isset($this->config['shout_version']) || !$this->config['shout_enable'])
		{
			return;
		}
		if (!$this->auth->acl_get('u_shout_view'))
		{
			return;
		}
		
		// Load language file
		$this->user->add_lang_ext('sylver35/breizhshoutbox', array('shout', 'info_acp_shoutbox'));
		$this->user->add_lang(array('ucp', 'posting'));
		
		$is_user = ($this->user->data['user_id'] != ANONYMOUS && !$this->user->data['is_bot']) ? true : false;
		// Protection for private and define sort of shoutbox
		$in_priv = false;
		switch ($sort)
		{
			case 1:  // Popup
				$_priv = '';
				$r_shout = '_pop';
				$in_sort = 's=1';
			break;
			case 2:  // Normal
				$_priv = '';
				$in_sort = 's=2';
			break;
			case 3: // Private
				$_priv = '_priv';
				$r_shout = '_priv';
				$in_sort = 's=3';
				$in_priv = true;
				if (!$this->auth->acl_get('u_shout_priv') || !$is_user)
				{
					return;
				}
			break;
		}
		
		// Load the user's preferences
		if ($is_user && $this->user->data['user_shout'])
		{
			$rep		= explode('||', $this->user->data['user_shout']);
			$correct	= $rep[0];
			$_index 	= ($rep[7] == 3) ? $this->config['shout_position_index'] : $rep[7];
			$_forum 	= ($rep[8] == 3) ? $this->config['shout_position_forum'] : $rep[8];
			$_topic 	= ($rep[9] == 3) ? $this->config['shout_position_topic'] : $rep[9];
		}
		else // Anonymous & robots
		{
			$correct	= $this->config['shout_sound_on'];
			$_index 	= $this->config['shout_position_index'];
			$_forum 	= $this->config['shout_position_forum'];
			$_topic 	= $this->config['shout_position_topic'];
		}
		// Active lateral panel or not
		$active_panel = false;
		if ($this->auth->acl_get('u_shout_lateral'))
		{
			if ($sort == 3) // Activate it in private shoutbox
			{
				$this->config['shout_panel_auto'] = true; // Force autoload here
				$active_panel = true;
			}
			else // And verifie in another pages
			{
				$active_panel = ($this->config['shout_panel'] && $this->config['shout_panel_all']) ? true : false;
			}
		}
		
		
		$this->template->assign_vars(array(
			'S_DISPLAY_SHOUTBOX'	=> true,
			'SHOUT_INP_SORT'		=> ($sort == 1) ? 1 : 0,
			'IN_SHOUT_POPUP'		=> ($sort == 1) ? true : false,
			'SHOUT_CORRECT'			=> $correct,
			'SMILIES_TOP'			=> $this->auth->acl_get('u_shout_smilies') ? true : false,
			'RULES_TOP'				=> ($this->auth->acl_get('u_shout_post') && $this->check_shout_rules($in_priv ? true : false)) ? true : false,
			'COLOR_TOP'				=> $this->auth->acl_get('u_shout_color') ? true : false,
			'CHARS_TOP'				=> $this->auth->acl_get('u_shout_chars') ? true : false,
			'ONLINE_TOP'			=> !$this->user->data['is_bot'] ? true : false,
			'TEXT_USER_TOP'			=> ($is_user && $this->auth->acl_get('u_shout_bbcode_change')) ? true : false,
			'ACTION_USERS_TOP'		=> ($is_user && ($this->auth->acl_get('u_shout_post_inp') || $this->auth->acl_get('a_') || $this->auth->acl_get('m_'))) ? true : false,
			'LANG_LEFT'				=> ($this->user->lang['DIRECTION'] == 'ltr') ? true : false,
			'COLOR_PANEL'			=> $this->config['shout_color_panel'],
			'PANEL_ALL'				=> $active_panel,
			'INDEX_SHOUT'			=> ($this->config['shout_index'] == 1) ? true : false,
			'INDEX_SHOUT_TOP'		=> ($_index == 0) ? true : false,
			'INDEX_SHOUT_AFTER'		=> ($_index == 1) ? true : false,
			'INDEX_SHOUT_END'		=> ($_index == 2) ? true : false,
			'FORUM_SHOUT'			=> ($this->config['shout_forum'] == 1) ? true : false,
			'POS_SHOUT_FORUM_TOP'	=> ($_forum == 0) ? true : false,
			'POS_SHOUT_FORUM_END'	=> ($_forum == 1) ? true : false,
			'TOPIC_SHOUT'			=> ($this->config['shout_topic'] == 1) ? true : false,
			'POS_SHOUT_TOPIC_TOP'	=> ($_topic == 0) ? true : false,
			'POS_SHOUT_TOPIC_END'	=> ($_topic == 1) ? true : false,
			'U_BOX_SHOUT'			=> ($this->auth->acl_get('u_shout_priv')) ? $this->helper->route('sylver35_breizhshoutbox_controller_private') : '',
			'U_BOARD'				=> $this->complete_shout_url(),
			'SHOUT_TITLE'			=> sprintf($this->config['shout_version_url'], $this->config['shout_title' .$_priv]),
			'SHOUT_TITLE_V'			=> sprintf($this->user->lang['SHOUTBOX_VER'], $this->config['shout_version']),
			'SHOUT_USERNAME'		=> (isset($_COOKIE[$this->config['cookie_name']. '_shout-name']) && $this->user->data['user_id'] == ANONYMOUS) ? utf8_normalize_nfc(request_var($this->config['cookie_name']. '_shout-name', '', true, true)) : '',
			'SHOUT_USERNAME_EXPLAIN' => sprintf($this->user->lang[$this->config['allow_name_chars']. '_EXPLAIN'], 3, 11),
			'SHOUT_EXT_PATH'		=> $this->ext_path_web,
			'SHOUT_CHARS_PATH'		=> $this->ext_path_web. 'assets/javascript/chars.js',
			'SHOUT_COLORS_PATH'		=> $this->ext_path_web. 'assets/javascript/colors_palettes.js',
			'S_SHOUT_VERSION' 		=> $this->config['shout_version'],
			'S_IN_PRIV'				=> $in_priv,
		));
		
		if ($this->auth->acl_get('u_shout_post') && $this->auth->acl_get('u_shout_bbcode'))
		{
			$this->template->assign_vars(array(
				'SHOUT_POSTING'			=> true,
				'S_BBCODE_ALLOWED'		=> true,
				'S_BBCODE_IMG'			=> true,
				'S_LINKS_ALLOWED'		=> true,
				'S_BBCODE_QUOTE'		=> true,
				'S_BBCODE_FLASH'		=> false,
			));
			
			if (!function_exists('display_custom_bbcodes'))
			{
				include($this->root_path . 'includes/functions_display.' . $this->php_ext);
			}
			// Build custom bbcodes array
			display_custom_bbcodes();
			
			if (isset($this->config['smiliecreator_version']))
			{
				$this->user->add_lang_ext('sylver35/smilecreator', array('lang_smilie_creator'));
				$this->template->assign_var('CREATOR_EXTENSION', true);
			}
		}
		
		$this->javascript_shout($sort, true);
		
		// Always post enter info in the private shoutbox -> toc toc toc, it's me ;)
		if ($in_priv && $this->auth->acl_get('u_shout_priv'))
		{
			$this->post_robot_shout($this->user->data['user_id'], $this->user->ip, true, false, false, false, false);
		}

		// Do the shoutbox Prune thang
		if ($this->config['shout_on_cron' .$_priv] && ($this->config['shout_max_posts' .$_priv] == 0))
		{
			if (($this->config['shout_last_run' .$_priv] + ($this->config['shout_prune' .$_priv] * 3600)) < time())
			{
				$this->execute_shout_cron($in_priv ? true : false);
			}
		}
	}
	
	public function shout_sql_append_forbidden_bbcodes($sql_where)
	{
		$disallowed_bbcodes = explode(', ', $this->config['shout_bbcode']);

		if (!empty($disallowed_bbcodes))
		{
			$sql_where .= ' AND ' . $this->db->sql_in_set('b.bbcode_tag', $disallowed_bbcodes, true);
		}

		return $sql_where;
	}
	
	public function remove_disallowed_bbcodes($sql_ary)
	{
		$sql_ary['WHERE'] = $this->shout_sql_append_forbidden_bbcodes($sql_ary['WHERE']);

		return $sql_ary;
	}

	/**
	* Search compatibles browsers
	* To display correctly the shout
	*/
	public function compatibles_browsers()
	{
		$browser = strtolower($this->user->browser);
		if (empty($browser))
		{
			return 0;
		}
		else
		{
			if (preg_match("#ipad|tablet#i", $browser)) // Ipad and tablet ok
			{
				return 3;
			}
			else if (preg_match("#mobile|android|iphone|mobi|ipod|fennec|webos|j2me|midp|cdc|cdlc|bada#i", $browser)) // Mobiles browsers
			{
				return 2;
			}
			else if (strpos($browser, 'msie') === false) // Another browsers not IE
			{
				return 5;
			}
			else if (preg_match("#msie 11\.0|msie 10\.0|msie 9\.0#i", $browser)) // IE 9, 10 & 11
			{
				return 4;
			}
			else // Another old IE versions
			{
				return 1;
			}
		}
	}

	/**
	* Displays the retractable lateral panel
	*/
	public function shout_panel()
	{
		// If it isn't installed we can't display it.
		if (!isset($this->config['shout_version']))
		{
			return false;
		}
		
		if (!$this->auth->acl_get('u_shout_lateral') || $this->config['board_disable'] && !$this->auth->acl_get('a_') || $this->user->data['is_bot'])
		{
			$this->template->assign_var('KILL_LATERAL', true);
			return false;
		}
		// Display only if we are not in excluded mode
		$display = $this->kill_lateral_on();
		if (!$display)
		{
			$this->template->assign_var('KILL_LATERAL', true);
			return false;
		}
		else
		{
			$this->template->assign_vars(array(
				'S_IN_SHOUT_POP'	=> true,
				'S_IN_PRIV'			=> false,
				'ACTIVE_PANEL'		=> $this->auth->acl_get('u_shout_lateral') ? true : false,
				'AUTO_PANEL'		=> $this->config['shout_panel_auto'] ? true : false,
				'S_AUTO_PANEL'		=> $this->config['shout_panel_auto'] ? 'true' : 'false',
				'U_SHOUTBOX'		=> $this->config['shout_panel_auto'] ? $this->helper->route('sylver35_breizhshoutbox_controller_lateral') : '',
				'U_SHOUTBOX2'		=> $this->helper->route('sylver35_breizhshoutbox_controller_lateral'),
				'PANEL_OPEN'		=> $this->ext_path_web. 'images/panel/' .$this->config['shout_panel_img'],
				'PANEL_CLOSE'		=> $this->ext_path_web. 'images/panel/' .$this->config['shout_panel_exit_img'],
				'PANEL_WIDTH'		=> $this->config['shout_panel_width']. 'px',
				'PANEL_HEIGHT'		=> $this->config['shout_panel_height']. 'px',
				'S_SHOUT_VERSION' 	=> $this->config['shout_version'],
				'S_IS_BOT'			=> ($this->user->data['is_bot']) ? true : false,
				'ABBC3_EXTENSION'	=> (file_exists($this->root_path . 'ext/vse/abbc3/composer.json')) ? true : false, // Active the ABBC3 extension if exist
			));
			return true;
		}
	}

	/*
	* Function for no display the lateral panel
	* based on page list in config
	* Never display it for mobile phones (ipad ok)
	*/
	public function kill_lateral_on()
	{
		if (!isset($this->config['shout_enable'])) // Not exist at this time?
		{
			return false;
		}
		else if (!$this->config['shout_panel'] || !$this->config['shout_enable'] || !$this->auth->acl_get('u_shout_lateral')) // Not enabled or no permission
		{
			return false;
		}
		
		$browser = strtolower($this->user->browser);
		if (preg_match("#mobile|android|iphone|mobi|ipod|webos#i", $browser) && strpos($browser, 'ipad') === false) // Not for mobile browsers (not ipad)
		{
			return false;
		}
		else
		{
			$is_param = $is_dir = $_page = $param = $dir = $Page = false; // Initialise
			if (preg_match("#ucp|mcp|search|adm#i", $this->user->page['page_name'])) // Exclude all pages in this list
			{
				return false;
			}
			$on_page = ($this->user->page['page_dir'] ? $this->user->page['page_dir'].'/' : '').$this->user->page['page_name'].($this->user->page['query_string'] ? '?'.$this->user->page['query_string'] : '');
			$on_page1 = ($this->user->page['page_dir'] ? $this->user->page['page_dir'].'/' : '').$this->user->page['page_name'];
			$exclude_list = str_replace('&amp;', '&', $this->config['shout_page_exclude']); // Exclude list in acp
			$pages = explode('||', $exclude_list);
			foreach ($pages as $page)
			{
				$page = str_replace('app.php/', '', $page);// In case if rewriting is enabled
				if (preg_match("#{$page}#i", $this->user->page['page_name']))
				{
					return false;
				}
				else if (strpos($page, '?') !== false)
				{
					$is_param = true;
					list($_page, $param) = explode('?', $page);
					$query_string = ($this->user->page['query_string']) ? explode('&', $this->user->page['query_string']) : '-';
				}
				
				if (!$is_param) // exclude all pages with or without parameters
				{
					if ($on_page1 == $_page)
					{
						return false;
					}
				}
				else
				{
					if (empty($this->user->page['query_string']))
					{
						if ($on_page == $page)
						{
							return false;
						}
					}
					else
					{
						if ($on_page1 == $_page && ($this->user->page['query_string'] == $param || $query_string[0] == $param))
						{
							return false;
						}
					}
				}
			}
		}
		return true; // Ok, let's go to display it baby (^_^)
	}

	/*
	* Personalize message before submit
	* Remove identical bbcode as present in the message
	*/
	public function personalize_shout_message($message, $shout_bbcode)
	{
		// Don't personalize if somes bbcodes are presents
		if (strpos($message, '[spoil') !== false || strpos($message, '[hidden') !== false || strpos($message, '[offtopic') !== false || strpos($message, '[mod=') !== false || strpos($message, '[quote') !== false || strpos($message, '[code') !== false || strpos($message, '[list') !== false)
		{
			return $message;
		}
		$on = array();
		list($open, $close) = explode('||', $shout_bbcode);

		if (strpos($message, '[/') !== false)
		{
			$user_open = array_reverse(explode(', ', str_replace('][', '], [', $open)));
			$user_close = explode(', ', str_replace('][', '], [', $close));
			// Search with close bbcode
			for ($i = 0, $nb = sizeof($user_close); $i < $nb; $i++)
			{
				$on[$i] = str_replace(array('[', ']', '/'), '', $user_close[$i]);
				if (preg_match('/\[\/'.$on[$i].'\]/i', $message))
				{
					$open = str_replace($user_open[$i], '', $open);
					$close = str_replace($user_close[$i], '', $close);
				}
				else
				{
					continue;
				}
			}
		}
		$message = $open. ' ' .$message. ' ' .$close;

		return $message;
	}

	/*
	* Personalize personal and robot messages before submit
	*/
	public function bbcode_shout_message($message, $personal = false, $row = false)
	{
		if ($personal)
		{
			return '[color=#' .$this->config['shout_color_message']. ']SHOUT_USER_POST [/color][color=#' .$row['user_colour']. '][b]' .$row['username']. '[/b][/color] :: ' .$message; // Color bbcode must be formated here
		}
		else
		{
			return '[color=#'.$this->config['shout_color_message'].'][i]' .$message. '[/i][/color]'; // BBcodes must be formated here
		}
	}

	public function shout_post_hide($mode, $s_hide_robot)
	{
		if ($this->auth->acl_get('u_shout_hide') && $s_hide_robot)
		{
			if ($mode == 'edit' && !$this->config['shout_edit_robot'] && !$this->config['shout_edit_robot_priv'])
			{
				return false;
			}
			else
			{
				return true;
			}
		}
		else
		{
			return false;
		}
	}

	/*
	* Parse bbcodes in personalisation
	* before submit
	*/
	public function parse_shout_bbcodes($open, $close, $other)
	{
		// Initalise the return data
		$data = array(
			'sort'		=> 0,
			'message'	=> '',
		);
		// Return error no permission for change personalisation of another
		if ($other > 0 && ($other != $this->user->data['user_id']))
		{
			if (!$this->auth->acl_get('a_') && !$this->auth->acl_get('m_'))
			{
				$data = array(
					'sort'		=> 5,
					'message'	=> $this->user->lang['NO_SHOUT_PERSO_PERM'],
				);
				return $data;
			}
		}

		// prepare the list
		$open = str_replace('][', '], [', $open);
		$close = str_replace('][', '], [', $close);
		// explode it
		$array_open = explode(', ', $open);
		$array_close = explode(', ', $close);
		// for this user or an another?
		$on_id = ($other > 0) ? (int)$other : (int)$this->user->data['user_id'];

		$sql = $this->db->sql_build_query('SELECT', array(
			'SELECT'    => 'shout_bbcode',
			'FROM'      => array(USERS_TABLE => ''),
			'WHERE'		=> 'user_id = ' .$on_id,
		));
		$result = $this->db->sql_query_limit($sql, 1);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		if ($open == 1 && $close == 1) // Any modification
		{
			if ($row['shout_bbcode'])
			{
				$data['sort'] = 1;
				return $data;
			}
			else
			{
				$data = array(
					'sort'		=> 4,
					'message'	=> $this->user->lang['SHOUT_BBCODE_ERROR_SHAME'],
				);
				return $data;
			}
		}
		else if ($open == '' && $close || $open && $close == '') // If one is empty
		{
			$data = array(
				'sort'		=> 2,
				'message'	=> $this->user->lang['SHOUT_BBCODE_ERROR'],
			);
			return $data;
		}
		else if (sizeof($array_open) != sizeof($array_close)) // If the number of bbcodes opening and closing is different
		{
			$data = array(
				'sort'		=> 2,
				'message'	=> $this->user->lang['SHOUT_BBCODE_ERROR_COUNT'],
			);
			return $data;
		}
		else if (!preg_match("#^\[|\[|\]|\]$#", $open) || !preg_match("#^\[|\[|\[/|\]|\]$#", $close)) // If a square bracket is absent
		{
			$data = array(
				'sort'		=> 2,
				'message'	=> $this->user->lang['SHOUT_BBCODE_ERROR_COUNT'],
			);
			return $data;
		}
		else
		{
			// Initalise closing of bbcodes and correct imbrication
			$s = $n = 0;
			$slash = $sort = array();
			$_open = array_reverse($array_open);
			for ($i = 0, $nb = sizeof($_open); $i < $nb; $i++)
			{
				$first = substr($_open[$i], 0, strlen($array_close[$i])-2).']';
				if (strpos($array_close[$i], '[/') === false)
				{
					$slash[] = $array_close[$i];
					$s++;
				}
				else if ($first != str_replace('/', '', $array_close[$i]))
				{
					$sort[] = $array_close[$i];
					$n++;
				}
				else
				{
					continue;
				}
			}
			// Check closing of bbcodes
			if ($s)
			{
				$slash = implode(', ', $slash);
				$data = array(
					'sort'		=> 2,
					'message'	=> sprintf($this->user->lang['SHOUT_BBCODE_ERROR_SLASH'.(($s > 1) ? 'S' : '')], $s, $slash),
				);
				return $data;
			}
			// Check the correct imbrication of bbcodes
			if ($n)
			{
				$sort = implode(', ', $sort);
				$data = array(
					'sort'		=> 2,
					'message'	=> sprintf($this->user->lang['SHOUT_BBCODE_ERROR_IMB'.(($n > 1) ? 'S' : '')], $n, $sort),
				);
				return $data;
			}
			// Check opening and closing of bbcodes
			if ($row['shout_bbcode'])
			{
				$shoutbbcode = explode('||', $row['shout_bbcode']);
				if (str_replace('][', '], [', $shoutbbcode[0]) == $open && str_replace('][', '], [', $shoutbbcode[1]) == $close)
				{
					$data = array(
						'sort'		=> 4,
						'message'	=> $this->user->lang['SHOUT_BBCODE_ERROR_SHAME'],
					);
					return $data;
				}
			}
			// See for unautorised bbcodes
			$bbcode_array = explode(', ', $this->config['shout_bbcode_user'].', '.$this->config['shout_bbcode']);
			foreach ($bbcode_array as $no)
			{
				if (strpos($close, '[/'.$no.']') !== false)
				{
					$data = array(
						'sort'		=> 2,
						'message'	=> sprintf($this->user->lang['SHOUT_NO_CODE'], '['.$no.'][/'.$no.']'),
					);
					return $data;
				}
			}
			// Limit font size for no admin
			if (strpos($open, '[size=') !== false && !$this->auth->acl_get('a_'))
			{
				$all = explode(', ', $open);
				foreach ($all as $is)
				{
					if (preg_match('/size=/i', $is))
					{
						$size = str_replace(array('[', 'size=', ']'), '', $is);
						if ($size > $this->config['shout_bbcode_size'])
						{
							$data = array(
								'sort'		=> 2,
								'message'	=> sprintf($this->user->lang['MAX_FONT_SIZE_EXCEEDED'], $this->config['shout_bbcode_size']),
							);
							return $data;
						}
					}
					else
					{
						continue;
					}
				}
			}
			// No video!
			$video_array = array('flash', 'swf', 'mp4', 'mts', 'avi', '3gp', 'asf', 'flv', 'mpeg', 'video', 'embed', 'BBvideo', 'scrippet', 'quicktime', 'ram', 'gvideo', 'youtube', 'veoh', 'collegehumor', 'dm', 'gamespot', 'gametrailers', 'ignvideo', 'liveleak');
			foreach ($video_array as $video)
			{
				if (strpos($open, '['.$video) !== false || strpos($open, '<'.$video) !== false)
				{
					$data = array(
						'sort'		=> 2,
						'message'	=> $this->user->lang['SHOUT_NO_VIDEO'],
					);
					return $data;
				}
				else
				{
					continue;
				}
			}
			// If all is ok, return 3
			if ($data['sort'] == 0)
			{
				$data['sort'] = 3;
			}
		}
		return $data;
	}

	/*
	* Parse message before submit
	* Prevent some hacking too...
	*/
	public function parse_shout_message($message, $shout = false, $mode = 'post')
	{
		$priv = $shout ? '_priv' : '';
		$Priv = $shout ? '_PRIV' : '';
		// Set the minimum of caracters to 1 in a message to parse all the time...
		// This will not alter the minimum in the post form...
		$this->config['min_post_chars'] = 1;
		$message2 = preg_replace("(\[.+?\])is", '', $message);
		
		if (!function_exists('add_log'))
		{
			include ($this->root_path . 'includes/functions_admin.' . $phpEx);
		}
		// Delete enter message before...
		if (strpos($message, $this->user->lang['SHOUT_AUTO']) !== false)
		{
			$message = str_replace($this->user->lang['SHOUT_AUTO'], '', $message);
		}
		// Never post an empty message
		if (empty($message) || empty($message2))
		{
			$this->shout_error('MESSAGE_EMPTY');
			return;
		}
		// Correct a bug with somes empty bbcodes
		if ($message == '[img][/img]' || $message == '[b][/b]' || $message == '[i][/i]' || $message == '[u][/u]' || $message == '[url][/url]')
		{
			$this->shout_error('MESSAGE_EMPTY');
			return;
		}
		if (strpos($message, '/]') !== false)
		{
			$message = str_replace('/]', ']', $message);
		}
		// Protect the split messages
		if (strpos($message, '|+|') !== false)
		{
			$message = str_replace('|+|', '+', $message);
		}
		if (strpos($message, '{posts}') !== false)
		{
			$message = str_replace('{posts}', '|posts|', $message);
		}

		// Store message length...
		// Permission to ignore the limit of characters in a message
		if (!$this->auth->acl_get('u_shout_limit_post') && $this->config['shout_max_post_chars'])
		{
			$message_length = ($mode == 'post') ? utf8_strlen($message) : utf8_strlen(preg_replace('#\[\/?[a-z\*\+\-]+(=[\S]+)?\]#ius', ' ', $message));
			if ($message_length > $this->config['shout_max_post_chars'])
			{
				$this->shout_error('TOO_MANY_CHARS_POST', $message_length, $this->config['shout_max_post_chars']);
				return;
			}
		}
		// See for unautorised bbcodes
		$bbcode_array = explode(', ', 'testlink, ' .$this->config['shout_bbcode']);
		foreach ($bbcode_array as $no)
		{
			if (strpos($message, '[/'.$no.']') !== false)
			{
				$this->shout_error('SHOUT_NO_CODE', '['.$no.'][/'.$no.']');
				return;
			}
		}
		// No video!
		$video_array = array('flash', 'swf', 'mp4', 'mts', 'avi', '3gp', 'asf', 'flv', 'mpeg', 'video', 'embed', 'BBvideo', 'scrippet', 'quicktime', 'ram', 'gvideo', 'youtube', 'veoh', 'collegehumor', 'dm', 'gamespot', 'gametrailers', 'ignvideo', 'liveleak');
		foreach ($video_array as $video)
		{
			if (strpos($message, '['.$video) !== false && strpos($message, '[/'.$video) !== false || strpos($message, '<'.$video) !== false && strpos($message, '</'.$video) !== false)
			{
				$this->shout_error('SHOUT_NO_VIDEO');
				return;
			}
			else
			{
				continue;
			}
		}
		// Die script and vbscript for all the time...  and log it
		if (strpos($message, '&lt;script') !== false && strpos($message, '&lt;/script') !== false || strpos($message, '<script') !== false && strpos($message, '</script') !== false || 
			strpos($message, '&lt;vbscript') !== false && strpos($message, '&lt;/vbscript') !== false || strpos($message, '<vbscript') !== false && strpos($message, '</vbscript') !== false)
		{
			$this->log->add('user', $this->user->data['user_id'], $this->user->ip, 'LOG_SHOUT_SCRIPT' .$Priv);
			set_config_count('shout_nr_log' .$priv, 1, true);
			$this->shout_error('SHOUT_NO_SCRIPT');
			return;
		}
		// Die applet for all the time...  and log it
		else if (strpos($message, '&lt;applet') !== false && strpos($message, '&lt;/applet') !== false || strpos($message, '<applet') !== false && strpos($message, '</applet') !== false)
		{
			$this->log->add('user', $this->user->data['user_id'], $this->user->ip, 'LOG_SHOUT_APPLET' .$Priv);
			set_config_count('shout_nr_log' .$priv, 1, true);
			$this->shout_error('SHOUT_NO_APPLET');
			return;
		}
		// Die activex for all the time...  and log it
		else if (strpos($message, '&lt;activex') !== false && strpos($message, '&lt;/activex') !== false || strpos($message, '<activex') !== false && strpos($message, '</activex') !== false)
		{
			$this->log->add('user', $this->user->data['user_id'], $this->user->ip, 'LOG_SHOUT_ACTIVEX' .$Priv);
			set_config_count('shout_nr_log' .$priv, 1, true);
			$this->shout_error('SHOUT_NO_ACTIVEX');
			return;
		}
		// Die about and chrome objects for all the time...  and log it
		else if (strpos($message, '&lt;object') !== false && strpos($message, '&lt;/object') !== false || strpos($message, '<object') !== false && strpos($message, '</object') !== false || 
				strpos($message, '&lt;about') !== false && strpos($message, '&lt;/about') !== false || strpos($message, '<about') !== false && strpos($message, '</about') !== false || 
				strpos($message, '&lt;chrome') !== false && strpos($message, '&lt;/chrome') !== false || strpos($message, '<chrome') !== false && strpos($message, '</chrome') !== false)
		{
			$this->log->add('user', $this->user->data['user_id'], $this->user->ip, 'LOG_SHOUT_OBJECTS' .$Priv);
			set_config_count('shout_nr_log' .$priv, 1, true);
			$this->shout_error('SHOUT_NO_OBJECTS');
			return;
		}
		// Die iframe for all the time...  and log it
		else if (strpos($message, '&lt;iframe') !== false && strpos($message, '&lt;/iframe') !== false || strpos($message, '<iframe') !== false && strpos($message, '</iframe') !== false || strpos($message, '[iframe') !== false && strpos($message, '[/iframe') !== false)
		{
			$this->log->add('user', $this->user->data['user_id'], $this->user->ip, 'LOG_SHOUT_IFRAME' .$Priv);
			set_config_count('shout_nr_log' .$priv, 1, true);
			$this->shout_error('SHOUT_NO_IFRAME');
			return;
		}

		return $this->shout_url_free_sid($message);
	}

	/*
	* Build a number to differentiate guests
	*/
	public function random_ip($ip)
	{
		$rand = 0;
		$in = array('a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z');
		$out = array('1','2','3','4','5','6','7','8','9','1','2','3','4','5','6','7','8','9','1','2','3','4','5','6','7','8');
		$ip = str_replace($in, $out, strtolower($ip));
		$act = explode('.', $ip);
		for ($i = 0, $nb = sizeof($act); $i < $nb; $i++)
		{
			if ($act[$i] == 0)
			{
				continue;
			}
			$rand = $rand+$act[$i];
		}

		return round($rand/sizeof($act));
	}

	/* 
	* Construct/change profile url
	* to add actions in javascript
	* Only if user have right permissions
	* But never in acp
	*/
	public function construct_action_shout($username, $id, $colour, $acp = false, $inp = false)
	{
		if ($this->user->data['user_id'] == ANONYMOUS || $this->user->data['is_bot'])
		{
			$name = get_username_string('no_profile', $id, $username, $colour);
			$name = str_replace(array('./../../../../', './../../../', './../../', './../', './'), generate_board_url(). '/', $name);
			return $name;
		}
		else if ($id == $this->user->data['user_id'] || $id == 0 || $id == ANONYMOUS || $acp)
		{
			$name = get_username_string('full', $id, $username, $colour);
			$name = str_replace(array('./../../../../', './../../../', './../../', './../', './'), generate_board_url(). '/', $name);
			return $name;
		}
		else
		{
			if ($this->auth->acl_get('u_shout_post_inp') || $this->auth->acl_get('a_') || $this->auth->acl_get('m_'))
			{
				$username_full = get_username_string('no_profile', $id, $username, $colour);
				$username_full = str_replace(array('./../../../../', './../../../', './../../', './../', './'), generate_board_url(). '/', $username_full);
				$username_full = '<a href="javascript:actionUser(' .$id. ');" title="' .$this->user->lang['SHOUT_ACTION_TITLE']. ' ' .$username. '" class="username-coloured action-user">' .$username_full. '</a>';
			}
			return $username_full;
		}
	}

	/* 
	* Construct url whithout sid
	* Because urls must be construct for all
	* Replace caracter '&' to correct a bug
	*/
	public function shout_url_free_sid($content)
	{
		if (strpos($content, 'sid=') !== false)
		{
			$rep = explode('sid=', $content);// explode url
			if (sizeof($rep[1]) >= 32)// the sid number is on second part
			{
				$sid_ = substr($rep[1], 0, 32);// substact it
				$content = str_replace($sid_, '', $content);
				$content = str_replace(array('&amp;sid=', '&sid=', '?sid=', '-sid='), '', $content);
			}
		}
		return str_replace(array('& ', '&&amp; ', '&amp&amp; '), '&amp; ', $content);
	}

	/* 
	* Force profile url if not exist
	* Never display sid because urls are for all
	* And it will be replaced by construct_action_shout()
	* Replace caracter '&' to correct a bug
	*/
	public function shout_force_url($username, $id)
	{
		if (strpos($username, '<a') === false)
		{
			$username = '<a class="username-coloured" href="' .$this->real_board_url(true). '&amp;u=' .$id. '">' .$username. '</a>';
		}
		else
		{
			$username = $this->shout_url_free_sid($username);
		}
		return str_replace(array('& ', '&&amp; ', '&amp&amp; '), '&amp; ', $username);
	}

	/*
	* Determine the real url of the shoutbox in case of bridge or not...
	* @param $profile boll Build url to profile if true
	*/
	public function real_board_url($profile = false)
	{
		if ($this->config['script_path'] == '/')
		{
			$board_url = generate_board_url(true).'/';
		}
		else if (strpos(generate_board_url(), $this->config['script_path']) === false)
		{
			$board_url = generate_board_url().$this->config['script_path'].'/';
		}
		else
		{
			$board_url = generate_board_url(true).'/';
		}
		if ($profile)
		{
			$board_url .= "memberlist.php?mode=viewprofile";
		}
		$board_url = str_replace('shoutbox/', '', $board_url);
		return $board_url;
	}
	
	/*
	 * for work fine everywhere wee want
	 *	real complete url of shoutbox
	 */
	public function complete_shout_url()
	{
		$url = generate_board_url(). '/' .str_replace(array('./../../../../', './../../../', './../../', './../', './'), '', $this->ext_path_web);
		return $url;
	}

	/*
	* protect enter value for robot messages
	*/
	public function shout_chars($value, $sort = false)
	{
		if ($sort)
		{
			return htmlspecialchars($value, ENT_QUOTES);
		}
		else
		{
			return htmlspecialchars($value, ENT_IGNORE);
		}
	}

	/*
	* Traduct and display very quickly infos robot
	* for all infos robot functions
	* Replace urls actions if nedded (never in acp)
	*/
	public function display_infos_robot($sort, $infos, $acp = false)
	{
		$start_info	= $this->user->lang['SHOUT_ROBOT_START'];
		// html style must be formated here...
		$open_tpl 	= '<span style="font-style:italic;color:#' .$this->config['shout_color_message']. '">';
		$tpl_bold	= '<span style="font-weight:bold;">';
		$tpl_url	= '<a href="';
		$close_tpl 	= '</span>';
		$close_url	= '</a>';
		$info		= explode('||', $infos);
		switch ($sort)
		{
			case 1:
				if (isset($info[2]))
				{
					$info[0] = $this->construct_action_shout($info[0], $info[1], $info[2], $acp);
				}
				$message = sprintf($this->user->lang['SHOUT_SESSION_ROBOT'], $info[0]);
			break;
			case 2:
				$message = sprintf($this->user->lang['SHOUT_SESSION_ROBOT_BOT'], $start_info, $info[0]);
			break;
			case 3:
				if (isset($info[1]))
				{
					$info[0] = $this->construct_action_shout($info[0], $info[1], $info[2], $acp);
				}
				$message = sprintf($this->user->lang['SHOUT_ENTER_PRIV'], $start_info, $info[0]);
			break;
			case 4:
				$message = sprintf($this->user->lang['SHOUT_PURGE_ROBOT'], $start_info);
			break;
			case 5:
				$message = sprintf($this->user->lang['SHOUT_PURGE_PRIV'], $start_info);
			break;
			case 6:
				$message = sprintf($this->user->lang['SHOUT_PURGE_SHOUT'], $start_info);
			break;
			case 7:
				$message = sprintf($this->user->lang['SHOUT_PURGE_AUTO'], $start_info, $info[0]);
			break;
			case 8:
				$message = sprintf($this->user->lang['SHOUT_PURGE_PRIV_AUTO'], $start_info, $info[0]);
			break;
			case 9:
				$message = sprintf($this->user->lang['SHOUT_DELETE_AUTO'], $start_info, $info[0]);
			break;
			case 10:
				$message = sprintf($this->user->lang['SHOUT_DELETE_PRIV_AUTO'], $start_info, $info[0]);
			break;
			case 11: // Birthdays
				$infos = $this->construct_action_shout($info[0], $info[1], $info[2], $acp);
				if ($info[3] > 0) // With age to display
				{
					$message = sprintf($this->user->lang['SHOUT_BIRTHDAY_ROBOT_FULL'], $this->config['sitename'], $infos, $close_tpl, $tpl_bold . $info[3]);
				}
				else // No age to display
				{
					$message = sprintf($this->user->lang['SHOUT_BIRTHDAY_ROBOT'], $this->config['sitename'], $infos);
				}
			break;
			case 12:
				$message = sprintf($this->user->lang['SHOUT_HELLO_ROBOT'], $close_tpl, $tpl_bold.$this->user->format_date($info[0], $this->user->lang['SHOUT_ROBOT_DATE'], true));
			break;
			case 13:
				$infos = $this->construct_action_shout($info[0], $info[1], $info[2], $acp);
				$message = sprintf($this->user->lang['SHOUT_NEWEST_ROBOT'], $infos, $this->config['sitename']);
			break;
			case 14:
				$infos = $this->construct_action_shout($info[0], $info[3], $info[4], $acp);
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_GLOBAL_ROBOT'], $start_info, $infos, $url);
			break;
			case 15:
				$infos = $this->construct_action_shout($info[0], $info[3], $info[4], $acp);
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_ANNOU_ROBOT'], $start_info, $infos, $url);
			break;
			case 16:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_POST_ROBOT'], $start_info, $infos, $url);
			break;
			case 60:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_ROBOT'], $start_info, $infos, $url);
			break;
			case 17:
				$infos = $this->construct_action_shout($info[0], $info[3], $info[4], $acp);
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_EDIT_ROBOT'], $start_info, $infos, $url);
			break;
			case 70:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_E_ROBOT'], $start_info, $infos, $url);
			break;
			case 71:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_ES_ROBOT'], $start_info, $infos, $url);
			break;
			case 18:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_TOPIC_ROBOT'], $start_info, $infos, $url);
			break;
			case 72:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_F_ROBOT'], $start_info, $infos, $url);
			break;
			case 73:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_FS_ROBOT'], $start_info, $infos, $url);
			break;
			case 19:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_LAST_ROBOT'], $start_info, $infos, $url);
			break;
			case 74:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_L_ROBOT'], $start_info, $infos, $url);
			break;
			case 75:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_LS_ROBOT'], $start_info, $infos, $url);
			break;
			case 20:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_QUOTE_ROBOT'], $start_info, $infos, $url);
			break;
			case 80:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_Q_ROBOT'], $start_info, $infos, $url);
			break;
			case 21:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_REPLY_ROBOT'], $start_info, $infos, $url);
			break;
			case 76:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_R_ROBOT'], $start_info, $infos, $url);
			break;
			case 77:
				$infos = isset($info[3]) ? $this->construct_action_shout($info[0], $info[3], $info[4], $acp) : $info[0];
				$url = $tpl_url.$info[1]. '" title="' .$this->shout_chars($info[2]). '">' .$info[2].$close_url;
				$message = sprintf($this->user->lang['SHOUT_PREZ_RS_ROBOT'], $start_info, $infos, $url);
			break;
			case 99:
				$message = $this->user->lang['SHOUT_WELCOME'];
			break;
			default:
				return '';
			break;
		}
		return $open_tpl . utf8_normalize_nfc($message) . $close_tpl;
	}

	/*
	* Display infos Robot for purge, delete messages
	* and enter in the private shoutbox
	*/
	public function post_robot_shout($user_id, $ip, $priv = false, $purge = false, $robot = false, $auto = false, $delete = false, $deleted = '')
	{
		$userid 	= (int)$user_id;
		$_priv 		= $priv ? '_priv' : '';
		$_Priv 		= $priv ? '_PRIV' : '';
		$_table 	= $priv ?  $this->shoutbox_priv_table :  $this->shoutbox_table;
		$enter_priv	= ($priv && !$purge && !$robot && !$auto && !$delete) ? true : false;
		$sort_info 	= 1;
		$message 	= '-';
		
		if (!$this->config['shout_enable_robot'] && !$enter_priv || !$this->config['shout_enable'])
		{
			return;
		}
		
		if ($priv && $purge && !$robot && !$auto && !$delete)
		{
			$info = 5;
		}
		else if (!$priv && $purge && !$robot && !$auto && !$delete)
		{
			$info = 6;
		}
		else if (!$priv && $purge && !$robot && $auto && !$delete)
		{
			$message = $deleted;
			$info = 7;
		}
		else if ($priv && $purge && !$robot && $auto && !$delete)
		{
			$message = $deleted;
			$info = 8;
		}
		else if (!$priv && $purge && !$robot && $auto && $delete)
		{
			$message = $deleted;
			$info = 9;
		}
		else if ($priv && $purge && !$robot && $auto && $delete)
		{
			$message = $deleted;
			$info = 10;
		}
		else if ($robot && !$auto && !$delete)
		{
			$info = 4;
		}
		else if ($enter_priv)
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_time',
				'FROM'      => array($_table => ''),
				'WHERE'		=> "shout_robot = 8 AND shout_robot_user = $userid AND shout_time BETWEEN " .(time() -60*30). " AND " .time(), // 30 min For no enter message
			));
			$result = $this->db->sql_query($sql);
			$is_posted = $this->db->sql_fetchfield('shout_time');
			$this->db->sql_freeresult($result);
			if ($is_posted)
			{
				return;
			}
			$message = $this->user->data['username']. '||' .$this->user->data['user_id']. '||' .$this->user->data['user_colour'];
			$sort_info = 8;
			$info = 3;
		}
		
		$sql_data = array(
			'shout_time'				=> time(),
			'shout_user_id'				=> 0,
			'shout_ip'					=> (string)$ip,
			'shout_text'				=> (string)$message,
			'shout_bbcode_uid'			=> '',
			'shout_bbcode_bitfield'		=> '',
			'shout_bbcode_flags'		=> 0,
			'shout_robot'				=> (int)$sort_info,
			'shout_robot_user'			=> (int)$userid,
			'shout_forum'				=> 0,
			'shout_info'				=> (int)$info,
		);
		
		$sql = 'INSERT INTO ' .$_table. ' ' . $this->db->sql_build_array('INSERT', $sql_data);
		$this->db->sql_query($sql);
		set_config_count('shout_nr' .$_priv, 1, true);
	}

	/*
	* Display infos Robot for connections
	*/
	public function post_session_shout($array)
	{
		if ($array['session_user_id'] == ANONYMOUS || !$this->config['shout_enable'] || !$this->config['shout_enable_robot'])
		{
			return;
		}
		if (!$this->config['shout_sessions'] && !$this->config['shout_sessions_priv'])
		{
			return;
		}
		if ($array['session_viewonline'] != true)
		{
			return;
		}
		
		$bot = $this->user->data['is_bot'] ? true : false;
		if ($bot)
		{
			if (!$this->config['shout_sessions_bots'] && !$this->config['shout_sessions_bots_priv'])
			{
				return;
			}
		}
		
		$userid		= (int)$array['session_user_id'];
		$interval	= (int)$this->config['shout_sessions_time']*60;
		$is_posted 	= $is_posted_priv = false;
		
		if ($bot && $this->config['shout_sessions_bots'] || !$bot && $this->config['shout_sessions'])
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_time',
				'FROM'      => array( $this->shoutbox_table => ''),
				'WHERE'		=> "shout_robot = 1 AND shout_robot_user = $userid AND shout_time BETWEEN " .(time() - $interval). " AND " .time(), // $interval For no enter message if user was connect less than X minutes before
			));
			$result = $this->db->sql_query($sql);
			$is_posted = $this->db->sql_fetchfield('shout_time');
			$this->db->sql_freeresult($result);
		}
		if ($bot && $this->config['shout_sessions_bots_priv'] || !$bot && $this->config['shout_sessions_priv'])
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_time',
				'FROM'      => array( $this->shoutbox_priv_table => ''),
				'WHERE'		=> "shout_robot = 1 AND shout_robot_user = $userid AND shout_time BETWEEN " .(time() - $interval). " AND " .time(), // $interval For no enter message if user was connect less than X minutes before
			));
			$result = $this->db->sql_query($sql);
			$is_posted_priv = $this->db->sql_fetchfield('shout_time');
			$this->db->sql_freeresult($result);
		}
		
		$shout_info = ($bot) ? 2 : 1;
		$view = ($array['session_viewonline']) ? 'view' : 'hide';
		$message = $this->user->data['username']. '||' .$userid. '||' .$this->user->data['user_colour']. '||' .$view;
		
		$sql_data = array(
			'shout_time'				=> time(),
			'shout_user_id'				=> 0,
			'shout_ip'					=> (string)$this->user->ip,
			'shout_text'				=> (string)$message,
			'shout_bbcode_uid'			=> '',
			'shout_bbcode_bitfield'		=> '',
			'shout_bbcode_flags'		=> 0,
			'shout_robot'				=> 1,
			'shout_robot_user'			=> $userid,
			'shout_forum'				=> 0,
			'shout_info'				=> $shout_info,
		);
		
		if (($bot && $this->config['shout_sessions_bots'] || !$bot && $this->config['shout_sessions']) && !$is_posted)
		{
			$sql = 'INSERT INTO ' .  $this->shoutbox_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
			$this->db->sql_query($sql);
			set_config_count('shout_nr', 1, true);
		}
		if (($bot && $this->config['shout_sessions_bots_priv'] || !$bot && $this->config['shout_sessions_priv']) && !$is_posted_priv)
		{
			$sql = 'INSERT INTO ' .  $this->shoutbox_priv_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
			$this->db->sql_query($sql);
			set_config_count('shout_nr_priv', 1, true);
		}
	}

	/*
	* Display infos Robot for new posts, subjects, topics...
	*/
	public function advert_post_shoutbox($event)
	{
		if (!$this->config['shout_enable_robot'] || !$this->config['shout_enable'])
		{
			return;
		}
		
		$ok_shout		= $this->config['shout_post_robot'] ? true : false;
		$ok_shout_priv	= $this->config['shout_post_robot_priv'] ? true : false;
		$ip				= (string)$this->user->ip;
		$userid 		= (int)$this->user->data['user_id'];
		$topic_id 		= (int)$event['data']['topic_id'];
		$forum_id 		= (int)$event['data']['forum_id'];
		$subject		= (string)$event['subject'];
		$post_mode		= (string)$event['mode'];
		$topic_type		= (string)$event['topic_type'];
		$url 			= (string)$this->shout_url_free_sid(str_replace(array('./../../../../', './../../../', './../../', './../', './'), generate_board_url(). '/', $event['url']));
		$hide_robot		= (isset($event['data']['hide_robot'])) ? $event['data']['hide_robot'] : false;
		$is_approved	= (isset($event['post_visibility'])) ? $event['post_visibility'] : false;
		$is_prez_form 	= (isset($this->config['shout_prez_form']) && ($forum_id == $this->config['shout_prez_form'])) ? true : false;
		$topic_poster 	= 0;

		if (!$ok_shout && !$ok_shout_priv)
		{
			return;
		}
		if ($hide_robot != false)
		{
			return;
		}
		
		$exclude_forums = array();
		if ($this->config['shout_exclude_forums'])
		{
			$exclude_forums = explode(',', $this->config['shout_exclude_forums']);
			if (in_array($forum_id, $exclude_forums))
			{
				return;
			}
		}
		
		// Parse web adress in subject to prevent bug
		$subject = str_replace(array('http://www.', 'http://', 'www.'), '', $subject);
		$subject = $this->db->sql_escape(str_replace("'", $this->user->lang['SHOUT_PROTECT'], $subject));
		
		if ($is_prez_form)
		{
			$sql = 'SELECT topic_poster 
				FROM ' . TOPICS_TABLE . ' 
				WHERE topic_id = ' .$topic_id;
			$result = $this->db->sql_query_limit($sql, 1);
			$topic_poster = $this->db->sql_fetchfield('topic_poster');
			$this->db->sql_freeresult($result);
		}

		$prez_poster	= ($topic_poster == $userid) ? 1 : 0;
		$username 		= ($userid == ANONYMOUS) ? $this->user->lang['GUEST'] : (($userid == 0) ? $this->user->lang['SHOUT_ROBOT'] :  $this->user->data['username']);
		$user_colour 	= ($userid == 0) ? $this->config['shout_color_robot'] :  $this->user->data['user_colour'];
		if ($topic_type == 3 && $post_mode == 'post')
		{
			$post_mode = 'global';
		}
		else if ($topic_type == 2 && $post_mode == 'post')
		{
			$post_mode = 'annoucement';
		}
		// Delete Re: in the subject
		if (strpos($subject, 'Re: ') !== false)
		{
			$subject = str_replace('Re: ', '', $subject);
		}
		
		$message = $username.'||'.$url.'||'.$subject.'||'.$userid.'||'.$user_colour;

		switch ($post_mode)
		{
			case 'global':
				$sort_info = 2;
				$info = 14;
			break;
			case 'annoucement':
				$sort_info = 2;
				$info = 15;
			break;
			case 'post':
				$sort_info = 2;
				$info = (!$is_prez_form) ? 16 : 60;
			break;
			case 'edit':
				if ($is_prez_form)
				{
					switch ($prez_poster)
					{
						case 0:
							$info = 70;
						break;
						case 1:
							$info = 71;
						break;
					}
				}
				else
				{
					$info = 17;
				}
				$ok_shout = ($this->config['shout_edit_robot']) ? true : false;
				$ok_shout_priv = ($this->config['shout_edit_robot_priv']) ? true : false;
				$sort_info = 3;
			break;
			case 'edit_topic':
			case 'edit_first_post':
				if ($is_prez_form)
				{
					switch ($prez_poster)
					{
						case 0:
							$info = 72;
						break;
						case 1:
							$info = 73;
						break;
					}
				}
				else
				{
					$info = 18;
				}
				$ok_shout = ($this->config['shout_edit_robot']) ? true : false;
				$ok_shout_priv = ($this->config['shout_edit_robot_priv']) ? true : false;
				$sort_info = 3;
			break;
			case 'edit_last_post':
				if ($is_prez_form)
				{
					switch ($prez_poster)
					{
						case 0:
							$info = 74;
						break;
						case 1:
							$info = 75;
						break;
					}
				}
				else
				{
					$info = 19;
				}
				$ok_shout = ($this->config['shout_edit_robot']) ? true : false;
				$ok_shout_priv = ($this->config['shout_edit_robot_priv']) ? true : false;
				$sort_info = 3;
			break;
			case 'quote':
				$ok_shout = ($this->config['shout_rep_robot']) ? true : false;
				$ok_shout_priv = ($this->config['shout_rep_robot_priv']) ? true : false;
				$sort_info = 3;
				$info = (!$is_prez_form) ? 20 : 80;
			break;
			case 'reply':
				if ($is_prez_form)
				{
					switch ($prez_poster)
					{
						case 0;
							$info = 76;
						break;
						case 1;
							$info = 77;
						break;
					}
				}
				else
				{
					$info = 21;
				}
				$ok_shout = ($this->config['shout_rep_robot']) ? true : false;
				$ok_shout_priv = ($this->config['shout_rep_robot_priv']) ? true : false;
				$sort_info = 3;
			break;
		}
		
		$message = str_replace(array('./../', './'), $this->real_board_url(), $message);

		$sql_data = array(
			'shout_time'				=> time(),
			'shout_user_id'				=> 0,
			'shout_ip'					=> (string)$ip,
			'shout_text'				=> (string)$message,
			'shout_bbcode_uid'			=> '',
			'shout_bbcode_bitfield'		=> '',
			'shout_bbcode_flags'		=> 0,
			'shout_robot'				=> (int)$sort_info,
			'shout_robot_user'			=> (int)$userid,
			'shout_forum'				=> (int)$forum_id,
			'shout_info'				=> (int)$info,
		);
		
		if ($ok_shout)
		{
			$sql = 'INSERT INTO ' .  $this->shoutbox_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
			$this->db->sql_query($sql);
			set_config_count('shout_nr', 1, true);
		}
		if ($ok_shout_priv)
		{
			$sql = 'INSERT INTO ' .  $this->shoutbox_priv_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
			$this->db->sql_query($sql);
			set_config_count('shout_nr_priv', 1, true);
		}
	}

	/*
	* Display info of birthdays
	*/
	//public function robot_birthday_shout($username, $id, $colour, $age)
	public function robot_birthday_shout($event)
	{
		$is_posted = false;
		if ($this->config['shout_hello'])
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_id',
				'FROM'      => array($this->shoutbox_table => ''),
				'WHERE'		=> 'shout_robot = 5 AND shout_info = 11 AND shout_time BETWEEN ' .(time() - 60 * 60). ' AND ' .time(),
			));
			$result = $this->db->sql_query($sql);
			$is_posted = $this->db->sql_fetchfield('shout_id') ? true : false;
			$this->db->sql_freeresult($result);
		}
		else if ($this->config['shout_hello_priv'])
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_id',
				'FROM'      => array($this->shoutbox_priv_table => ''),
				'WHERE'		=> 'shout_robot = 5 AND shout_info = 11 AND shout_time BETWEEN ' .(time() - 60 * 60). ' AND ' .time(),
			));
			$result = $this->db->sql_query($sql);
			$is_posted = $this->db->sql_fetchfield('shout_id');
			$this->db->sql_freeresult($result);
		}
		if (!$is_posted)
		{
			$time = $this->user->create_datetime();
			$now = phpbb_gmgetdate($time->getTimestamp() + $time->getOffset());
			
			foreach ($event['rows'] as $row)
			{
				$birthday_year	= (int) substr($row['user_birthday'], -4);
				$birthday_age	= ($birthday_year) ? max(0, $now['year'] - $birthday_year) : 0;
				$message = $row['username']. '||' .$row['user_id']. '||' .$row['user_colour']. '||' .$birthday_age;
				
				$sql_data = array(
					'shout_time'			=> time(),
					'shout_user_id'			=> 0,
					'shout_ip'				=> '0.0.0.0',
					'shout_text'			=> (string)$message,
					'shout_bbcode_uid'		=> '',
					'shout_bbcode_bitfield'	=> '',
					'shout_bbcode_flags'	=> 0,
					'shout_robot'			=> 5,
					'shout_robot_user'		=> 0,
					'shout_forum'			=> 0,
					'shout_info'			=> 11,
				);
			
				if ($this->config['shout_birthday'])
				{
					$sql = 'INSERT INTO ' .  $this->shoutbox_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
					$this->db->sql_query($sql);
					set_config_count('shout_nr', 1, true);
				}
				if ($this->config['shout_birthday_priv'])
				{
					$sql = 'INSERT INTO ' .  $this->shoutbox_priv_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
					$this->db->sql_query($sql);
					set_config_count('shout_nr_priv', 1, true);
				}
			}
		}
	}

	/*
	* Display the date info Robot
	*/
	public function hello_robot_shout()
	{
		if (!$this->config['shout_enable_robot'] || !$this->config['shout_hello'] &&  !$this->config['shout_hello_priv'] || $this->config['shout_cron_run'] == date('d-m-Y'))
		{
			return;
		}
		
		$is_posted = false;
		if ($this->config['shout_hello'])
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_id',
				'FROM'      => array($this->shoutbox_table => ''),
				'WHERE'		=> 'shout_robot = 4 AND shout_info = 12 AND shout_time BETWEEN ' .(time() - 60*60). ' AND ' .time(),
			));
			$result = $this->db->sql_query($sql);
			$is_posted = $this->db->sql_fetchfield('shout_id') ? true : false;
			$this->db->sql_freeresult($result);
		}
		else if ($this->config['shout_hello_priv'])
		{
			$sql = $this->db->sql_build_query('SELECT', array(
				'SELECT'    => 'shout_id',
				'FROM'      => array($this->shoutbox_priv_table => ''),
				'WHERE'		=> 'shout_robot = 4 AND shout_info = 12 AND shout_time BETWEEN ' .(time() - 60*60). ' AND ' .time(),
			));
			$result = $this->db->sql_query($sql);
			$is_posted = $this->db->sql_fetchfield('shout_id');
			$this->db->sql_freeresult($result);
		}
		if (!$is_posted)
		{
			$sql_data = array(
				'shout_time'			=> time(),
				'shout_user_id'			=> 0,
				'shout_ip'				=> '0.0.0.0',
				'shout_text'			=> (string)(time(). '||' .date('d-m-Y')),
				'shout_bbcode_uid'		=> '',
				'shout_bbcode_bitfield'	=> '',
				'shout_bbcode_flags'	=> 0,
				'shout_robot'			=> 4,
				'shout_robot_user'		=> 0,
				'shout_forum'			=> 0,
				'shout_info'			=> 12,
			);

			if ($this->config['shout_hello'])
			{
				$sql = 'INSERT INTO ' . $this->shoutbox_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
				$this->db->sql_query($sql);
				set_config_count('shout_nr', 1, true);
			}
			if ($this->config['shout_hello_priv'])
			{
				$sql = 'INSERT INTO ' . $this->shoutbox_priv_table . ' ' . $this->db->sql_build_array('INSERT', $sql_data);
				$this->db->sql_query($sql);
				set_config_count('shout_nr_priv', 1, true);
			}
		}
	}

	/*
	* Display first connection for new users
	*/
	public function shout_add_newest_user($event)
	{
		if (!$this->config['shout_enable_robot'] || !$this->config['shout_newest'] && !$this->config['shout_newest_priv'])
		{
			return true;
		}
		
		$message = (string)$event['user_row']['username']. '||' .(int)$event['user_id']. '||' .(string)$this->config['newest_user_colour'];
		
		$sql_data = array(
			'shout_time'				=> time(),
			'shout_user_id'				=> 0,
			'shout_ip'					=> (string)$this->user->ip,
			'shout_text'				=> (string)$message,
			'shout_bbcode_uid'			=> '',
			'shout_bbcode_bitfield'		=> '',
			'shout_bbcode_flags'		=> 0,
			'shout_robot'				=> 6,
			'shout_robot_user'			=> (int)$event['user_id'],
			'shout_forum'				=> 0,
			'shout_info'				=> 13,
		);
		
		if ($this->config['shout_newest'])
		{
			$sql = 'INSERT INTO ' .$this->shoutbox_table. ' ' . $this->db->sql_build_array('INSERT', $sql_data);
			$this->db->sql_query($sql);
			set_config_count('shout_nr', 1, true);
		}
		if ($this->config['shout_newest_priv'])
		{
			$sql = 'INSERT INTO ' .$this->shoutbox_priv_table. ' ' . $this->db->sql_build_array('INSERT', $sql_data);
			$this->db->sql_query($sql);
			set_config_count('shout_nr_priv', 1, true);
		}
	}

	/*
	* Build tables for the select sounds
	* in the user shout panel
	* Must be construct here to include javascript
	* Ability to add new sounds automatically
	*/
	public function build_sound_select($sound_user, $sort, $no = false, $is = false)
	{
		if (!function_exists('filelist'))
		{
			include ($this->root_path . 'includes/functions_admin.' . $this->php_ext);
		}
		
		$soundlist = @filelist($this->ext_path. '/images/sounds/', '', 'mp3');
		$width 	= 150;
		$height = 35;
		$nb_cols = 5;
		$cols 	= $i = 0;
		$larg 	= $width*$nb_cols;
		$tpl_1	= '<table cellspacing="5" cellpadding="2" border="0" width="' .$larg. '">'."\n";
		$tpl_2	= '<tr style="height:' .$height. 'px;">';
		$tpl_3	= '<td style="width:' .$width. 'px;height:' .$height. 'px;">'."\n";
		$tpl_4	= '<td><label>';
		
		$sort_name 	= $sort.'_sound';
		$sort_span 	= 'sound_'.$sort;
		switch ($sort)
		{
			case 'new':
				$displayInfos = $displayInfos2 = '';
			break;
			case 'error':
				$displayInfos = "displayInfos('info3_no', 'info3');";
				$displayInfos2 = "displayInfos('info3', 'info3_no');";
			break;
			case 'del':
				$displayInfos = "displayInfos('info4_no', 'info4');";
				$displayInfos2 = "displayInfos('info4', 'info4_no');";
			break;
			case 'add':
				$displayInfos = "displayInfos('info5_no', 'info5');";
				$displayInfos2 = "displayInfos('info5', 'info5_no');";
			break;
			case 'edit':
				$displayInfos = "displayInfos('info6_no', 'info6');";
				$displayInfos2 = "displayInfos('info6', 'info6_no');";
			break;
		}
		$is			= ($is == 1) ? ':' : (($is == 2) ? 'A' : '');
		$select_no	= ($sound_user == '0') ? ' checked="checked"' : '';
		$onclick	= ' onclick="' .$displayInfos. 'playSoundUser(\'discretion.mp3\');"';
		$select_sound = $tpl_1. "\n";
		$select_sound .= ($no) ? $tpl_2.$tpl_4. "\n" : '';
		$select_sound .= ($no) ? ' ' .$this->shout_img('woofer.png', 'SHOUT_ANY', 20, 20, $this->ext_path_web. 'images/') : '';
		$select_sound .= ($no) ? ' <input type="radio" name="' .$sort_name. '"' .$onclick. ' value="0"' .$select_no. ' /> <strong>' .$this->user->lang['SHOUT_ANY']. '</strong></label></td></tr>'."\n" : '';
		$select_sound .= $tpl_2."\n";
		$soundlist = array_values($soundlist);
		foreach ($soundlist as $key => $sounds)
		{
			asort($sounds);
			foreach ($sounds as $_sounds)
			{
				if ($nb_cols - $cols == 0)
				{
					$select_sound .= "</tr>\n";
					$select_sound .= $tpl_2."\n";
					$cols = 0;
				}
				$name_sounds = str_replace('.mp3', '', $_sounds);
				$selected	= ($_sounds == $sound_user) ? ' checked="checked"' : '';
				$infos_2	= $no ? $displayInfos2 : '';
				$select_sound .= $tpl_3."\n";
				$select_sound .= '<a style="cursor:pointer;" onclick="playSoundUser(\'' .$_sounds. '\');return false;">' .$this->shout_img('woofer.png', 'SHOUT_SOUND_ECOUTE', 20, 20, $this->ext_path_web. 'images/'). '</a>'."\n";
				$select_sound .= '<label><input type="radio" id="' .$is.$name_sounds. '" name="' .$sort_name. '" onclick="' .$infos_2. 'changeValue(this.id,\'' .$sort_span. '\');playSoundUser(\'' .$_sounds. '\');" value="' .$_sounds. '"' .$selected. ' /> ' .$name_sounds. "</label></td>\n";
				$cols++;
				$i++;
			}
		}
		$select_sound .= "</tr></table>\n";
		
		return $select_sound;
	}
	
	public function build_user_sound_select($actual, $sort, $no = false)
	{
		if (!function_exists('filelist'))
		{
			include ($this->root_path . 'includes/functions_admin.' . $this->php_ext);
		}
		$soundlist = @filelist($this->ext_path. '/images/sounds/', '', 'mp3');
		
		if (sizeof($soundlist))
		{
			$select = ($actual != 0) ? '' : ' selected="selected"';
			$sound_select = '<select title="' .$actual. '" id="shout_sound_' .$sort. '" name="shout_sound_' .$sort. '" onchange="playSoundUser(this.value);changeValue(this.value,\'sound_' .$sort. '\',this.id);">'."\n";
			$sound_select .= $no ? '<option value="0"' .$select. '>' .$this->user->lang['SHOUT_SOUND_EMPTY']. '</option>'."\n" : '';
			$soundlist = array_values($soundlist);
			foreach ($soundlist as $key => $sounds)
			{
				asort($sounds);
				foreach ($sounds as $sound)
				{
					$name_sound = str_replace('.mp3', '', $sound);
					$selected = ($sound != $actual) ? '' : ' selected="selected"';
					$sound_select .= '<option value="' .$sound. '"' .$selected. '>' .$name_sound. "</option>\n";
				}
			}
			$sound_select .= "</select>\n";
			return $sound_select;
		}
		return false;
	}

	/*
	* Build radio with specific lang
	* Add play sound and class if needed
	*/
	public function construct_radio_shout($name, $sort = 1, $inline = true)
	{
		switch ($sort)
		{
			case 1:
				$title1 = 'YES';
				$title2 = 'NO';
			break;
			case 2:
			case 3:
			case 4:
				$title1 = 'ENABLE';
				$title2 = 'DISABLE';
			break;
		}
		
		$check1 = (isset($this->config[$name]) && $this->config[$name]) ? ' checked="checked" id="' .$name. '"' : '';
		$check2 = (isset($this->config[$name]) && !$this->config[$name]) ? ' checked="checked" id="' .$name. '"' : '';
		$return = '<label><input type="radio" class="radio" name="' .$name. '" value="1"' .$check1. ' /> ' .$this->user->lang[$title1]. '</label>';
		$return .= $inline ? '<br /><br />' : '';
		$return .= ' <label><input type="radio" class="radio" name="' .$name. '" value="0"' .$check2. ' /> ' .$this->user->lang[$title2]. '</label>';
		
		return $return;
	}

	/*
	* Build select for infos hour
	* 24 hours format only
	*/
	public function hour_select($value = '09', $select_name)
	{
		$select = '<select id="' .$select_name. '" name="' .$select_name. '">';
		for ($i = 1; $i <= 24; $i++)
		{
			$in = array('1', '2', '3', '4', '5', '6', '7', '8', '9');
			$out = array('01', '02', '03', '04', '05', '06', '07', '08', '09');
			$selected = ($i == $value) ? ' selected="selected"' : '';
			$i = ($i < 10) ? str_replace($in, $out, $i) : $i;
			$select .= '<option value="' .$i. '"' .$selected. '>' .$i. "</option>\n";
		}
		$select .= '</select>';

		return $select;
	}

	/*
	* Build images with traductions
	*/
	public function shout_img($src, $lang, $w = false, $h = false, $another = false, $alt = false, $style = false, $class = false)
	{
		$lang = isset($this->user->lang[$lang]) ? $this->user->lang[$lang] : $lang;
		$alt = $alt ? (isset($this->user->lang[$alt]) ? $this->user->lang[$alt] : $alt) : $lang;
		$src_img = $another ? $another : $this->complete_shout_url(). 'images/';
		$img_src = 'src="' .$src_img.$src. '"';
		$alt 	= ' alt="' .$alt. '"';
		$title 	= ' title="' .$lang. '"';
		$width 	= $w ? ' width="' .$w. '"' : '';
		$height = $h ? ' height="' .$h. '"' : '';
		$style 	= $style ? ' style="' .$style. '"' : '';
		$class 	= $class ? ' class="' .$class. '"' : '';
		
		return '<img ' .$img_src.$alt.$title.$width.$height.$style.$class. ' /> ';
	}

	/*
	* Display user avatar with resizing
	* Add avatar type for robot, users with no avatar and anonymous
	* Add title with username
	*/
	public function shout_user_avatar($id, $row = false, $_height = false, $force = false)
	{
		if ($force == false)
		{
			if (!$this->user->optionget('viewavatars') || !$this->config['shout_avatar'] || !$this->config['allow_avatar'])
			{
				return '';
			}
		}
		$avatar_height = $_height ? $_height : $this->config['shout_avatar_height'];
		if ($id == 0 && $this->config['shout_avatar_robot'])
		{
			$avatar = $this->complete_shout_url(). 'images/' .$this->config['shout_avatar_img_robot'];
			$alt = sprintf($this->user->lang['SHOUT_AVATAR_TITLE'], 'Robot');
		}
		else if ($id == ANONYMOUS && $this->config['shout_avatar_user'])
		{
			$avatar = $this->complete_shout_url(). 'images/anonym.png';
			$alt = sprintf($this->user->lang['SHOUT_AVATAR_TITLE'], 'Anonymous');
		}
		else if ($row['user_id'] && !$row['user_avatar'] && $this->config['shout_avatar_user'])
		{
			$avatar = $this->complete_shout_url(). 'images/anonym.png';
			$alt = sprintf($this->user->lang['SHOUT_AVATAR_NONE'], $row['username']);
		}
		else if ($row['user_id'] && $row['user_avatar'])
		{
			if ($row['user_avatar_width'] >= $row['user_avatar_height'])
			{
				$width = $avatar_height;
				$height = round($avatar_height/($row['user_avatar_width']*$row['user_avatar_height']));
			}
			else
			{
				$width = round($avatar_height/($row['user_avatar_height']*$row['user_avatar_width']));
				$height = $avatar_height;
			}
			$row['user_avatar_width'] = $width;
			$row['user_avatar_height'] = $height;
			$avatar = phpbb_get_user_avatar($row, sprintf($this->user->lang['SHOUT_AVATAR_TITLE'], $row['username']));
			$avatar = str_replace('alt="' .sprintf($this->user->lang['SHOUT_AVATAR_TITLE'], $row['username']). '"', 'alt="' .sprintf($this->user->lang['SHOUT_AVATAR_TITLE'], $row['username']). '" title="' .sprintf($this->user->lang['SHOUT_AVATAR_TITLE'], $row['username']). '"', $avatar);
			return '<span class="avatar-shout">' .$avatar. '</span>';
		}
		else
		{
			return;
		}
		$avatar = $this->shout_img('', $alt, $avatar_height, $avatar_height, str_replace(' ', '%20', $avatar), 'SHOUT_AVATAR_SHORT', false, 'avatar');
		return '<span class="avatar-shout">' .$avatar. '</span>';
	}

	public function build_adm_sound_select($sort)
	{
		$actual = $this->config['shout_sound_' .$sort];
		$soundlist = @filelist($this->ext_path. '/images/sounds/', '', 'mp3');
		if (sizeof($soundlist))
		{
			$select = (!$actual) ? ' selected="selected"' : '';
			$sound_select = '<option value="0"' .$select. '>' .$this->user->lang['SHOUT_SOUND_EMPTY']. '</option>';
			$soundlist = array_values($soundlist);
			foreach ($soundlist as $key => $sounds)
			{
				asort($sounds);
				foreach ($sounds as $_sounds)
				{
					$name_sounds = str_replace('.mp3', '', $_sounds);
					$selected = ($_sounds == $actual) ? ' selected="selected"' : '';
					$sound_select .= '<option title="' .$name_sounds. '" value="' .$_sounds. '"' .$selected. '>' .$name_sounds. "</option>\n";
				}
			}
			return $sound_select;
		}
		return false;
	}
	
	public function plural($nr)
	{
		if ($nr > 1)
		{
			return 'S';
		}
		else
		{
			return;
		}
	}
	
	/**
	* Select shoutbox default dateformat
	*/
	public function shout_dateformat_select($value)
	{
		// Let the format_date function operate with the acp values
		$old_tz = $this->user->timezone;
		$old_dst = $this->user->dst;

		$dateformat_options = '';
		foreach ($this->user->lang['dateformats'] as $format => $null)
		{
			$dateformat_options .= '<option value="' . $format . '"' . (($format == $value) ? ' selected="selected"' : '') . '>';
			$dateformat_options .= $this->user->format_date(time(), $format, false) . ((strpos($format, '|') !== false) ? $this->user->lang['VARIANT_DATE_SEPARATOR'] . $this->user->format_date(time(), $format, true) : '');
			$dateformat_options .= '</option>';
		}

		$dateformat_options .= '<option value="custom"';
		if (!isset($this->user->lang['dateformats'][$value]))
		{
			$dateformat_options .= ' selected="selected"';
		}
		$dateformat_options .= '>' . $this->user->lang['CUSTOM_DATEFORMAT'] . '</option>';

		// Reset users date options
		$this->user->timezone = $old_tz;
		$this->user->dst = $old_dst;

		return "<select name=\"shout_dateformat\" id=\"shout_dateformat\" onchange=\"if(this.value=='custom'){document.getElementById('shout_dateformat2').value='" .addslashes($value). "';}else{document.getElementById('shout_dateformat2').value=this.value;}\">$dateformat_options</select>
		<input type=\"text\" name=\"shout_dateformat2\" id=\"shout_dateformat2\" value=\"$value\" maxlength=\"30\" />";
	}
	
	public function purge_shout_admin($sort, $priv = false)
	{
		$_table = $priv ? $this->shoutbox_priv_table : $this->shoutbox_table;
		$_priv = $priv ? '_priv' : '';
		$_Priv = $priv ? '_PRIV' : '';
		
		$sql = 'DELETE FROM ' . $_table . ' 
			WHERE shout_robot = 1 
				OR shout_robot = ' .(int)$sort;
		$this->db->sql_query($sql);
		$deleted = $this->db->sql_affectedrows();
		
		if ($deleted)
		{
			set_config_count('shout_del_purge' .$_priv, $deleted, true);
			add_log('admin', 'LOG_PURGE_SHOUTBOX' .$_Priv. '_ROBOT');
			$this->post_robot_shout(0, $this->user->ip, ($priv ? true : false), true, true);
			return false;
		}
		else
		{
			return true;
		}
	}
	
	public function build_select_img($list, $sort, $first = true, $panel = false)
	{
		$selected = $select = '';
		foreach ($list as $key => $_img)
		{
			asort($_img);
			foreach ($_img as $img)
			{
				$img_id = $img;
				$img = substr($img, 0, strrpos($img, '.'));
				$imgId = $first ? ':' .$img : 'a:' .$img;
				$value = $panel ? $img_id : $img;
				if (isset($this->config[$sort]) && $this->config[$sort])
				{
					$selected = ($this->config[$sort] == $value) ? ' selected="selected"' : '';
				}
				$select .= '<option id="' .htmlspecialchars($imgId). '" title="' .htmlspecialchars($img). '" value="' .htmlspecialchars($value). '"' .$selected. '>' .htmlspecialchars($img). "</option>\n";
			}
		}
		return $select;
	}
	
	public function active_config_shoutbox()
	{
		if ($this->user->data['user_id'] == ANONYMOUS || $this->user->data['is_bot'] || !$this->auth->acl_get('u_shout_post'))
		{
			trigger_error($this->user->lang['NO_ADMIN_PERM']);
		}
		
		$this->user->add_lang(array('ucp'));
		$this->javascript_shout(2);
		$this_path = generate_board_url(). str_replace(array('..', '.', '//'), array('', '', '/'), $this->ext_path_web);
		if ($this->request->is_set_post('submit'))
		{
			$new_sound 			= utf8_normalize_nfc($this->request->variable('shout_sound_new', '', true));
			$error_sound 		= utf8_normalize_nfc($this->request->variable('shout_sound_error', '', true));
			$del_sound 			= utf8_normalize_nfc($this->request->variable('shout_sound_del', '', true));
			$add_sound 			= utf8_normalize_nfc($this->request->variable('shout_sound_add', '', true));
			$edit_sound 		= utf8_normalize_nfc($this->request->variable('shout_sound_edit', '', true));
			$new_sound_priv 	= utf8_normalize_nfc($this->request->variable('shout_sound_new_priv', '', true));
			$user_correct 		= $this->request->variable('user_correct', 2);
			$out_index 			= $this->request->variable('pos_index', 3);
			$out_forum 			= $this->request->variable('pos_forum', 3);
			$out_topic 			= $this->request->variable('pos_topic', 3);
			$shout_bar			= $this->request->variable('shout_bar', 0);
			$shout_pagin		= $this->request->variable('shout_pagin', 0);
			$shout_bar_pop		= $this->request->variable('shout_bar_pop', 0);
			$shout_pagin_pop	= $this->request->variable('shout_pagin_pop', 0);
			$shout_bar_priv		= $this->request->variable('shout_bar_priv', 0);
			$shout_pagin_priv	= $this->request->variable('shout_pagin_priv', 0);
			$shout_defil		= $this->request->variable('shout_defil', 0);
			$shout_dateformat	= $this->request->variable('dateformat', 'N', true);
			
			$config_user_shout	= implode('||', array(
					$user_correct,
					$new_sound,
					$error_sound,
					$del_sound,
					$add_sound,
					$edit_sound,
					$new_sound_priv,
					$out_index,
					$out_forum,
					$out_topic,
				));
			$config_user_shoutbox = implode(';', array(
					$shout_bar,
					$shout_pagin,
					$shout_bar_pop,
					$shout_pagin_pop,
					$shout_bar_priv,
					$shout_pagin_priv,
					$shout_defil,
					$shout_dateformat,
				));
			
			$sql = 'UPDATE ' . USERS_TABLE . " 
				SET user_shout = '" .$this->db->sql_escape($config_user_shout). "' 
				WHERE user_id = " .(int)$this->user->data['user_id'];
			$this->db->sql_query($sql);
			
			$sql = 'UPDATE ' . USERS_TABLE . " 
				SET user_shoutbox = '" .$this->db->sql_escape($config_user_shoutbox). "' 
				WHERE user_id = " .(int)$this->user->data['user_id'];
			$this->db->sql_query($sql);
			
			redirect($this->helper->route('sylver35_breizhshoutbox_controller_configshout'));
		}
		else if ($this->request->is_set_post('retour'))
		{
			$sql = 'UPDATE ' . USERS_TABLE . " 
				SET user_shout = '2||0||0||0||0||0||0||3||3||3' 
				WHERE user_id = " .(int)$this->user->data['user_id'];
			$this->db->sql_query($sql);
			
			$sql = 'UPDATE ' . USERS_TABLE . " 
				SET user_shoutbox = 'N;N;N;N;N;N;N;N' 
				WHERE user_id = " .(int)$this->user->data['user_id'];
			$this->db->sql_query($sql);
			
			redirect($this->helper->route('sylver35_breizhshoutbox_controller_configshout'));
		}

		list(
			$correct,
			$sound_new,
			$sound_error,
			$sound_del,
			$sound_add,
			$sound_edit,
			$sound_new_priv,
			$_index,
			$_forum,
			$_topic,
		) = explode('||', $this->user->data['user_shout']);
		list(
			$shout_bar,
			$shout_pagin,
			$shout_bar_pop,
			$shout_pagin_pop,
			$shout_bar_priv,
			$shout_pagin_priv,
			$shout_defil,
			$shout_dateformat,
		) = explode(';', $this->user->data['user_shoutbox']);
		
		$correct = ($correct == '2') ? ($this->config['shout_sound_on'] ? 0 : 1) : $correct;
		$sound_new = ($sound_new == '0') ? $this->config['shout_sound_new'] : $sound_new;
		$sound_error = ($sound_error == '0') ? $this->config['shout_sound_error'] : $sound_error;
		$sound_del = ($sound_del == '0') ? $this->config['shout_sound_del'] : $sound_del;
		$sound_add = ($sound_add == '0') ? $this->config['shout_sound_add'] : $sound_add;
		$sound_edit = ($sound_edit == '0') ? $this->config['shout_sound_edit'] : $sound_edit;
		$sound_new_priv = ($sound_new_priv == '0') ? $this->config['shout_sound_new_priv'] : $sound_new_priv;
		$_index = ($_index == '3') ? $this->config['shout_position_index'] : $_index;
		$_forum = ($_forum == '3') ? $this->config['shout_position_forum'] : $_forum;
		$_topic = ($_topic == '3') ? $this->config['shout_position_topic'] : $_topic;
		$shout_bar = ($shout_bar != 'N') ? $shout_bar : $this->config['shout_bar_option'];
		$shout_pagin = ($shout_pagin != 'N') ? $shout_pagin : $this->config['shout_pagin_option'];
		$shout_bar_pop = ($shout_bar_pop != 'N') ? $shout_bar_pop : $this->config['shout_bar_option_pop'];
		$shout_pagin_pop = ($shout_pagin_pop != 'N') ? $shout_pagin_pop : $this->config['shout_pagin_option_pop'];
		$shout_bar_priv = ($shout_bar_priv != 'N') ? $shout_bar_priv : $this->config['shout_bar_option_priv'];
		$shout_pagin_priv = ($shout_pagin_priv != 'N') ? $shout_pagin_priv : $this->config['shout_pagin_option_priv'];
		$shout_defil = ($shout_defil != 'N') ? $shout_defil : $this->config['shout_defil'];
		
		if (strpos($shout_dateformat, '|') !== false)
		{
			$enable_sound = sprintf($this->user->lang['MINUTES_CHOICE'], $this->user->format_date((time() - 60*50), $shout_dateformat));
			$is_minutes = true;
		}
		else
		{
			$enable_sound = $this->user->lang['DISPLAY_SOUND_CHOICE'];
			$is_minutes = false;
		}
		$shout_dateformat = ($shout_dateformat != 'N') ? $shout_dateformat : $this->config['shout_dateformat'];
		
		$dateformat_options = '';
		foreach ($this->user->lang['dateformats'] as $format => $null)
		{
			$dateformat_options .= '<option value="' . $format . '"' . (($format == $shout_dateformat) ? ' selected="selected"' : '') . '>';
			$dateformat_options .= $this->user->format_date(time(), $format, false) . ((strpos($format, '|') !== false) ? $this->user->lang['VARIANT_DATE_SEPARATOR'] . $this->user->format_date(time(), $format, true) : '');
			$dateformat_options .= '</option>';
		}

		$s_custom = false;
		$dateformat_options .= '<option value="custom"';
		if (!isset($this->user->lang['dateformats'][$shout_dateformat]))
		{
			$dateformat_options .= ' selected="selected"';
			$s_custom = true;
		}
		$dateformat_options .= '>' . $this->user->lang['CUSTOM_DATEFORMAT'] . '</option>';
		
		$this->template->assign_vars(array(
			'S_PRIVATE'				=> $this->auth->acl_get('u_shout_priv') ? true : false,
			'EXPLAIN_DATE'			=> $enable_sound,
			'USER_CORRECT'			=> $correct,
			'USER_CORRECT_DISP'		=> $correct ? 'none' : 'block',
			'USER_CORRECT_DISP2'	=> $correct ? 'block' : 'none',
			'SOUND_DEL_DISP'		=> $sound_del ? 'block' : 'none',
			'SOUND_DEL_DISP2'		=> $sound_del ? 'none' : 'block',
			'SOUND_ERROR_DISP'		=> $sound_error ? 'block' : 'none',
			'SOUND_ERROR_DISP2'		=> $sound_error ? 'none' : 'block',
			'SOUND_ADD_DISP'		=> $sound_add ? 'block' : 'none',
			'SOUND_ADD_DISP2'		=> $sound_add ? 'none' : 'block',
			'SOUND_EDIT_DISP'		=> $sound_edit ? 'block' : 'none',
			'SOUND_EDIT_DISP2'		=> $sound_edit ? 'none' : 'block',
			'NO_SOUND_DEL'			=> $sound_del ? false : true,
			'NO_SOUND_ERROR'		=> $sound_error ? false : true,
			'NO_SOUND_ADD'			=> $sound_add ? false : true,
			'NO_SOUND_EDIT'			=> $sound_edit ? false : true,
			'DATE_CORRECT'			=> $is_minutes,
			'LANG_CORRECT'			=> $is_minutes ? $this->user->lang['SHOUT_SOUND_MINUTES'] : $this->user->lang['SHOUT_SOUND_NO'],
			'NEW_SOUND'				=> $this->build_user_sound_select($sound_new, 'new', false),
			'ERROR_SOUND'			=> $this->build_user_sound_select($sound_error, 'error', true),
			'DEL_SOUND'				=> $this->build_user_sound_select($sound_del, 'del', true),
			'ADD_SOUND'				=> $this->build_user_sound_select($sound_add, 'add', true),
			'EDIT_SOUND'			=> $this->build_user_sound_select($sound_edit, 'edit', true),
			'NEW_SOUND_PRIV'		=> $this->build_user_sound_select($sound_new_priv, 'new_priv', true),
			'USER_SOUND'			=> $sound_new,
			'USER_SOUND_E'			=> $sound_error,
			'USER_SOUND_D'			=> $sound_del,
			'USER_SOUND_A'			=> $sound_add,
			'USER_SOUND_ED'			=> $sound_edit,
			'USER_SOUND_PRIV'		=> $this->auth->acl_get('u_shout_priv') ? $sound_new_priv : '',
			'USER_SOUND_INFO'		=> str_replace('.mp3', '', $sound_new),
			'USER_SOUND_INFO_E'		=> str_replace('.mp3', '', $sound_error),
			'USER_SOUND_INFO_D'		=> str_replace('.mp3', '', $sound_del),
			'USER_SOUND_INFO_A'		=> str_replace('.mp3', '', $sound_add),
			'USER_SOUND_INFO_ED'	=> str_replace('.mp3', '', $sound_edit),
			'USER_SOUND_INFO_PRIV'	=> str_replace('.mp3', '', $sound_new_priv),
			'SHOUT_BAR'				=> $shout_bar,
			'SHOUT_PAGIN'			=> $shout_pagin,
			'SHOUT_BAR_POP'			=> $shout_bar_pop,
			'SHOUT_PAGIN_POP'		=> $shout_pagin_pop,
			'SHOUT_BAR_PRIV'		=> $shout_bar_priv,
			'SHOUT_PAGIN_PRIV'		=> $shout_pagin_priv,
			'SHOUT_DEFIL'			=> $shout_defil,
			'IS_INDEX'				=> $this->config['shout_index'] ? true : false,
			'IS_FORUM'				=> $this->config['shout_forum'] ? true : false,
			'IS_TOPIC'				=> $this->config['shout_topic'] ? true : false,
			'ON_INDEX'				=> $_index,
			'ON_FORUM'				=> $_forum,
			'ON_TOPIC'				=> $_topic,
			'W_INDEX'				=> $this->user->lang['SHOUT_POSITION_' .($_index ? (($_index == 1) ? 'AFTER' : 'END') : 'TOP')],
			'W_FORUM'				=> $this->user->lang['SHOUT_POSITION_' .($_forum ? 'END' : 'TOP')],
			'W_TOPIC'				=> $this->user->lang['SHOUT_POSITION_' .($_topic ? 'END' : 'TOP')],
			'DATE_FORMAT'			=> $shout_dateformat,
			'DATE_FORMAT_EX'		=> $this->user->format_date((time() - 60*60*60), $shout_dateformat),
			'S_DATEFORMAT_OPTIONS'	=> $dateformat_options,
			'S_CUSTOM_DATEFORMAT'	=> $s_custom,
			'DEFAULT_DATEFORMAT'	=> $shout_dateformat,
			'PERM_POP'				=> $this->auth->acl_get('u_shout_popup'),
			'PERM_PRIV'				=> $this->auth->acl_get('u_shout_priv'),
			'SHOUT_EXT_PATH'		=> $this_path,
			'SHOUT_SOUNDS_PATH'		=> $this_path. 'images/sounds/',
			'U_SHOUT_ACTION'		=> $this->helper->route('sylver35_breizhshoutbox_controller_configshout'),
			'SHOUTBOX_VERSION'		=> sprintf($this->user->lang['SHOUTBOX_VERSION_ACP_COPY'], $this->config['shout_board_url'], $this->config['shout_version']),
		));
	}

	public function javascript_shout($shout, $onShout = false)
	{
		$private = false;
		switch ($shout)
		{
			case 1: // Popup shoutbox
				$sort = $Priv = '';
				$param = '&s=1';
				$sort_p = '_pop';
				$sort_auth = 'view';
				$onRules = '&pr=0';
				$auth_shout = 'manage';
				$sort_title = $this->user->lang['ACP_SHOUT_POPUP_CAT'];
			break;
			case 2: // Normal shoutbox
				$sort = $sort_p = $Priv = '';
				$param = '&s=2';
				$sort_auth = 'view';
				$onRules = '&pr=0';
				$auth_shout = 'manage';
				$sort_title = $this->user->lang['ACP_SHOUT_PRINCIPAL_CAT'];
			break;
			case 3: // Private shoutbox
				$param = '&s=3';
				$sort = $sort_p = '_priv';
				$sort_auth = 'priv';
				$Priv = '_PRIV';
				$onRules = '&pr=1';
				$auth_shout = 'priv';
				$private = true;
				$sort_title = $this->user->lang['SHOUTBOX_SECRET'];
			break;
		}

		// Prevent direct entries and exit if not enabled...
		if (!$this->auth->acl_get('u_shout_' .$sort_auth) || !$this->config['shout_enable'])
		{
			exit_handler();
			return;
		}

		// for work fine everywhere wee want
		$this_path = generate_board_url(). '/' .str_replace(array('./../../../../', './../../../', './../../', './../', './'), '', $this->ext_path_web);

		// See compatibles browsers to change the display
		$compatible = ($this->compatibles_browsers() < 3) ? 'false' : 'true';

		// Real member or not
		$is_user = ($this->user->data['user_id'] != ANONYMOUS && !$this->user->data['is_bot']) ? true : false;

		// The number of messages to display
		$shout_number = ($compatible == 'true') ? $this->config['shout_non_ie_nr' .$sort_p] : $this->config['shout_ie_nr' .$sort_p];

		// Display the rules
		$rules = ($this->config['shout_rules'] && isset($this->config['shout_rules_' .$this->user->lang_name])) ? (($this->config['shout_rules_' .$this->user->lang_name]) ? 'true' : 'false') : 'false';

		// Temp of refresh for differents users
		$refresh = ($this->user->data['is_bot'] ? 500 : (!$this->user->data['is_registered'] ? $this->config['shout_temp_anonymous'] : $this->config['shout_temp_users']))*1000;

		// Temp of inactivity for differents users
		$inactiv = $is_user ? ($this->auth->acl_get('u_shout_inactiv') ? 0 : $this->config['shout_inactiv_member']) : $this->config['shout_inactiv_anony'];
		$inactiv = $inactiv ? round($inactiv * 60 / ($refresh / 1000)) : 0;

		// Title shout
		$this->config['shout_title'] = $this->config['shout_title'] ? $this->config['shout_title'] : $this->user->lang['SHOUT_START'];

		// load first message to display
		$message = $this->user->lang['LOADING'];

		// Construct the user's preferences
		if ($is_user)
		{
			list(
				$correct,
				$new_sound,
				$error_sound,
				$del_sound,
				$add_sound,
				$edit_sound,
				$new_sound_priv,
				$_index,
				$_forum,
				$_topic,
			) = explode('||', $this->user->data['user_shout']);
			
			list(
				$shout_bar,
				$shout_pagin,
				$shout_bar_pop,
				$shout_pagin_pop,
				$shout_bar_priv,
				$shout_pagin_priv,
				$shout_defil,
				$shout_dateformat,
			) = explode(';', $this->user->data['user_shoutbox']);
			
			$correct = ($correct == '2') ? ($this->config['shout_sound_on'] ? 0 : 1) : (int)$correct;
			$new_sound = ($new_sound == '0') ? $this->config['shout_sound_new'] : $new_sound;
			$error_sound = ($error_sound == '0') ? $this->config['shout_sound_error'] : (($error_sound == '0') ? '' : $error_sound);
			$del_sound = ($del_sound == '0') ? $this->config['shout_sound_del'] : (($del_sound == '0') ? '' : $del_sound);
			$add_sound = ($add_sound == '0') ? $this->config['shout_sound_add'] : (($add_sound == '0') ? '' : $add_sound);
			$edit_sound = ($edit_sound == '0') ? $this->config['shout_sound_edit'] : (($edit_sound == '0') ? '' : $edit_sound);
			$new_sound_priv = ($new_sound_priv == '0') ? $this->config['shout_sound_new_priv'] : $new_sound_priv;
			$this->config['shout_bar_option'] = ($shout_bar != 'N') ? $shout_bar : $this->config['shout_bar_option'];
			$this->config['shout_bar_option_pop'] = ($shout_bar_pop != 'N') ? $shout_bar_pop : $this->config['shout_bar_option_pop'];
			$this->config['shout_bar_option_priv'] = ($shout_bar_priv != 'N') ? $shout_bar_priv : $this->config['shout_bar_option_priv'];
			$this->config['shout_pagin_option'] = ($shout_pagin != 'N') ? $shout_pagin : $this->config['shout_pagin_option'];
			$this->config['shout_pagin_option_pop'] = ($shout_pagin_pop != 'N') ? $shout_pagin_pop : $this->config['shout_pagin_option_pop'];
			$this->config['shout_pagin_option_priv'] = ($shout_pagin_priv != 'N') ? $shout_pagin_priv : $this->config['shout_pagin_option_priv'];
			$this->config['shout_defil'] = ($shout_defil != 'N') ? $shout_defil : $this->config['shout_defil'];
			$this->config['shout_dateformat'] = ($shout_dateformat != 'N') ? $shout_dateformat : $this->config['shout_dateformat'];
		}
		else
		{
			$correct = $this->config['shout_sound_on'] ? 0 : 1;
			if (!$this->user->data['is_bot'] && isset($_COOKIE[$this->config['cookie_name']. '_shout']))
			{
				$cookie_sound = request_var($this->config['cookie_name']. '_shout', 'on', false, true);
				$this->config['shout_sound_on'] = ($cookie_sound == 'on') ? true : false;
				$correct = $this->config['shout_sound_on'] ? 0 : 1;
			}
			if (!$this->user->data['is_bot'])
			{
				$new_sound = $this->config['shout_sound_new'];
				$error_sound = $this->config['shout_sound_error'];
				$del_sound = $this->config['shout_sound_del'];
				$add_sound = $this->config['shout_sound_add'];
				$edit_sound = $this->config['shout_sound_edit'];
				$new_sound_priv = '';
			}
			else
			{
				$correct = 1; // No sounds for bots, they have no ears ;)
				$new_sound = $error_sound = $del_sound = $add_sound = $edit_sound = $new_sound_priv = '';
			}
			
		}
		$this->config['shout_title'] = (!$this->config['shout_title']) ? $this->user->lang['SHOUT_START'] : $this->config['shout_title'];
		
		if ($this->user->data['user_id'] == ANONYMOUS && !$this->user->data['is_bot'])
		{
			$replace = isset($_COOKIE[$this->config['cookie_name'].'_shout']) ? (($this->request->variable($this->config['cookie_name']. '_shout', 'on', false, true) == 'on') ? '0' : '1') : ($this->config['shout_sound_on'] ? '0' : '1');
		}
		else if (isset($this->user->data['user_shout']) && !$this->user->data['is_bot'])
		{
			$user_shout = explode('||', $this->user->data['user_shout']);
			$replace = $user_shout[0];
		}
		else
		{
			$replace = '1';
		}
		$robot = '<span style="color:#' .$this->config['shout_color_robot']. ';font-weight:bold;">' .$this->user->lang['SHOUT_ROBOT']. '</span>';
		
		$shout_creator = false;
		$lang_smilie_creator = '';
		$smilecreator_controller = '';
		if (isset($this->config['smiliecreator_version']))
		{
			$shout_creator = true;
			$lang_smilie_creator = $this->user->lang['SMILIE_CREATOR'];
			$smilecreator_controller = $this->helper->route('sylver35_smilecreator_controller');
		}

		// Construct global vars
		$this->template->assign_vars(array(
			'ON_SHOUT_DISPLAY'			=> $onShout,
			'S_ON_PRIVATE'				=> $private,
			'shout_replace'				=> $replace,
			'shout_title'				=> sprintf($this->config['shout_version_url'], $this->config['shout_title' .$sort]),
			'shout_inactivity'			=> $inactiv,
			'shout_temps' 				=> $refresh,
			'shout_per_page'			=> $shout_number,
			'shout_is_priv' 			=> $private ? 'true' : 'false',
			'shout_go_sound'			=> !$correct ? 'true' : 'false',
			'shout_correct_on' 			=> !$correct ? 'on' : 'off',
			'shout_isGuest' 			=> !$this->user->data['is_registered'] ? 'true' : 'false',
			'shout_userId'				=> $this->user->data['user_id'],
			'shout_cookie_name' 		=> $this->config['cookie_name'].'_',
			'shout_cookie_domain' 		=> '; domain='.$this->config['cookie_domain'].($this->config['cookie_secure'] ? '; secure' : ''),
			'shout_cookie_path' 		=> '; path='.$this->config['cookie_path'],
			'shout_extension_url' 		=> $this->complete_shout_url(), 
			'shout_soundClass'			=> ('button_shout_sound'.(($correct == 1) ? '_off' : '')),
			'shout_sound' 				=> !$private ? $new_sound : $new_sound_priv,
			'shout_error_sound' 		=> $error_sound,
			'shout_del_sound' 			=> $del_sound,
			'shout_add_sound' 			=> $add_sound,
			'shout_edit_sound' 			=> $edit_sound,
			'shout_rules_ok' 			=> $rules,
			'shout_max_post_shout' 		=> $this->config['shout_max_post_chars'], 
			'shout_sort_shout' 			=> ($shout == 3) ? 'priv' : (($shout == 1) ? 'popup' : 'normal'),
			'shout_lang_direction' 		=> ($this->user->lang['DIRECTION'] == 'ltr') ? 'left' : 'right', 
			'shout_button_bg' 			=> 'button_background_'.$this->config['shout_color_background'.$sort_p], 
			'shout_is_compatible' 		=> $compatible, 
			'shout_message_on' 			=> htmlspecialchars($message), 
			'shout_shout_height' 		=> ($shout == 2) ? $this->config['shout_height'] : $this->config['shout_non_ie_height'.$sort_p],  
			'shout_width_post' 			=> $this->config['shout_width_post'.$sort_p].'px', 
			'shout_shout_popup_width' 	=> $this->config['shout_popup_width'], 
			'shout_shout_popup_height' 	=> $this->config['shout_popup_height'], 
			'shout_shout_smilies_width' => $this->config['shout_smilies_width'], 
			'shout_shout_smilies_height' => $this->config['shout_smilies_height'], 
			'shout_print_ver' 			=> sprintf($this->user->lang['SHOUTBOX_VER'], $this->config['shout_version']), 
			'shout_print_ver_alt' 		=> sprintf($this->user->lang['SHOUTBOX_VER_ALT'], $this->config['shout_version']),
			'shout_endClassBg' 			=> $this->config['shout_button_background'.$sort_p] ? ' button_background_'.$this->config['shout_color_background'.$sort_p] : '',
			'shout_see_buttons' 		=> ($this->config['shout_see_buttons'] == 1) ? 'true' : 'false',
			'shout_see_buttonsleft' 	=> ($this->config['shout_see_buttons_left'] == 1) ? 'true' : 'false',
			'shout_bar_haute' 			=> ($this->config['shout_bar_option'.$sort_p] == 1) ? 'true' : 'false',
			'shout_sort_pagin' 			=> ($this->config['shout_pagin_option'.$sort_p] == 1) ? 'true' : 'false',
			'shout_shout_defil' 		=> ($this->config['shout_defil'] == 1) ? 'true' : 'false',
			'shout_is_user' 			=> $is_user ? 'true' : 'false',
			'shout_delete_url' 			=> $is_user ? $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'delete'.$sort, 's' => $shout)) : '',
			'shout_edit_url' 			=> $is_user ? $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'edit'.$sort, 's' => $shout)) : '',
			'shout_config_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_configshout'),
			'shout_priv_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_private'),
			'shout_popup_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_popup'),
			'shout_check_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'check'.$sort_p, 's' => $shout)),		
			'shout_view_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'view'.$sort_p, 's' => $shout)),
			'shout_post_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'post'.$sort, 's' => $shout)),
			'shout_smilies_url' 		=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'smilies', 's' => $shout)),
			'shout_smilies_popup_url' 	=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'smilies_popup', 's' => $shout)),
			'shout_online_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'online', 's' => $shout)),
			'shout_rules_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'rules', 's' => $shout)),
			'shout_ubbcode_url' 		=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'user_bbcode', 's' => $shout)), 
			'shout_charge_bbcode_url' 	=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'charge_bbcode', 's' => $shout)),
			'shout_action_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'action_user', 's' => $shout)), 
			'shout_actionPost_url' 		=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'action_post', 's' => $shout)), 
			'shout_actionDel_url' 		=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'action_del', 's' => $shout)),
			'shout_actionDelTo_url' 	=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'action_del_to', 's' => $shout)), 
			'shout_actionRemove_url' 	=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'action_remove', 's' => $shout)),
			'shout_cite_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'cite', 's' => $shout)),
			'shout_actionSound_url' 	=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'action_sound', 's' => $shout)),
			'shout_number_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'number'.$sort_p, 's' => $shout)),
			'shout_purge_url' 			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'purge'.$sort, 's' => $shout)),
			'shout_purge_robot_url' 	=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'purge_robot'.$sort, 's' => $shout)),
			'shout_img_charge'			=> '<img src="' .$this->complete_shout_url(). 'images/run.gif" alt="" style="margin-right:15px;" />',
			'shout_img_load'			=> '<img src="' .$this->complete_shout_url(). 'images/load.gif" alt="" style="margin-right:15px;" />',
			'shout_img_turn'			=> '<img src="' .$this->complete_shout_url(). 'images/loading.gif" alt="" />',
			'shout_creator'				=> 'true',
			'shout_creator_url'			=> $smilecreator_controller,
		));

		// Construct auth vars
		$this->template->assign_vars(array(
			'BZH_ACTIVE_DELETE' 		=> ($this->auth->acl_get('m_shout_delete') || $this->auth->acl_get('u_shout_delete_s') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? 'true' : 'false',
			'BZH_ACTIVE_INFO' 			=> ($this->auth->acl_get('m_shout_info') || $this->auth->acl_get('u_shout_info_s') || $this->auth->acl_get('a_shout_'.$auth_shout)) ? 'true' : 'false',
			'BZH_ACTIVE_PURGE' 			=> ($this->auth->acl_get('m_shout_purge') || $this->auth->acl_get('a_shout_manage')) ? 'true' : 'false',
			'BZH_POST_OK' 				=> ($this->auth->acl_get('u_shout_post') ? 'true' : 'false'),
			'BZH_LIMIT_POST' 			=> ($this->auth->acl_get('u_shout_limit_post') ? 'true' : 'false'),
			'BZH_SMILIES_OK' 			=> ($this->auth->acl_get('u_shout_smilies') ? 'true' : 'false'),
			'BZH_IMAGE_OK' 				=> ($this->auth->acl_get('u_shout_image') ? 'true' : 'false'),
			'BZH_COLOR_OK' 				=> ($this->auth->acl_get('u_shout_color') ? 'true' : 'false'),
			'BZH_BBCODE_OK' 			=> ($this->auth->acl_get('u_shout_bbcode') ? 'true' : 'false'),
			'BZH_CHARS_OK' 				=> ($this->auth->acl_get('u_shout_chars') ? 'true' : 'false'),
			'BZH_POPUP_OK' 				=> ($this->auth->acl_get('u_shout_popup') ? 'true' : 'false'),
			'BZH_FORMAT_OK' 			=> (($this->auth->acl_get('u_shout_bbcode_change') && $is_user) ? 'true' : 'false'),
			'BZH_ONLINE_OK'	 			=> (($this->auth->acl_get('u_viewonline') && $is_user) ? 'true' : 'false'),
			'BZH_PRIV_OK' 				=> (($this->auth->acl_get('u_shout_priv') && $is_user) ? 'true' : 'false'),
		));

		// Conctruct Lang vars
		$this->template->assign_vars(array(
			'SHOUT_CREATOR'					=> $shout_creator,
			'BZH_COMMA_SEPARATOR'			=> $this->user->lang['COMMA_SEPARATOR'],
			'BZH_SHOUT_SEP'					=> $this->user->lang['SHOUT_SEP'],
			'BZH_NO_AJAX'					=> $this->user->lang['NO_AJAX'],
			'BZH_SERVER_ERR'				=> $this->user->lang['SERVER_ERR'],
			'BZH_XML_ER'					=> $this->user->lang['XML_ER'],
			'BZH_JS_ERR'					=> $this->user->lang['JS_ERR'],
			'BZH_LINE'						=> $this->user->lang['LINE'],
			'BZH_FILE'						=> $this->user->lang['FILE'],
			'BZH_LOADING'					=> $this->user->lang['LOADING'],
			'BZH_MSG_DEL_DONE'				=> $this->user->lang['MSG_DEL_DONE'],
			'BZH_NO_MESSAGE'				=> $this->user->lang['NO_MESSAGE'],
			'BZH_SHOUT_PAGE'				=> $this->user->lang['SHOUT_PAGE'],
			'BZH_ONLY_ONE_OPEN'				=> $this->user->lang['ONLY_ONE_OPEN'],
			'BZH_EDIT'						=> $this->user->lang['EDIT'],
			'BZH_SHOUT_EDIT'				=> $this->user->lang['SHOUT_EDIT'],
			'BZH_NO_SHOUT_EDIT'				=> $this->user->lang['NO_SHOUT_EDIT'],
			'BZH_SENDING_EDIT'				=> $this->user->lang['SENDING_EDIT'],
			'BZH_EDIT_DONE'					=> $this->user->lang['EDIT_DONE'],
			'BZH_CANCEL'					=> $this->user->lang['CANCEL'],
			'BZH_NEXT'						=> $this->user->lang['NEXT'],
			'BZH_PREVIOUS'					=> $this->user->lang['PREVIOUS'],
			'BZH_PURGE_PROCESS'				=> $this->user->lang['PURGE_PROCESS'],
			'BZH_MISSING_DIV'				=> $this->user->lang['MISSING_DIV'],
			'BZH_SHOUT_AUTO'				=> $this->user->lang['SHOUT_AUTO'],
			'BZH_SHOUT_EXEMPLE'				=> $this->user->lang['SHOUT_EXEMPLE'],
			'BZH_SHOUT_BBCODE_SUP'			=> $this->user->lang['SHOUT_BBCODE_SUP'],
			'BZH_SHOUT_BBCODE_SUCCESS'		=> $this->user->lang['SHOUT_BBCODE_SUCCESS'],
			'BZH_SHOUT_ACTION_DELETE_EXPLAIN' => $this->user->lang['SHOUT_ACTION_DELETE_EXPLAIN'],
			'BZH_SHOUT_PERSO'				=> $this->user->lang['SHOUT_PERSO'],
			'BZH_SHOUT_DIV_BBCODE_CLOSE'	=> $this->user->lang['SHOUT_DIV_BBCODE_CLOSE'],
			'BZH_SHOUT_ACTION_MSG'			=> $this->user->lang['SHOUT_ACTION_MSG'],
			'BZH_SHOUT_ACTION_MSG_ROBOT'	=> sprintf($this->user->lang['SHOUT_ACTION_MSG_ROBOT'], $robot),
			'BZH_SHOUT_OUT_TIME'			=> $this->user->lang['SHOUT_OUT_TIME'],
			'BZH_SHOUT_DEL'					=> $this->user->lang['SHOUT_DEL'],
			'BZH_DEL_SHOUT'					=> $this->user->lang['DEL_SHOUT'],
			'BZH_NO_SHOUT_DEL'				=> $this->user->lang['NO_SHOUT_DEL'],
			'BZH_SHOUT_POST_IP'				=> $this->user->lang['SHOUT_POST_IP'],
			'BZH_SHOUT_IP'					=> $this->user->lang['SHOUT_IP'],
			'BZH_NO_SHOW_IP_PERM'			=> $this->user->lang['NO_SHOW_IP_PERM'],
			'BZH_NO_SHOUT_BBCODE'			=> $this->user->lang['NO_SHOUT_BBCODE'],
			'BZH_SHOUT_CLICK_SOUND_ON'		=> $this->user->lang['SHOUT_CLICK_SOUND_ON'],
			'BZH_SHOUT_CLICK_SOUND_OFF'		=> $this->user->lang['SHOUT_CLICK_SOUND_OFF'],
			'BZH_MESSAGE_EMPTY'				=> $this->user->lang['MESSAGE_EMPTY'],
			'BZH_SHOUT_CLICK_HERE'			=> $this->user->lang['SHOUT_CLICK_HERE'],
			'BZH_SHOUT_DIV_CLOSE'			=> $this->user->lang['SHOUT_DIV_CLOSE'],
			'BZH_NO_POST_PERM'				=> $this->user->lang['NO_POST_PERM'],
			'BZH_NO_POST_PERM_GUEST'		=> $this->user->lang['NO_POST_PERM_GUEST'],
			'BZH_NO_SHOUT_POP'				=> $this->user->lang['NO_SHOUT_POP'],
			'BZH_POST_MESSAGE'				=> $this->user->lang['POST_MESSAGE'],
			'BZH_POST_MESSAGE_ALT'			=> $this->user->lang['POST_MESSAGE_ALT'],
			'BZH_POSTED'					=> $this->user->lang['POSTED'],
			'BZH_SHOUT_POP'					=> $this->user->lang['SHOUT_POP'],
			'BZH_SHOUT_PRIV'				=> $this->user->lang['SHOUT_PRIV'],
			'BZH_SHOUT_ONLINE'				=> $this->user->lang['SHOUT_ONLINE'],
			'BZH_SHOUT_ONLINE_CLOSE'		=> $this->user->lang['SHOUT_ONLINE_CLOSE'],
			'BZH_SHOUT_COLOR'				=> $this->user->lang['SHOUT_COLOR'],
			'BZH_NO_SHOUT_COLOR'			=> $this->user->lang['NO_SHOUT_COLOR'],
			'BZH_SHOUT_COLOR_CLOSE'			=> $this->user->lang['SHOUT_COLOR_CLOSE'],
			'BZH_SMILIES'					=> $this->user->lang['SMILIES'],
			'BZH_NO_SMILIES'				=> $this->user->lang['NO_SMILIES'],
			'BZH_SMILIES_CLOSE'				=> $this->user->lang['SMILIES_CLOSE'],
			'BZH_SHOUT_CHARS'				=> $this->user->lang['SHOUT_CHARS'],
			'BZH_NO_SHOUT_CHARS'			=> $this->user->lang['NO_SHOUT_CHARS'],
			'BZH_SHOUT_CHARS_CLOSE'			=> $this->user->lang['SHOUT_CHARS_CLOSE'],
			'BZH_SHOUT_RULES'				=> $this->user->lang['SHOUT_RULES'],
			'BZH_SHOUT_RULES_PRIV'			=> $this->user->lang['SHOUT_RULES_PRIV'],
			'BZH_SHOUT_RULES_CLOSE'			=> $this->user->lang['SHOUT_RULES_CLOSE'],
			'BZH_SHOUT_MORE_SMILIES'		=> $this->user->lang['SHOUT_MORE_SMILIES'],
			'BZH_SHOUT_MORE_SMILIES_ALT'	=> $this->user->lang['SHOUT_MORE_SMILIES_ALT'],
			'BZH_SHOUT_CONFIG_OPEN'			=> $this->user->lang['SHOUT_CONFIG_OPEN'],
			'BZH_SHOUT_CHOICE_NAME'			=> $this->user->lang['SHOUT_CHOICE_NAME'],
			'BZH_SHOUT_CHOICE_NAME_ERROR'	=> $this->user->lang['SHOUT_CHOICE_NAME_ERROR'],
			'BZH_SHOUT_TOO_BIG'				=> $this->user->lang['SHOUT_TOO_BIG'],
			'BZH_SHOUT_TOO_BIG2'			=> $this->user->lang['SHOUT_TOO_BIG2'],
			'BZH_SHOUT_PURGE_ROBOT_ALT'		=> $this->user->lang['SHOUT_PURGE_ROBOT_ALT'],
			'BZH_SHOUT_PURGE_ROBOT_BOX'		=> $this->user->lang['SHOUT_PURGE_ROBOT_BOX'],
			'BZH_SHOUT_PURGE_ALT'			=> $this->user->lang['SHOUT_PURGE_ALT'],
			'BZH_SHOUT_PURGE_BOX'			=> $this->user->lang['SHOUT_PURGE_BOX'],
			'BZH_SENDING'					=> $this->user->lang['SENDING'],
			'BZH_SHOUT_ACTION_CITE_ON'		=> $this->user->lang['SHOUT_ACTION_CITE_ON'],
			'BZH_SHOUT_CLOSE'				=> $this->user->lang['SHOUT_CLOSE'],
			'BZH_SHOUT_BBCODES'				=> $this->user->lang['SHOUT_BBCODES'],
			'BZH_SHOUT_BBCODES_CLOSE'		=> $this->user->lang['SHOUT_BBCODES_CLOSE'],
			'BZH_SHOUT_TITLESOUND' 			=> $this->user->lang['SHOUT_CLICK_SOUND_'.(($correct == 0) ? 'OFF' : 'ON')],
			'BZH_SMILIE_CREATOR'			=> $lang_smilie_creator,
		));
	}
}
