<?php
/**
*
* @package Breizh Shoutbox Extension
* @version $Id: main_listener.php 100
* @copyright (c) 2018 Sylver35  https://breizhcode.com
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace sylver35\breizhshoutbox\event;

/**
* @ignore
*/
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
* Event listener
*/
class main_listener implements EventSubscriberInterface
{

	/** @var \phpbb\config\config */
	protected $config;
	
	/* @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\db\driver\driver */
	protected $db;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;
	
	/** @var \phpbb\auth\auth */
	protected $auth;

	/** @var \phpbb\user */
	protected $user;
	
	/** @var \phpbb\log\log */
	protected $log;
	
	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;
	
	/** @var \phpbb\path_helper */
	protected $path_helper;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string phpEx */
	protected $php_ext;
	
	/* @var \sylver35\breizhshoutbox\core\breizhshoutbox */
	protected $shoutbox;
	
	/**
	* Constructor
	*
	*/
	public function __construct(
		\phpbb\config\config $config, 
		\phpbb\controller\helper $helper, 
		\phpbb\db\driver\driver_interface $db, 
		\phpbb\request\request $request, 
		\phpbb\template\template $template, 
		\phpbb\auth\auth $auth, 
		\phpbb\user $user,  
		\phpbb\log\log $log,
		\phpbb\extension\manager $ext_manager,
		\phpbb\path_helper $path_helper,
		$phpbb_root_path, 
		$php_ext,
		\sylver35\breizhshoutbox\core\shoutbox $shoutbox)
	{
		$this->config 		= $config;
		$this->helper 		= $helper;
		$this->db 			= $db;
		$this->request 		= $request;
		$this->template 	= $template;
		$this->auth 		= $auth;
		$this->user 		= $user;
		$this->log			= $log;
		$this->ext_manager	 = $ext_manager;
		$this->path_helper	 = $path_helper;
		$this->root_path 	= $phpbb_root_path;
		$this->php_ext 		= $php_ext;
		$this->shoutbox 	= $shoutbox;
		
		$this->ext_path = $this->ext_manager->get_extension_path('sylver35/breizhshoutbox', true);
		$this->ext_path_web = $this->path_helper->update_web_root_path($this->ext_path);
	}
	
	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup' 							=> 'load_language_on_setup',
			'core.page_header' 							=> 'add_page_header_shout',
			'core.session_create_after'					=> 'charge_post_session_shout',
			'core.index_modify_page_title' 				=> 'charge_shout_display',
			'core.viewforum_modify_page_title' 			=> 'charge_shout_display',
			'core.viewtopic_modify_page_title'			=> 'charge_shout_display',
			'core.index_modify_birthdays_list' 			=> 'charge_shout_birthdays',
			'core.submit_post_end' 						=> 'charge_advert_post',
			'core.user_add_after' 						=> 'charge_add_newest_user',
			'core.display_custom_bbcodes_modify_sql'	=> 'remove_disallowed_bbcodes',
			'core.posting_modify_post_data'				=> 'shout_modify_post_data',
			'core.posting_modify_message_text'			=> 'shout_modify_post_before',
			'core.posting_modify_submit_post_before'	=> 'shout_modify_post_before_data',
			'core.posting_modify_submission_errors'		=> 'shout_submission_post_data',
			'core.posting_modify_template_vars'			=> 'shout_modify_template_vars',
		);
	}

	/**
	 * @param data $event
	 */
	public function load_language_on_setup($event)
	{
		$lang_set_ext = $event['lang_set_ext'];
		$lang_set_ext[] = array(
			'ext_name' => 'sylver35/breizhshoutbox',
			'lang_set' => array('shout', 'info_acp_shoutbox')
		);
		$event['lang_set_ext'] = $lang_set_ext;
	}

	/**
	 * Add urls for shoutbox
	 */
	public function add_page_header_shout()
	{
		$this->template->assign_vars(array(
			'U_SHOUT_PRIV_PAGE'		=> ($this->auth->acl_get('u_shout_priv')) ? $this->helper->route('sylver35_breizhshoutbox_controller_private') : '',
			'U_SHOUT_POPUP'			=> ($this->auth->acl_get('u_shout_popup')) ? $this->helper->route('sylver35_breizhshoutbox_controller_popup') : '',
			'U_SHOUT_LATERAL'		=> ($this->auth->acl_get('u_shout_lateral')) ? $this->helper->route('sylver35_breizhshoutbox_controller_lateral') : '',
			'U_SHOUT_CONFIG_SHOUT'	=> ($this->auth->acl_get('u_shout_post')) ? $this->helper->route('sylver35_breizhshoutbox_controller_configshout') : '',
			'U_SHOUT_AJAX'			=> $this->helper->route('sylver35_breizhshoutbox_controller_ajax', array('m' => 'display_smilies', 's' => 2)),
		));
		$this->shoutbox->shout_panel();
	}
	
	public function charge_shout_display()
	{
		$this->shoutbox->shout_display(2);
	}
	
	/**
	 * @param data $event
	 */
	public function charge_shout_birthdays($event)
	{
		// If it isnt installed or Innactive we cant display it.
		if (isset($this->config['shout_version']) && $this->config['shout_enable'])
		{
			if ($this->config['shout_birthday'] || $this->config['shout_birthday_priv'])
			{
				if ($this->config['shout_enable_robot'] && $this->config['shout_cron_hour'] == date('H'))
				{
					$this->shoutbox->robot_birthday_shout($event);
				}
			}
		}
	}
	
	/**
	 * @param data $event
	 */
	public function charge_post_session_shout($event)
	{
		$this->shoutbox->post_session_shout($event['session_data']);
	}
	
	/**
	 * @param data $event
	 */
	public function charge_advert_post($event)
	{
		$this->shoutbox->advert_post_shoutbox($event);
	}
	
	/**
	 * @param data $event
	 */
	public function charge_add_newest_user($event)
	{
		$this->shoutbox->shout_add_newest_user($event);
	}
	
	/**
	 * @param data $event
	 */
	public function remove_disallowed_bbcodes($event)
	{
		if (!preg_match("#posting#i", $this->user->page['page_name']))
		{
			$event['sql_ary'] = $this->shoutbox->remove_disallowed_bbcodes($event['sql_ary']);
		}
	}
	
	/**
	 * @param data $event
	 */
	public function shout_modify_post_data($event)
	{
		$event['post_data'] = array_merge($event['post_data'], array(
			'hide_robot'	=> false,
		));
	}
	
	/**
	 * @param data $event
	 */
	public function shout_modify_post_before_data($event)
	{
		$event['data'] = array_merge($event['data'], array(
			'hide_robot'	=> $event['post_data']['hide_robot'],
		));
	}
	
	/**
	 * @param data $event
	 */
	public function shout_modify_post_before($event)
	{
		$event['post_data'] = array_merge($event['post_data'], array(
			'hide_robot'	=> $this->request->variable('hide_robot', false),
		));
	}
	
	/**
	 * @param data $event
	 */
	public function shout_submission_post_data($event)
	{
		$event['post_data'] = array_merge($event['post_data'], array(
			'hide_robot'	=> $this->request->variable('hide_robot', false),
		));
	}
	
	/**
	 * @param data $event
	 */
	public function shout_modify_template_vars($event)
	{
		$s_hide_robot = true;
		$shout_hide_allowed = false;
		$exclude_forums = array();
		if ($this->config['shout_exclude_forums'])
		{
			$exclude_forums = explode(',', $this->config['shout_exclude_forums']);
			if (in_array($event['forum_id'], $exclude_forums))
			{
				$s_hide_robot = false;
			}
		}
		if ($this->auth->acl_get('u_shout_hide') && $s_hide_robot)
		{
			if ($event['mode'] == 'edit')
			{
				$shout_hide_allowed = ($this->config['shout_edit_robot'] || $this->config['shout_edit_robot_priv']) ? true : false;
			}
			else
			{
				$shout_hide_allowed = true;
			}
		}
		
		$event['page_data'] = array_merge($event['page_data'], array(
			'S_SHOUT_HIDE_CHECKED'		=> ($event['post_data']['hide_robot']) ? ' checked="checked"' : '',
			'S_SHOUT_HIDE_ALLOWED'		=> $shout_hide_allowed,
		));
	}
}
